#!/usr/bin/perl

# Utilisable dans un cron mensuel

use strict;
use locale;
use FDN::Adsl::Basic;
use FDN::Adsl::Factu;
use FDN::Adsl::Wf;
use FDN::Common::Logger;

use POSIX qw(locale_h strftime);

my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();

# Note: c'est le mois precedent pour lequel on fait un prelevement
if ($mon == 0) {
	# Janvier, passer a l'annee precedente
	$mon = 12;
	$year = $year - 1;
}
$mon -= 1;

setlocale(LC_ALL, "fr_FR");
my $nom = strftime("FDN %B %Y", 0, 0, 0, 1, $mon, $year);

# On preleve le mois en cours
$mon += 1;

if ($mon == 12) {
	# Janvier, repasser de nouveau a l'annee suivante
	$mon = 0;
	$year = $year + 1;
}

my $mois = sprintf "%02u", $mon + 1;
my $annee = sprintf "%u", $year + 1900;
# pour l'instant le 10
my $jour = "10";

my $date = strftime("%A %d %B %Y", 0, 0, 0, $jour, $mon, $year);

my $erreurs = "";

my @prl_en_cours = prlbq_par_etat('du');
for my $prl ( @prl_en_cours ) {
	$erreurs .= "Le prelevement\nhttps://vador.fdn.fr/adsl/prlbq/view?pid=$prl->{PRLBQ_ID}&do=yes\nn'a pas encore ete envoye\n\n";
}
my @prl_en_cours = prlbq_par_etat('envoye');

for my $prl ( @prl_en_cours ) {
	$erreurs .= "Le prelevement\nhttps://vador.fdn.fr/adsl/prlbq/view?pid=$prl->{PRLBQ_ID}&do=yes\nn'a pas encore ete encaisse\n\n";
}

if ($erreurs) {
	debug $erreurs;
	envoie_mail('tresorier@fdn.fr', "prelevement-err", {
			'erreurs' => $erreurs,
			'nom' => $nom,
		});
	exit 1;
}

my $pid = creation_prelevement(
		$nom,
		$annee, $mois,
		"prl-$annee-$mois-$jour.xml",
		"$annee-$mois-$jour");

envoie_mail('tresorier@fdn.fr', "prelevement-ok", {
		'nom' => $nom,
		'jour' => $jour,
		'mois' => $mois,
		'date' => $date,
		'annee' => $annee,
		'pid' => $pid,
	});

envoie_mail('ag@fdn.fr', "prelevement-ag", {
		'date' => $date,
		'nom' => $nom,
		'pid' => $pid,
	});

debug "j'ai fini de creer le prelevement $nom prevu le $date, $annee $mois $jour : $pid"
