package FDN::Adsl::Infos;

use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		toutes_infos_client
		toutes_infos_ligne
		toutes_infos_compta
		toutes_infos_vpn
		new_info_client
		new_info_ligne
		new_info_compta
		new_info_new
		new_info_vpn
		store_info_image
                );
our @EXPORT_OK = qw ();

use FDN::Common::Config;
use FDN::Common::Logger;
use FDN::Images::Jpeg;
use FDN::Adsl::Basic;

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

sub toutes_infos_ligne($) { # Test
	my ( $lid ) = @_;
	my @lis = ligneinfo_par_ligne_id($lid);
	__debug "Infos pour la ligne $lid.";
	return __toutes_infos(@lis);
}

sub toutes_infos_client($) { # Test
	my ( $cid ) = @_;
	my @cis = clientinfo_par_client_id($cid);
	__debug "Infos pour le.la client.e $cid.";
	return __toutes_infos(@cis);
}

sub toutes_infos_compta($) { # Test
	my ( $cid ) = @_;
	my @cis = comptainfo_par_compta_id($cid);
	__debug "Infos pour le compta $cid.";
	return __toutes_infos(@cis);
}

sub toutes_infos_vpn($) { # Test
	my ( $vid ) = @_;
	my @vis = vpninfo_par_vpn_id($vid);
	__debug "Infos pour le vpn $vid.";
	return __toutes_infos(@vis);
}

sub __toutes_infos(@) { # Test
	my @infos = @_;
	my @res;
	my $info;
	for my $i ( @infos ) {
		$info = info_par_id($i->{'INFO_ID'});
		if ( defined $info ) {
			__debug "Ajout infos ".$i->{'INFO_ID'}.".";
			push @res, $info;
		}
	}
	return @res;
}

sub new_info_vpn($$$$) {
	my ( $vpnid, $type, $titre, $info ) = @_;
	my $iid = new_info($type, $titre, $info);
	__debug "Creation de l'info $iid pour le vpn $vpnid.";
	new_vpninfo($vpnid,$iid);
	return $iid;
}

sub new_info_client($$$$) { # Test
	my ( $cid, $type,$titre, $info ) = @_;
	my $iid = new_info($type,$titre, $info);
	__debug "Creation de l'info $iid pour le.la client.e $cid.";
	new_clientinfo($cid,$iid);
	return $iid;
}

sub new_info_ligne($$$$) { # Test
	my ( $lid, $type, $titre, $info ) = @_;
	my $iid = new_info($type, $titre, $info);
	__debug "Creation de l'info $iid pour la ligne $lid.";
	new_ligneinfo($lid,$iid);
	return $iid;
}

sub new_info_compta($$$$) { # Test
	my ( $cid, $type, $titre, $info ) = @_;
	my $iid = new_info($type, $titre, $info);
	__debug "Creation de l'info $iid pour le compta $iid.";
	new_comptainfo($cid,$iid);
	return $iid;
}

sub new_info_new($) {
	my ( $src ) = @_;
	my $iid = new_info('fichier', 'New', 'no yet');
	__debug "Creation de l'info $iid en rattachement nulle part";
	new_newinfo($src, $iid);
	return $iid;
}

sub store_info_image($) {
	my ( $image ) = @_;

	my $target_dir = Config_Key('adsl/global.cfg','dirs','imgdir');
	if ( ! -d $target_dir ) {
		die "Je n'ai pas mon repertoire cible...";
	}
	if ( ! -w "$target_dir/jpeg" ) {
		die "Je ne peux pas ecrire dans le repertoire des images";
	}
	if ( ! -w "$target_dir/thumbnails" ) {
		die "Je ne peux pas ecrire dans le repertoire des vignettes";
	}

	if ( ! -d '/tmp/info_store' ) {
		mkdir '/tmp/info_store';
	}

	my $tmpdir = "/tmp/info_store/$$";
	if ( -e $tmpdir ) {
		die "Mon repertoire temporaire existe deja...";
	}
	mkdir $tmpdir;
	if ( ! -d $tmpdir ) {
		die "Je n'arrive pas a crer ma zone de jeu...";
	}

	if ( ! -f $image ) {
		die "File not found: $image\n";
	}
	__debug "Je convertis $image en jpeg";
	if ( ! force_jpeg($image, "$tmpdir/image.jpg") ) {
		die "Failed to convert $image to $tmpdir/image.jpg";
	}
	__debug "Je fais un thumbnail";
	if ( ! resize_jpeg("$tmpdir/image.jpg", "$tmpdir/tn.jpg", "210x297") ) {
		die "Failed to make thumbnail to $tmpdir/tn.jpg";
	}

	my $num = new_info_new('mail');
	__debug "Je range l'image et le thumbnail";
	`mv $tmpdir/image.jpg $target_dir/jpeg/img$num.jpg`;
	`mv $tmpdir/tn.jpg $target_dir/thumbnails/tn-img$num.jpg`;
	set_info_val($num, "img$num");

	rmdir $tmpdir or error "Failed to remove $tmpdir. Should be empty...";
	rmdir '/tmp/info_store';
}

1;

