package FDN::Adsl::New;

use strict;
use Exporter;
our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	renseigne_adresse
	modifie_adresse
	form_adresse

	renseigne_info
	modife_info
	form_info

	renseigne_client
	modifie_client
	form_client

	renseigne_creance
	modifie_creance
	form_creance

	renseigne_ligne
	modifie_ligne
	form_ligne

	renseigne_don
	modifie_don
	form_don

	renseigne_vpn
	modifie_vpn
	form_vpn

	renseigne_gattr
	modifie_gattr
	form_gattr

	renseigne_uattr
	modifie_uattr
	form_uattr

	renseigne_ipaddr
	modifie_ipaddr
	form_ipaddr

	renseigne_raduser
	modifie_raduser
	form_raduser

	renseigne_paiement
	modifie_paiement
	form_paiement

	renseigne_payant
	modifie_payant
	form_payant
	form_nadh

	renseigne_prelevemt
	modifie_prelevemt
	form_prelevemt

	renseigne_prix
	modifie_prix
	form_prix

	renseigne_compta
	modifie_compta
	form_compta

	renseigne_adhaccess
	modifie_adhaccess
	form_adhaccess

	renseigne_site
	modifie_site
	form_site

	renseigne_domaine
	modifie_domaine
	form_domaine

	renseigne_unixuser
	modifie_unixuser
	form_unixuser

	renseigne_unixgroup
	modifie_unixgroup
	form_unixgroup

	renseigne_validation
	modifie_validation
	form_validation
);
our @EXPORT_OK = qw (
);
our %EXPORT_TAGS = qw (
);

use Clone qw(clone);
use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Web::Forms;
use FDN::Sql::New;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;
use FDN::Adsl::DBC;
use FDN::Adsl::Radius;
use FDN::Adsl::Ag;

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

sub __do_form {
	my ( $tbl, $vars ) = @_;
	reset_form_errors();
	return __form($tbl, $vars);	
}

my $adresse_tbl = {
	'PAR_ID' => \&adresse_par_id,
	'MODIF'  => {
		'nomadr' => [ 'ADRESSE_NOM'  , \&set_adresse_nom   ], # Tests
		'adr'    => [ 'ADRESSE_ADR'  , \&set_adresse_adr   ], # Tests
		'cp'     => [ 'ADRESSE_CP'   , \&set_adresse_cp    ], # Tests
		'ville'  => [ 'ADRESSE_VILLE', \&set_adresse_ville ], # Tests
		'email'  => [ 'ADRESSE_MAIL',  \&set_adresse_mail  ], # Tests
		'pgp'    => [ 'ADRESSE_PGP',   \&set_adresse_pgp   ], # Tests
		'tel'    => [ 'ADRESSE_TEL',   \&set_adresse_tel   ], # Tests
		'fax'    => [ 'ADRESSE_FAX',   \&set_adresse_fax   ]  # Tests
		},
	'CREATE' => \&new_adresse, # Tests
	'NEW' => {
		'nomadr' => 0, # Tests
		'adr' => 1,    # Tests
		'cp' => 2,     # Tests
		'ville' => 3,  # Tests
		'email' => \&set_adresse_mail,
		'pgp'   => \&set_adresse_pgp,
		'tel'   => \&set_adresse_tel, # Tests
		'fax'   => \&set_adresse_fax,
		'cid' => [ \&set_client_adresse ],
		'lid' => [ \&set_ligne_adresse ],
		'pid' => [ \&set_prelevemt_adresse ]
	},
	'FORM' => 'nadresse',
	'HIDDEN' => [
		[ 'id',  'hdo', 'hid'  ],
		[ 'lid', 'hdo', 'hlid' ],
		[ 'cid', 'hdo', 'hcid' ],
		[ 'pid', 'hdo', 'hpid' ],
	],
	'HTMLIDS' => {}
};
sub renseigne_adresse($$) { # Tests
	my ( $vars, $aid ) = @_;
	return __renseigne($adresse_tbl, $vars, $aid);
}

sub modifie_adresse($$) { # Tests
	my ( $vars, $aid ) = @_;
	my $client = client_par_adresse_id($aid);

	conduse('FDN::Adsl::Ldap');

	if ( defined($client) ) {
		desabonne_liste_ag($client->{CLIENT_ID});
	}
	my $ret =  __modifie($adresse_tbl, $vars, $aid);
	if ( defined($client) ) {
		FDN::Adsl::Ldap::ldap_update_client($client->{CLIENT_ID});
		abonne_liste_ag($client->{CLIENT_ID});
	}
	return $ret; 
}

sub form_adresse($) {
	my ( $vars ) = @_;
	return __do_form($adresse_tbl, $vars);	
}

my $don_tbl = {
	'PAR_ID' => \&don_par_id,
	'MODIF' => {
		'state'   => [ 'DON_STATE',   \&set_don_state ],
		'type'    => [ 'DON_TYPE',    \&set_don_type ],
		'montant' => [ 'DON_MONTANT', \&set_don_montant ],
		'libelle' => [ 'DON_LIBELLE', \&set_don_libelle ],
		'date'    => [ 'DON_DATE',    \&set_don_date ],
		'debut'   => [ 'DON_DEBUT',   \&set_don_debut ],
		'fin'     => [ 'DON_FIN',     \&set_don_fin ],
		'client'  => [ 'CLIENT_ID',   \&set_don_client_id ]
	},
	'CREATE' => \&new_don,
	'NEW' => {
		'client'  => 0,
		'state'   => 1,
		'type'    => 2,
		'libelle' => 3,
		'montant' => 4,
		'date'    => \&set_don_date,
		'debut'   => \&set_don_debut,
		'fin'     => \&set_don_fin
	},
	'FORM' => 'ndon',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
#	'HTMLIDS' => {}
};

sub renseigne_don($$) {
	my ( $vars, $did ) = @_;
	return __renseigne($don_tbl,$vars,$did);
}

sub modifie_don($$) {
	my ( $vars, $did ) = @_;
	return __modifie($don_tbl,$vars,$did);
}

sub form_don($) {
	my ( $vars ) = @_;
	my $form = __do_form($don_tbl, $vars);
	ajoute_liste_clients($form);
	return $form;
}

my $vpn_tbl = {
	'PAR_ID' => \&vpn_par_id,
	'MODIF' => {
		'prix'   => [ 'PRIX_ID',     \&set_vpn_prix_id ],
		'client' => [ 'CLIENT_ID',   \&set_vpn_client_id ],
		'state'  => [ 'VPN_STATE',   \&set_vpn_state ],
		'dtin'   => [ 'VPN_DTIN',    \&set_vpn_dtin ],
		'dtfac'  => [ 'VPN_DTFAC',   \&set_vpn_dtfac ],
		'dtout'  => [ 'VPN_DTOUT',   \&set_vpn_dtout ]
	},
	'CREATE' => \&new_vpn,
	'NEW' => {
		'prix' => 0,
		'client' => 1,
		'-1' => 2,
		'state' => \&set_vpn_state,
		'dtin' => \&set_vpn_dtin,
		'dtfac' => \&set_vpn_dtfac,
		'dtout' => \&set_vpn_dtout
	},
	'FORM' => 'nvpn',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {
		'ADRESSE_ID' => 'ladresse'
	}
};

sub renseigne_vpn($$) { # Test
	my ( $vars, $vid ) = @_;
	return __renseigne($vpn_tbl,$vars,$vid);
}

sub modifie_vpn($$) { # Test
	my ( $vars, $vid ) = @_;
	return __modifie($vpn_tbl,$vars,$vid);
}

sub form_vpn($) {
	my ( $vars ) = @_;
	my $form = __do_form($vpn_tbl, $vars);
	ajoute_liste_prix($form);
	ajoute_liste_clients($form);
	return $form;
}

my $ligne_tbl = {
	'PAR_ID' => \&ligne_par_id,
	'MODIF' => {
		'debit'  => [ 'DEBIT_ID',    \&set_ligne_debit  ], # Test
		'prix'   => [ 'PRIX_ID',     \&set_ligne_prix   ], # Test
		'client' => [ 'CLIENT_ID',   \&set_ligne_client ], # Test
		'tel'    => [ 'LIGNE_TEL',   \&set_ligne_tel    ], # Test
		'option' => [ 'LIGNE_OPT',   \&set_ligne_option ], # Test
		'state'  => [ 'LIGNE_STATE', \&set_ligne_state  ], # Test
		'idld'   => [ 'LIGNE_IDLD',  \&set_ligne_idld   ], # Test
		'dtin'   => [ 'LIGNE_DTIN',  \&set_ligne_dtin   ],
		'dtfac'  => [ 'LIGNE_DTFAC', \&set_ligne_dtfac  ],
		'dtok'   => [ 'LIGNE_DTOK',  \&set_ligne_dtok   ],
		'dtout'  => [ 'LIGNE_DTOUT', \&set_ligne_dtout  ]
	},
	'CREATE' => \&new_ligne, # Test
	'NEW' => {
		'tel' => 0,
		'debit' => 1,
		'option' => 2,
		'prix' => 3,
		'client' => 4,
		-1 => 5,
		'state' => \&set_ligne_state, # Test
		'idld' => \&set_ligne_idld, # Test
		'dtin' => \&set_ligne_dtin,
		'dtfac' => \&set_ligne_dtfac,
		'dtok' => \&set_ligne_dtok,
		'dtout' => \&set_ligne_dtout,
	},
	'FORM' => 'nligne',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {
		'ADRESSE_ID' => 'ladresse'
	}
};

sub renseigne_ligne($$) { # Test
	my ( $vars, $lid ) = @_;
	return __renseigne($ligne_tbl,$vars,$lid);
}

sub modifie_ligne($$) { # Test
	my ( $vars, $lid ) = @_;
	return __modifie($ligne_tbl,$vars,$lid);
}

sub form_ligne($) {
	my ( $vars ) = @_;
	my $form = __do_form($ligne_tbl, $vars);
	ajoute_liste_debits($form);
	ajoute_liste_prix($form);
	ajoute_liste_clients($form);
	return $form;
}

my $client_tbl = {
	'PAR_ID' => \&client_par_id,
	'MODIF' => {
		'nomclient' => [ 'CLIENT_NOM',    \&set_client_nom    ], # Test
		'numadh'    => [ 'CLIENT_NUMADH', \&set_client_numadh ], # Test
		'type'      => [ 'CLIENT_TYPE',   \&set_client_type   ], # Test
		'etat'      => [ 'CLIENT_ETAT',   \&set_client_etat   ], # Test
	},
	'CREATE' => \&new_client, # Test
	'NEW' => {	
		'nomclient' => 0,
		'type' => 1,
		'numadh' => \&set_client_numadh, # Test
		'etat' => \&set_client_etat      # Test
	},
	'FORM' => 'nclient',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {	
		'ADRESSE_ID' => 'adresse',
		'PRELEVEMT_ID' => 'prelevemt'
	}
};

sub renseigne_client($$) { # Test
	my ( $vars, $cid ) = @_;
	return __renseigne($client_tbl,$vars,$cid);
}

sub modifie_client($$) { # Test
	my ( $vars, $cid ) = @_;
	my $ret = __modifie($client_tbl,$vars,$cid);

	conduse('FDN::Adsl::Ldap');
	FDN::Adsl::Ldap::ldap_update_client($cid);

	return $ret;
}

sub form_client($) {
	my ( $vars ) = @_;
	return __do_form($client_tbl, $vars);
}

my $creance_tbl = {
	'PAR_ID' => \&creance_par_id, # Test
	'MODIF' => {
		'ht'      => [ 'CREANCE_HT',      \&set_creance_ht ], # Test
		'ttc'     => [ 'CREANCE_TTC',     \&set_creance_ttc ], # Test
		'tva'     => [ 'CREANCE_TVA',     \&set_creance_tva ], # Test
		'libelle' => [ 'CREANCE_LIBELLE', \&set_creance_libelle ], # Test
		'datedue' => [ 'CREANCE_DATEDUE', \&set_creance_datedue ], # Test
		'etat'    => [ 'CREANCE_ETAT',    \&set_creance_etat ], # Test
		'client'  => [ 'CLIENT_ID',       \&set_creance_client  ], # Test
	},
	'CREATE' => \&new_creance, # Test
	'NEW' => {
		'client' => 0,
		-1 => 1,
		-2 => 2,
		'ht' => 3,
		'ttc' => 4,
		'tva' => 5,
		'libelle' => 6,
		'datedue' => 7, # Test
	},
	'FORM' => 'ncreance',
	'HIDDEN' => [
		[ 'id', 'hdo',     'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_creance($$) { # Test
	my ( $vars, $cid ) = @_;
	return __renseigne($creance_tbl, $vars, $cid);
}

sub modifie_creance($$) { # Test
	my ( $vars, $cid ) = @_;
	return __modifie($creance_tbl, $vars, $cid);
}

sub form_creance($) {
	my ( $vars ) = @_;
	my $form = __do_form($creance_tbl, $vars);
	ajoute_liste_clients($form);
	return $form;
}

my $gattr_tbl = {
	'PAR_ID' => \&gattr_par_id, # Test
	'MODIF' => {	
		'type' => [ 'GATTR_TYPE',  \&set_gattr_type        ], # Test
		'gid'  => [ 'RADGROUP_ID', \&set_gattr_radgroup_id ], # Test
		'attr' => [ 'GATTR_ATTR',  \&set_gattr_attr        ], # Test
		'val'  => [ 'GATTR_VALUE', \&set_gattr_value       ], # Test
		'op'   => [ 'GATTR_OP',    \&set_gattr_op          ], # Test
	},
	'CREATE' => \&new_gattr, # Test
	'NEW' => {
		'gid'  => 0,
		'type' => 1,
		'attr' => 2,
		'op'   => 3,
		'val'  => 4
	},
	'FORM' => 'ngattr',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_gattr($$) { # Test
	my ( $vars, $gid ) = @_;
	return __renseigne($gattr_tbl, $vars, $gid);
}

sub modifie_gattr($$) { # Test
	my ( $vars, $gid ) = @_;
	return __modifie($gattr_tbl, $vars, $gid);
}

sub form_gattr($) {
	my ( $vars ) = @_;
	return __do_form($gattr_tbl, $vars);
}

my $uattr_tbl = {
	'PAR_ID' => \&uattr_par_id, # Test
	'MODIF' => {
		'type' => [ 'UATTR_TYPE',  \&set_uattr_type       ], # Test
		'uid'  => [ 'RADUSER_ID',  \&set_uattr_raduser_id ], # Test
		'attr' => [ 'UATTR_ATTR',  \&set_uattr_attr       ], # Test
		'val'  => [ 'UATTR_VALUE', \&set_uattr_value      ], # Test
		'op'   => [ 'UATTR_OP',    \&set_uattr_op         ], # Test
	},
	'CREATE' => \&new_uattr, # Test
	'NEW' => {
		'uid'  => 0,
		'type' => 1,
		'attr' => 2,
		'op'   => 3,
		'val'  => 4
	},
	'FORM' => 'nuattr',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_uattr($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($uattr_tbl, $vars, $id);
}

sub modifie_uattr($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($uattr_tbl, $vars, $id);
}

sub form_uattr($) {
	my ( $vars ) = @_;
	return __do_form($uattr_tbl, $vars);
}

my $ipaddr_tbl = {
	'PAR_ID' => \&ipaddr_par_id, # Test
	'MODIF' => {
		'uid'   => [ 'RADUSER_ID',   \&set_ipaddr_raduser_id ], # Test
		'state' => [ 'IPADDR_ETAT',  \&set_ipaddr_etat ], # Test
	},
	'FORM' => 'mipaddr',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_ipaddr($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($ipaddr_tbl, $vars, $id);
}

sub modifie_ipaddr($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($ipaddr_tbl, $vars, $id);
}

sub form_ipaddr($) {
	my ( $vars ) = @_;
	return __do_form($ipaddr_tbl, $vars);
}

my $raduser_tbl = {
	'PAR_ID' => \&raduser_par_id, # Test
	'MODIF' => {
		'login' => [ 'RADUSER_LOGIN', \&set_raduser_login ], # Test
		'gid'   => [ 'RADGROUP_ID', \&set_raduser_radgroup_id ], # Test
		'lid'   => [ 'LIGNE_ID', \&set_raduser_ligne_id ], # Test
		'vid'   => [ 'VPN_ID', \&set_raduser_vpn_id ], # Test
		'adhid' => [ 'ADHACCESS_ID', \&set_raduser_adhaccess_id ], # Test
	},
	'FORM' => 'mraduser',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_raduser($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($raduser_tbl, $vars, $id);
}

sub modifie_raduser($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($raduser_tbl, $vars, $id);
}

sub form_raduser($) {
	my ( $vars ) = @_;
	return __do_form($raduser_tbl, $vars);
}

sub create_paiement($$$$$);
sub change_paiement_etat($$);
my $paiement_tbl = {
	'PAR_ID' => \&paiement_par_id, # Test
	'MODIF' => {
		'ttc'     => [ 'PAIEMENT_TTC',     \&set_paiement_ttc       ], # Test
		'type'    => [ 'PAIEMENT_TYPE',    \&set_paiement_type      ], # Test
		'numero'  => [ 'PAIEMENT_NUM',     \&set_paiement_num       ], # Test
		'client'  => [ 'CLIENT_ID',        \&set_paiement_client_id ], # Test
		'libelle' => [ 'PAIEMENT_LIBELLE', \&set_paiement_libelle   ], # Test
		'date'    => [ 'PAIEMENT_DATE',    \&set_paiement_date      ], # Test
		'etat'    => [ 'PAIEMENT_ETAT',    \&change_paiement_etat   ], # Test
	},
	'CREATE' => \&create_paiement, # Test
	'NEW' => {
		'client'  => 0,
		'libelle' => 1,
		'numero'  => 2,
		'ttc'     => 3,
		'type'    => 4,
		'date'    => \&set_paiement_date, # Test
		'etat'    => \&change_paiement_etat, # Test
	},
	'FORM' => 'npaiement',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ],
		[ 'id', 'paiement', 'date' ]
	],
	'HTMLIDS' => {}
};

sub create_paiement($$$$$) {
	conduse('FDN::Adsl::Factu');
	return encaisser_paiement(@_);
}

sub change_paiement_etat($$) {
	conduse('FDN::Adsl::Factu');
	return changer_etat_paiement(@_);
}

sub renseigne_paiement($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($paiement_tbl, $vars, $id);
}

sub modifie_paiement($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($paiement_tbl, $vars, $id);
}

sub form_paiement($) {
	my ( $vars ) = @_;
	my $form = __do_form($paiement_tbl, $vars );
	ajoute_liste_clients($form);
	return $form;
}

my $payant_tbl = {
	'PAR_ID' => \&payant_par_id, # Test
	'MODIF' => {
		'libelle' => [ 'PAYANT_LIBELLE', \&set_payant_libelle   ], # Test
		'type'    => [ 'PAYANT_TYPE',    \&set_payant_type      ], # Test
		'client'  => [ 'CLIENT_ID',      \&set_payant_client_id ], # Test
		'prix'    => [ 'PRIX_ID',        \&set_payant_prix_id   ], # Test
		'date'    => [ 'PAYANT_DATE',    \&set_payant_date      ], # Test
		'debut'   => [ 'PAYANT_DEBUT',   \&set_payant_debut     ], # Test
		'fin'     => [ 'PAYANT_FIN',     \&set_payant_fin       ], # Test
	},
	'CREATE' => \&new_payant, # Test
	'NEW' => {
		'type' => 0,
		'libelle' => 1,
		'date' => 2,
		'debut' => 3,
		'fin' => 4,
		'prix' => 5,
		'client' => 6
	},
	'FORM' => 'npayant',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_payant($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($payant_tbl, $vars, $id);
}

sub modifie_payant($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($payant_tbl, $vars, $id);
}

sub form_payant($) {
	my ( $vars ) = @_;
	my $form = __do_form($payant_tbl, $vars);
	ajoute_liste_clients($form);
	ajoute_liste_prix($form);
	return $form;
}

my $adh_tbl = clone($payant_tbl);
$adh_tbl->{'FORM'} = 'nadh';

sub form_nadh($) {
	my ( $vars ) = @_;
	my $form = __do_form($adh_tbl, $vars);
	ajoute_liste_clients($form);
	ajoute_liste_prix($form);
	return $form;
}

my $prelevemt_tbl = {
	'PAR_ID' => \&prelevemt_par_id, # Test
	'MODIF' => {
		'banque'    => [ 'PRELEVEMT_BANQUE',    \&set_prelevemt_banque    ], # Test
		'guichet'   => [ 'PRELEVEMT_GUICHET',   \&set_prelevemt_guichet   ], # Test
		'compte'    => [ 'PRELEVEMT_COMPTE',    \&set_prelevemt_compte    ], # Test
		'rib'       => [ 'PRELEVEMT_RIB',       \&set_prelevemt_rib       ], # Test
		'bic'       => [ 'PRELEVEMT_BIC',       \&set_prelevemt_bic       ], # Test
		'iban'      => [ 'PRELEVEMT_IBAN',      \&set_prelevemt_iban      ], # Test
		'etat'      => [ 'PRELEVEMT_ETAT',      \&set_prelevemt_etat      ], # Test
		'titulaire' => [ 'PRELEVEMT_TITULAIRE', \&set_prelevemt_titulaire ], # Test
		'domicil'   => [ 'PRELEVEMT_DOMICIL',   \&set_prelevemt_domicil   ], # Test
		'maxprel'   => [ 'PRELEVEMT_MAXPREL',   \&set_prelevemt_maxprel   ], # Test
		'mandat'    => [ 'PRELEVEMT_MANDAT',    \&set_prelevemt_mandat    ], # Test
		'datemandat' => [ 'PRELEVEMT_DATEMANDAT', \&set_prelevemt_datemandat ], # Test
		'first'     => [ 'PRELEVEMT_FIRST',     \&set_prelevemt_first     ], # Test
		'cid'       => [ 'CLIENT_ID',		\&set_prelevemt_client_id ], # Test
	},
	'CREATE' => \&new_prelevemt, # Test
	'NEW' => {
		'cid'       => 0,
		'banque'    => 1,
		'guichet'   => 2,
		'compte'    => 3,
		'rib'       => 4,
		'titulaire' => 5,
		'domicil'   => 6,
		# 'cid' => [ \&set_client_prelevemt ], # Test
		'etat' => \&set_prelevemt_etat, # Test
		'maxprel' => \&set_prelevemt_maxprel, # Test
		'bic' => \&set_prelevemt_bic, # Test
		'iban' => \&set_prelevemt_iban, # Test
		'first' => \&set_prelevemt_first, # Test
	},
	'FORM' => 'nprelevemt',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ],
		[ 'id', 'nprelevemt', 'preletat' ],
		[ 'cid', 'hdo', 'hcid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_prelevemt($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($prelevemt_tbl, $vars, $id);
}

sub modifie_prelevemt($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($prelevemt_tbl, $vars, $id);
}

sub form_prelevemt($) {
	my ( $vars ) = @_;
	return __do_form($prelevemt_tbl, $vars);
}

my $prix_tbl = {
	'PAR_ID' => \&prix_par_id, # Test
	'MODIF' => {
		'nomtarif'   => [ 'PRIX_NOM',    \&set_prix_nom    ], # Test
		'prixfasht'  => [ 'PRIX_FASHT',  \&set_prix_fasht  ], # Test
		'prixfasttc' => [ 'PRIX_FASTTC', \&set_prix_fasttc ], # Test
		'prixaboht'  => [ 'PRIX_ABOHT',  \&set_prix_aboht  ], # Test
		'prixabottc' => [ 'PRIX_ABOTTC', \&set_prix_abottc ], # Test
		'tva'        => [ 'PRIX_TVA',    \&set_prix_tva    ], # Test
	},
	'CREATE' => \&new_prix, # Test
	'NEW' => {
		'nomtarif'   => 0,
		'prixfasht'  => 1,
		'prixaboht'  => 2,
		'prixfasttc' => 3,
		'prixabottc' => 4,
		'tva'        => 5
	},
	'FORM' => 'nprix',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_prix($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($prix_tbl, $vars, $id);
}

sub modifie_prix($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($prix_tbl, $vars, $id);
}

sub form_prix($) {
	my ( $vars ) = @_;
	return __do_form($prix_tbl, $vars);
}

my $compta_tbl = {
	'PAR_ID' => \&compta_par_id, # Test
	'MODIF' => {
		'nomcompta' => [ 'COMPTA_NOM', \&set_compta_nom ], # Test
		'etatcompta' => [ 'COMPTA_ETAT', \&set_compta_etat ], # Test
		'assocompta' => [ 'COMPTA_ASSO', \&set_compta_asso ], # Test
	},
	'CREATE' => \&new_compta, # Test
	'NEW' => {
		'nomcompta' => 0,
		'etatcompta' => 1,
		'assocompta' => 2
	},
	'FORM' => 'ncompta',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_compta($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($compta_tbl, $vars, $id);
}

sub modifie_compta($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($compta_tbl, $vars, $id);
}

sub form_compta($) {
	my ( $vars ) = @_;
	return __do_form($compta_tbl, $vars );
}

my $adhaccess_tbl = {
	'PAR_ID' => \&adhaccess_par_id, # Test
	'MODIF' => {
		'login'  => [ 'ADHACCESS_LOGIN',  \&update_adhaccess_login     ], # Test
		'passwd' => [ 'ADHACCESS_PASSWD', \&update_adhaccess_passwd    ], # Test
		'client' => [ 'CLIENT_ID',        \&set_adhaccess_client_id ], # Test
	},
	'CREATE' => \&new_adhaccess, # Test
	'NEW' => {
		'client' => 0,
		'login'  => 1,
		'passwd' => 2
	},
	'FORM' => 'nadhaccess',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_adhaccess($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($adhaccess_tbl, $vars, $id);
}

sub modifie_adhaccess($$) { # Test
	my ( $vars, $id ) = @_;
	my $old_login;
	my $new_login;

	my $adhacc = adhaccess_par_id($id);
	if ($adhacc) {
		$old_login = $adhacc->{ADHACCESS_LOGIN};
	}

	my $ret = __modifie($adhaccess_tbl, $vars, $id);

	conduse('FDN::Adsl::Ldap');

	my $new_adhacc = adhaccess_par_id($id);
	if ($new_adhacc) {
		$new_login = $new_adhacc->{ADHACCESS_LOGIN};
	}

	if ($old_login && $new_login && $vars->{login} &&
		$old_login ne $vars->{login} && $new_login eq $vars->{login}) {
		FDN::Adsl::Ldap::ldap_rename_login($old_login, $vars->{login});
	}

	FDN::Adsl::Ldap::ldap_update_client($vars->{client});

	return $ret;
}

sub form_adhaccess($) {
	my ( $vars ) = @_;
	my $form = __do_form($adhaccess_tbl, $vars);
	ajoute_liste_clients($form);
	return $form;
}

my $site_tbl = {
	'PAR_ID' => \&site_par_id, # Test
	'MODIF' => {
		'site'   => [ 'SITE_SITE',   \&set_site_site      ], # Test
		'login'  => [ 'SITE_LOGIN',  \&set_site_login     ], # Test
		'passwd' => [ 'SITE_PASSWD', \&set_site_passwd    ], # Test
		'client' => [ 'CLIENT_ID',   \&set_site_client_id ], # Test
	},
	'CREATE' => \&new_site, # Test
	'NEW' => {
		'site'   => 0,
		'login'  => 1,
		'passwd' => 2,
		'client' => \&set_site_client_id, # Test
	},
	'FORM' => 'nsite',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_site($$) { # Test
	my ( $vars, $id ) = @_;
	return __renseigne($site_tbl, $vars, $id );
}

sub modifie_site($$) { # Test
	my ( $vars, $id ) = @_;
	return __modifie($site_tbl, $vars, $id );
}

sub form_site($) {
	my ( $vars ) = @_;
	my $form = __do_form($site_tbl, $vars );
	ajoute_liste_clients($form);
	return $form;
}

my $domaine_tbl = {
	'PAR_ID' => \&domaine_par_id,
	'MODIF' => {
		'domaine' => [ 'DOMAINE_NOM',    \&set_domaine_nom       ], # Test
		'client'  => [ 'CLIENT_ID',      \&set_domaine_client_id ], # Test
		'type'    => [ 'DOMAINE_TYPE',   \&set_domaine_type      ], # Test
		'zone'    => [ 'DOMAINE_ZONE',   \&set_domaine_zone      ], # Test
		'fichier' => [ 'DOMAINE_FILE',   \&set_domaine_file      ], # Test
		'dnspri'  => [ 'DOMAINE_DNSPRI', \&set_domaine_dnspri    ], # Test
	},
	'CREATE' => \&new_domaine, # Test
	'NEW' => {
		'domaine' => 0,
		'type'    => 1,
		'client'  => \&set_domaine_client_id,
		'zone'    => \&set_domaine_zone,
		'fichier' => \&set_domaine_file,
		'dnspri'  => \&set_domaine_dnspri,
	},
	'FORM' => 'ndomaine',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_domaine($$) {
	my ( $vars, $id ) = @_;
	return __renseigne($domaine_tbl, $vars, $id);
}

sub modifie_domaine($$) {
	my ( $vars, $id ) = @_;
	return __modifie($domaine_tbl, $vars, $id);
}

sub form_domaine($) {
	my ( $vars ) = @_;
	my $form = __do_form($domaine_tbl, $vars);
	ajoute_liste_clients($form);
	return $form;
}

my $unixuser_tbl = {
	'NAME' => 'UNIXUSER',
	'DBH' => $dbh,
	'EQUIV' => {
		'name' => 'NAME',
		'passwd' => 'PASSWD',
		'uid' => 'UID',
		'groupe' => 'GID',
		'gecos' => 'GECOS',
		'home' => 'HOME',
		'shell' => 'SHELL',
		'state' => 'STATE',
		'dtin' => 'DTIN',
		'dtout' => 'DTOUT',
		'client' => 'CLIENT_ID'
		},
	'FORM' => 'nunixuser',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_unixuser($$) {
	my ( $vars, $id ) = @_;
	return __renseigne($unixuser_tbl, $vars, $id);
}

sub modifie_unixuser($$) {
	my ( $vars, $id ) = @_;
	return __modifie($unixuser_tbl, $vars, $id);
}

sub form_unixuser($) {
	my ( $vars ) = @_;
	my $form = __do_form($unixuser_tbl, $vars);
	ajoute_liste_unixgroup($form);
	ajoute_liste_clients($form);
	return $form;
}

my $unixgroup_tbl = {
	'NAME' => 'UNIXGROUP',
	'DBH' => $dbh,
	'EQUIV' => {
		'name' => 'NAME',
		'gid' => 'GID'
		},
	'FORM' => 'nunixgroup',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_unixgroup($$) {
	my ( $vars, $id ) = @_;
	return __renseigne($unixgroup_tbl, $vars, $id);
}

sub modifie_unixgroup($$) {
	my ( $vars, $id ) = @_;
	return __modifie($unixgroup_tbl, $vars, $id);
}

sub form_unixgroup($) {
	my ( $vars ) = @_;
	my $form = __do_form($unixgroup_tbl, $vars);
	return $form;
}

my $validation_tbl = {
	'NAME' => 'VALIDATION',
	'DBH' => $dbh,
	'EQUIV' => {
		'event' => 'EVENT',
		'info' => 'INFO',
		'etat' => 'ETAT',
		'ligne' => 'LIGNE_ID',
		'client' => 'CLIENT_ID'
		},
	'FORM' => 'nvalidation',
	'HIDDEN' => [
		[ 'id', 'hdo', 'hid' ]
	],
	'HTMLIDS' => {}
};

sub renseigne_validation($$) {
	my ( $vars, $id ) = @_;
	return __renseigne($validation_tbl, $vars, $id);
}

sub modifie_validation($$) {
	my ( $vars, $id ) = @_;
	return __modifie($validation_tbl, $vars, $id);
}

sub form_validation($) {
	my ( $vars ) = @_;
	my $form = __do_form($validation_tbl, $vars);
	ajoute_liste_clients($form);
	ajoute_liste_lignes($form);
	return $form;
}

BEGIN {
	__set_form_config_file('adsl/form/new.cfg');
}


1;


