package FDN::Adsl::Plutar;

use strict;
use Exporter;
our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	test_eligibilite
	plutar_commande_liazo
	get_pgp
	resil_change_radius
	attends
);
our @EXPORT_OK = qw ();
our %EXPORT_TAGS = ();

use strict;
use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Sql::DBC;
use FDN::Sql::Plutar::Constantes;
use FDN::Sql::Plutar::Basic;
use FDN::Adsl::Basic;
use FDN::Adsl::SelN;
use FDN::Adsl::Wf;
use FDN::Sql::Workflow::Run;
use FDN::Sql::Workflow::Basic;

sub x_error($) {
	print STDERR "ERR: ".$_[0]."\n";
	error $_[0];
}

sub x_info($) {
	print STDERR "INF: ".$_[0]."\n";
	info $_[0];
}

our $dbh;
our $notify_str = '';

BEGIN {
	$dbh = do_connect('adsl');
}

sub get_dbh() {
	return $dbh;
}

sub get_notify() {
	return $notify_str;
}

sub test_eligibilite($) {
	my ( $args ) = @_;
#	conduse('FDN::Adsl::Sel9');
	conduse('FDN::Sql::Workflow::Run');
	workflow_select_db('adsl');
	my $offre = $args->{'offre'};
	$offre =~ s/option/opt/;
	my $tel   = $args->{'tel'};
	$tel =~ s/^MIGRATION//;
	my $cp    = $args->{'cp'};
	my $eid   = $args->{'eid'};
	my $typend = $args->{'typend'};
	my $refimmeuble = $args->{'refimmeuble'};
	debug "J'interroge le serveur d'eligibilite sur l'offre $offre pour la ligne $tel ref immeuble $refimmeuble";
	my $res;
	if ($offre =~ /Liazo/i) {
		conduse('FDN::Adsl::SelL');
		$res = FDN::Adsl::SelL::eligibilite_liazo($tel,$typend,$refimmeuble,$cp,[$offre]);
	} else {
		conduse('FDN::Adsl::SelN');
		$res = FDN::Adsl::SelN::eligibilite($tel,$cp,[$offre]);
	}
	$tel =~ s/^VOISIN//;
	$tel =~ s/^FTTH//;
	if ( ! defined $args->{'resultats' } ) {
		$args->{'resultats'} = [];
	}
	if ( ! defined $res->{'body'} ) {
		$res->{'body'} = '<UNDEF>';
	}
	if ( $res->{'etat'} == 1 ) {
		# Serveur indisponible
		push @{$args->{'resultats'}}, "Retour etat 1 (serveur FT ou TD indispo). Retry.";
		return PLUTAR_RETRY;
	}
	if ( $res->{'etat'} == 2 ) {
		# Erreur chez FDN
		push @{$args->{'resultats'}}, "Retour etat 2. Bug FDN. A creuser.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	if ( $res->{'etat'} == 3 ) {
		# Probleme sur le numero de telephone
		push @{$args->{'resultats'}}, "Retour etat 3. Numero de telephone invalide.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	if ( $res->{'etat'} ) {
		# Etat impossible.
		push @{$args->{'resultats'}}, "Retour etat inconnu. Regarder la reponse dans la base.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	if ( ! defined $res->{'offres'} ||
	     ! defined $res->{'offres'}->{$offre} ||
	     ! defined $res->{'offres'}->{$offre}->{'faisable'} ) {
	     	push @{$args->{'resultats'}}, "Reponse du module FDN::Adsl::SelN invalide.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	set_eligibilite_reponse($eid,$res->{'body'});
	set_eligibilite_dttest($eid);
	if ( $res->{'offres'}->{$offre}->{'faisable'} ) {
		send_event_elig($eid,1);
		# send_event faisable
	} else {
		# send_event pas faisable
		send_event_elig($eid,0);
	}
	return PLUTAR_SUCCESS;
}

sub send_event_elig($$) {
	my ( $eid, $faisable ) = @_;
	send_event('ELIG-'.$eid,{'faisable'=>$faisable});
}

sub plutar_commande_liazo($$$) {
	my ( $props ) = @_;

	conduse('FDN::Adsl::SelL');
	workflow_select_db('adsl');
        # Récupérer le typend depuis la tâche, pour pouvoir facilement le
	# modifier depuis l'interface web pendant les échecs de commande
	my $task_props = get_task_properties($props->{'tid'});
	my $typend = $task_props->{'typend'};

	my $contact_tel = $props->{'telcontact'};

	if (!defined $contact_tel || length($contact_tel) != 10) {
		my $conf = 'adsl/elig/liazo.cfg';
		# pas un numéro normal, mettons le téléphone de FDN
		$contact_tel = Config_Key($conf,'ariane','suivi_tel');
		my $validateur = $props->{'validateur'};
		if ($validateur) {
			my $tel = Config_Key('adsl/global.cfg',$validateur,'suivi_tel');
			if ($tel) {
				$contact_tel = $tel;
			}
		}
	}

	my $tel = $props->{'telephone'};
	$tel =~ s/^FTTH//;

	my $res = commande_liazo(
		$props->{'debit'}, $props->{'option'},
		$tel,
		$typend,
		$props->{'refimmeuble'},

		$props->{'nom'},
		$contact_tel,
		$props->{'adresse'},

		$props->{'ensemble'},
		$props->{'building'},
		$props->{'stairs'},
		$props->{'floor'},
		$props->{'door'},
		$props->{'logo'},

		$props->{'cp'},
		$props->{'ville'},

		'', # dispo des locaux
		'', # date d'activation souhaitee

		# Infos sur un rendez-vous pour l'install
		undef, # Reference
		'', # Date
		'', # Plage horaire

		$props->{'ligneid'},

		$props->{'validateur'},

		# test : Pour ne pas vraiment passer la commande, mettre un truc vrai
		1
	);
	if (! defined($res)) {
		x_error "erreur commande Liazo pour la ligne $props->{'ligneid'}, telephone $props->{'telephone'}: commande invalide";
		my $ptid = $task_props->{'plutar-id'};
		$props->{'plutar-id'} = $ptid;
		my $plutar = plutar_par_id($ptid);
		my $mail = undef;
		if (! defined $plutar or $plutar->{'PLUTAR_NBRUN'} > 1) {
			$mail = Config_Key('adsl/global.cfg','mail','err_repe');
		}
		__envoie_mail($mail, 'bureau-rejetcde', $props->{'telephone'}, $props);
		return PLUTAR_RETRY;
	}
	debug "La commande a retourne $res";
	set_ordretd_ligne($props->{'ordretd-id'},$res);
	set_deploiement_cde($props->{deplid});
	$props->{tel} = $props->{telephone};
	$props->{urlsuivi} = Config_Key('adsl/global.cfg','web','urlsuivi');
	# FIXME: del existing property
	set_task_properties($props->{ptid}, { 'ordretd' => $res });
	envoie_mail($props->{'email'}, 'envoi', $props);
	return PLUTAR_SUCCESS;
}

sub get_pgp($) {
	my ( $args ) = @_;
	my $pgp = $args->{'pgp'};
	my $email = $args->{'email'};
	if (! ( $pgp =~ "^[0-9a-fA-F]{1,40}\$" ) ) {
		envoie_mail($email, 'pgp-failed', $args);
		return PLUTAR_FAILED;
	}
	my $res = system("gpg --keyserver keys.openpgp.org --recv-key $pgp > /dev/null 2>&1");
	if ($res == 0) {
		envoie_mail($email, 'pgp-success', $args);
		return PLUTAR_SUCCESS;
	} else {
		envoie_mail($email, 'pgp-failed', $args);
		return PLUTAR_RETRY;
	}
}

sub resil_change_radius($) {
	my ( $args ) = @_;
	my $rid = $args->{'radid'};
	my $email = $args->{'email'};

	my $ru = raduser_par_id($rid);
	if ( ! defined $ru ) {
		push @{$args->{'resultats'}}, "Le compte radius numero $rid n'existe pas.";
		return PLUTAR_FAILED;
	}
	my $login = $ru->{'RADUSER_LOGIN'};
	$args->{'ancien_login'} = $login;
	if ( $login =~ /fdn\.(fr|nerim|dslnet.fr|ilf.kosc|ilf.bouygues)$/ ) {
		$login =~ s/fdn\.(fr|nerim|dslnet.fr|ilf.kosc|ilf.bouygues)$/\/VALID/;
	} elsif ( $login =~ /VALID$/ ) {
		push @{$args->{'resultats'}}, "Le compte radius numero $rid est deja invalide";
		return PLUTAR_SUCCESS;
	} else {
		push @{$args->{'resultats'}}, "Le compte radius numero $rid ne se termine pas par 'fdn.fr' ou 'fdn.nerim' ou 'fdn.dslnet.fr' ou 'fdn.ilf.kosc' ou 'fdn.ilf.bouygues'";
		return PLUTAR_FAILED;
	}
	set_raduser_login($rid,$login);
	envoie_mail($email, 'resil-change-rad', $args);
	return PLUTAR_SUCCESS;
}

sub attends($) {
	my ( $args ) = @_;
	my $tid = $args->{'tid'};
	workflow_select_db('adsl');
	send_event('OK-attends-'.$tid);
	return PLUTAR_SUCCESS;
}

1;
