package FDN::Adsl::Purge;

use strict;
use Exporter;

# TODO: lister
# - les vieux CLIENT
# - les vieilles COMPTAINFO

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		purge_client
		purge_comptainfo

		purge_cloture
		purge_prlbq
		purge_newinfo
		);
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Adsl::Basic;
use FDN::Sql::Workflow::Basic;
use FDN::Sql::Workflow::Run;
use FDN::Sql::Plutar::Basic;
use FDN::Sql::Plutar::Run;

BEGIN {
	workflow_select_db('adsl');
	workflow_select_config('adsl/wf.cfg');
	plutar_select_db('adsl');
}

my $stockpdf = Config_Key('adsl/global.cfg','dirs','stockpdf');

#
# Note: pour être sûr de n'avoir pas oublié de table SQL, pour chaque table
# FOO on a une fonction purge_foo, qui ne fait qu'appeler des purge_bar, et
# finir par un del_foo. del_foo ne fait que supprimer la ligne dans la table
# FOO, il faut vérifier que pour toutes les lignes filles dans d'autres tables
# BAR on a bien appelé purge_bar dans purge_foo.
#

sub purge_image($) {
	my ( $name ) = @_;

	if ( $name eq 'invalid' ) {
		return;
	}
	if ( $name eq '' ) {
		return;
	}

	debug "je purge l'image $name";

	my $file;

	if ( $name =~ /^test/ ) {
		$file = "/test/$name.jpg";
	} elsif ( $name =~ /^tn-test/ ) {
		$file = "/test/$name.jpg";
	} elsif ( $name =~ /^tn-/ ) {
		$file = "/thumbnails/$name.jpg";
	} else {
		$file = "/jpeg/$name.jpg";
	}

	my $dir = Config_Key('adsl/global.cfg','dirs','imgdir');

	unlink $dir.$file;
}

sub purge_fichier($) {
	my ( $name ) = @_;

	if ( $name eq 'invalid' ) {
		return;
	}
	if ( $name eq '' ) {
		return;
	}

	debug "je purge le fichier $name";

	if ( $name =~ /^FDNC_.*\.xml$/ ) {
		my $dir_ntout = Config_Key('adsl/global.cfg','dirs','9tout');
		my $dir_ntoutdone = Config_Key('adsl/global.cfg','dirs','9toutdone');

		unlink "$dir_ntout/$name";
		unlink "$dir_ntoutdone/$name";
	} elsif ( $name =~ /^173_.*\.agp$/ ) {
		my $dir_tdout = Config_Key('adsl/global.cfg','dirs','tdout');
		my $dir_tdoutdone = Config_Key('adsl/global.cfg','dirs','tdoutdone');

		unlink "$dir_tdout/$name";
		unlink "$dir_tdoutdone/$name";
	} elsif ( $name =~ /^fdn_.*\.csv$/ ) {
		my $dir_nerimout = Config_Key('adsl/global.cfg','dirs','nerimout');
		my $dir_nerimoutdone = Config_Key('adsl/global.cfg','dirs','nerimoutdone');

		unlink "$dir_nerimout/$name";
		unlink "$dir_nerimoutdone/$name";
	} elsif ( $name =~ /\.pdf$/ ) {
		unlink $name;
		$name =~ s/\.pdf$/\.tex/;
		unlink $name;
		$name =~ s/\.tex$/\.aux/;
		unlink $name;
		$name =~ s/\.aux$/\.log/;
		unlink $name;
	} else {
		unlink $name;
	}
}

sub purge_plutar_par_propplutar($$$) {
	my ( $action, $nom, $val ) = @_;

	lock_plutar();
	my @plutars = plutar_par_action_propplutar($action, $nom, $val);
	for my $plutar ( @plutars ) {
		purge_plutar($plutar->{PLUTAR_ID});
	}
	unlock_plutar();
}

sub purge_task_par_proptask($$$) {
	my ( $name, $nom, $val ) = @_;

	my @tasks = task_par_name_proptask($name, $nom, $val);
	for my $task ( @tasks ) {
		purge_task($task->{TASK_ID});
	}
}

# SEQ nope
# STAT nope

# LIGNE plus bas
# VPN plus bas

sub purge_adresse($) {
	my ( $aid ) = @_;
	if ( $aid ) {
		purge_plutar_par_propplutar('gpgp', 'adresse-id', $aid);
		del_adresse($aid);
	}
}

# PRIX nope
# DEBIT nope
# PRIXDEBIT nope
# TARIF nope

# CLIENT plus bas

# ADHACCESS plus bas

# DEPLOIEMENT plus bas

sub purge_mandat($) {
	my ( $mid ) = @_;
	if ( $mid ) {
		my $mandat = mandat_par_id($mid);
		if (defined $mandat) {
			purge_fichier($mandat->{MANDAT_FICHIER});
			purge_image("tn-".$mandat->{MANDAT_IMAGE});
			purge_image($mandat->{MANDAT_IMAGE});
		}
		del_mandat($mid);
	}
}

sub purge_eligibilite($) {
	my ( $eid ) = @_;
	if ( $eid ) {
		purge_event_par_event("ELIG-$eid");
		purge_plutar_par_propplutar('elig', 'eid', $eid);
		purge_plutar_par_propplutar('elig9', 'eid', $eid);
		purge_plutar_par_propplutar('eligN', 'eid', $eid);
		purge_plutar_par_propplutar('eligL', 'eid', $eid);

		del_eligibilite($eid);
	}
}

sub purge_deploiement($) {
	my ( $did ) = @_;
	if ( $did ) {
		my $depl = deploiement_par_id($did);

		if (defined $depl) {
			# tâches depl-vpn, depl-adsl/9/L/N, change-adsl9/N/L
			my $tid = $depl->{'TASK_ID'};
			my $task = task_par_id($tid);
			if ( defined $task ) {
				my @selfs = taskself_par_parent($tid);
				for my $self ( @selfs ) {
					my $props = get_task_properties($self->{SELF_ID});
					if (defined $props->{'ordretd-id'}) {
						purge_ordretd($props->{'ordretd-id'});
					}
					if (defined $props->{'eligibilite-id'}) {
						purge_eligibilite($props->{'eligibilite-id'});
					}
				}
				purge_task($depl->{'TASK_ID'});
			}
		}

		purge_fichier("$stockpdf/$depl->{DEPLOIEMENT_NUM}/adhesion.pdf");
		purge_fichier("$stockpdf/$depl->{DEPLOIEMENT_NUM}/autorisation.pdf");
		purge_fichier("$stockpdf/$depl->{DEPLOIEMENT_NUM}/mandat-sepa.pdf");
		purge_fichier("$stockpdf/$depl->{DEPLOIEMENT_NUM}/souscription.pdf");
		purge_fichier("$stockpdf/$depl->{DEPLOIEMENT_NUM}/fdnforms.sty");
		purge_fichier("$stockpdf/$depl->{DEPLOIEMENT_NUM}/logoFDN.png");
		rmdir "$stockpdf/$depl->{DEPLOIEMENT_NUM}";

		purge_plutar_par_propplutar('commandeL', 'deplid', $did);

		my @mandats = mandat_par_deploiement_id($did);
		for my $mandat ( @mandats ) {
			purge_mandat($mandat->{MANDAT_ID});
		}

		del_deploiement($did);
	}
}

sub purge_creance($) {
	my ( $cid ) = @_;
	if ( $cid ) {
		del_creance($cid);
	}
}

sub purge_facture($) {
	my ( $fid ) = @_;
	if ( $fid ) {
		my $facture = facture_par_id($fid);
		if (defined $facture) {
			purge_fichier("$stockpdf/factures/fact-".$facture->{FACTURE_NUM}.".pdf");
		}
		del_facture($fid);
	}
}

sub purge_paiement($) {
	my ( $pid ) = @_;
	if ( $pid ) {
		del_paiement($pid);
	}
}

# TODO: CLOTURE ? Bof?

sub purge_prelevemt($) {
	my ( $pid ) = @_;

	if ( $pid ) {
		my $prl = prelevemt_par_id($pid);
		if (defined $prl) {
			my $cid = $prl->{CLIENT_ID};
			purge_adresse($prl->{'ADRESSE_ID'});
			purge_fichier("$stockpdf/adhacc-$cid/prel$pid/mandat-sepa.pdf");
			purge_fichier("$stockpdf/adhacc-$cid/prel$pid/fdnforms.sty");
			purge_fichier("$stockpdf/adhacc-$cid/prel$pid/logoFDN.png");
			rmdir "$stockpdf/adhacc-$cid/prel$pid";
		}
		del_prelevemt($pid);
	}
}

# PRLBQ plus bas

sub purge_prllig($) {
	my ( $pid ) = @_;
	if ( $pid ) {
		my $prllig = prllig_par_id($pid);
		my $prlbq = prlbq_par_id($prllig->{PRLBQ_ID});
		if (defined $prllig and defined $prlbq) {
			purge_fichier("$stockpdf/prllig/$prllig->{CLIENT_ID}/prl-$prlbq->{PRLBQ_DATE}-$prllig->{PRLLIG_ID}.pdf");
		}
		del_prllig($pid);
	}
}

sub purge_prlbq($) {
	my ( $pid ) = @_;
	if ( $pid ) {
		my $prlbq = prlbq_par_id($pid);
		my @prlligs = prllig_par_prlbq_id($pid);
		my @prlligssepa = prlligsepa_par_prlbq_id($pid);
		for my $prllig ( @prlligs ) {
			purge_prllig($prllig->{PRLLIG_ID});
		}
		for my $prlligsepa ( @prlligssepa ) {
			purge_prlligsepa($prlligsepa->{PRLLIGSEPA_ID});
		}
		my $prldir = Config_Key('adsl/global.cfg','dirs','prldir');
		purge_fichier("$prldir/$prlbq->{PRLBQ_FICHIER}");
		purge_fichier("$stockpdf/prlbq/prlbq-$prlbq->{PRLBQ_DATE}-$prlbq->{PRLBQ_ID}.pdf");
		del_prlbq($pid);
	}
}

sub purge_prlligsepa($) {
	my ( $pid ) = @_;
	if ( $pid ) {
		my $prlligsepa = prlligsepa_par_id($pid);
		my $prlbq = prlbq_par_id($prlligsepa->{PRLBQ_ID});
		if (defined $prlligsepa and defined $prlbq) {
			purge_fichier("$stockpdf/prllig/$prlligsepa->{CLIENT_ID}/prl-$prlbq->{PRLBQ_DATE}-$prlligsepa->{PRLLIGSEPA_ID}.pdf");
		}
		del_prlligsepa($pid);
	}
}

# TVA nope
# TVADCL nope

sub purge_info($) {
	my ( $iid ) = @_;
	if ( $iid ) {
		my $info = info_par_id($iid);
		if (defined $info and $info->{INFO_TYPE} eq 'fichier') {
			purge_image("tn-".$info->{INFO_VAL});
			purge_image($info->{INFO_VAL});
		}
		del_info($iid);
	}
}

sub purge_clientinfo($) {
	my ( $iid ) = @_;
	if ( $iid ) {
		purge_info($iid);
		del_clientinfo($iid);
	}
}

sub purge_ligneinfo($) {
	my ( $iid ) = @_;
	if ( $iid ) {
		purge_info($iid);
		del_ligneinfo($iid);
	}
}

sub purge_comptainfo($) {
	my ( $iid ) = @_;
	if ( $iid ) {
		purge_info($iid);
		del_comptainfo($iid);
	}
}

sub purge_vpninfo($) {
	my ( $iid ) = @_;
	if ( $iid ) {
		purge_info($iid);
		del_vpninfo($iid);
	}
}

# TODO: NEWINFO, what's this??

sub purge_ordretd($) {
	my ( $oid ) = @_;
	if ( $oid ) {
		my $ordretd = ordretd_par_id($oid);
		if (defined $ordretd) {
			purge_fichier($ordretd->{ORDRETD_FICHIER});
		}
		purge_event_par_event("ARTD-$oid");
		purge_event_par_event("CRTD-$oid");
		del_ordretd($oid);
	}
}

sub purge_payant($) {
	my ( $pid ) = @_;
	if ( $pid ) {
		del_payant($pid);
	}
}

sub purge_don($) {
	my ( $did ) = @_;
	if ( $did ) {
		del_don($did);
	}
}

# RADUSER plus bas

sub purge_uattr($) {
	my ( $uaid ) = @_;
	if ( $uaid ) {
		del_uattr($uaid);
	}
}

sub purge_raduser($) {
	my ( $radid ) = @_;
	if ( $radid ) {
		my @uattrs = uattr_par_raduser_id($radid);
		for my $uattr ( @uattrs ) {
			purge_uattr($uattr->{UATTR_ID});
		}

		purge_plutar_par_propplutar('resil_change_radius', 'radid', $radid);

		del_raduser($radid);
	}
}

sub purge_adhaccess($) {
	my ( $aid ) = @_;
	if ( $aid ) {
		my $raduser = raduser_par_adhaccess_id($aid);
		if (defined $raduser) {
			purge_raduser($raduser->{RADUSER_ID});
		}
		del_adhaccess($aid);
	}
}

# RADGROUP nope
# GATTR nope

# IPADDR nope

# COMPTA nope

# EVENT in task functions
# PROPEVENT in task functions
# TASK in task functions
# PROPTASK in task functions
# TASKSELF in task functions
# RUN in task functions
# WAITING in task functions

# PLUTAR in plutar functions
# PROPPLUTAR in plutar functions

sub purge_site($) {
	my ( $sid ) = @_;
	if ( $sid ) {
		del_site($sid);
	}
}

sub purge_domaine($) {
	my ( $did ) = @_;
	if ( $did ) {
		del_domaine($did);
	}
}

sub purge_notif($) {
	my ( $nid ) = @_;
	if ( $nid ) {
		del_notif($nid);
	}
}

# UNIXUSER plus bas

sub purge_unixuserunixgroup($$) {
	my ( $uuid, $ugid ) = @_;
	if ( $uuid and $ugid) {
		del_unixuserunixgroup($uuid, $ugid);
	}
}

sub purge_unixuser($) {
	my ( $uuid ) = @_;
	if ( $uuid ) {
		my @uugs = unixuserunixgroup_par_unixuser_id($uuid);
		for my $uug ( @uugs ) {
			purge_unixuserunixgroup($uug->{UNIXUSER_ID},
			                        $uug->{UNIXGROUP_ID});
		}
		del_unixuser($uuid);
	}
}

# UNIXGROUP nope

sub purge_validation($) {
	my ( $vid ) = @_;
	if ( $vid ) {
		del_validation($vid);
	}
}

sub purge_convocation($) {
	my ( $cid ) = @_;
	if ( $cid ) {
		del_convocation($cid);
	}
}

sub purge_vote($) {
	my ( $cid ) = @_;
	if ( $cid ) {
		del_vote($cid);
	}
}

sub purge_ligne($) {
	my ( $lid ) = @_;
	if ( $lid ) {
		my $ligne = ligne_par_id($lid);

		if (defined $ligne) {
			purge_adresse($ligne->{'ADRESSE_ID'});
		}

		my $raduser = raduser_par_ligne_id($lid);
		purge_raduser($raduser->{'RADUSER_ID'});

		my @depls = deploiement_par_ligne_id($lid);
		for my $depl ( @depls ) {
			purge_deploiement($depl->{'DEPLOIEMENT_ID'});
		}

		my @ligneinfos = ligneinfo_par_ligne_id($lid);
		for my $ligneinfo ( @ligneinfos ) {
			purge_ligneinfo($ligneinfo->{INFO_ID});
		}

		my @notifs = notif_par_ligne_id($lid);
		for my $notif ( @notifs ) {
			purge_notif($notif->{NOTIF_ID});
		}

		my @validations = validation_par_ligne_id($lid);
		for my $validation ( @validations ) {
			purge_validation($validation->{VALIDATION_ID});
		}

		purge_task_par_proptask('notif-perte', 'ligneid', $lid);
		purge_task_par_proptask('perte-ligne-normal', 'ligneid', $lid);
		purge_task_par_proptask('perte-ligne-anormal', 'ligneid', $lid);

		purge_task_par_proptask('resil-adsl', 'ligneid', $lid);

		del_ligne($lid);
	}
}

sub purge_vpn($) {
	my ( $vid ) = @_;
	if ( $vid ) {
		my $vpn = vpn_par_id($vid);

		if (defined $vpn) {
			purge_adresse($vpn->{'ADRESSE_ID'});
		}

		my $raduser = raduser_par_vpn_id($vid);
		purge_raduser($raduser->{'RADUSER_ID'});

		my @depls = deploiement_par_vpn_id($vid);
		for my $depl ( @depls ) {
			purge_deploiement($depl->{'DEPLOIEMENT_ID'});
		}

		my @vpninfos = vpninfo_par_vpn_id($vid);
		for my $vpninfo ( @vpninfos ) {
			purge_vpninfo($vpninfo->{INFO_ID});
		}

		purge_task_par_proptask('resil-vpn', 'vpnid', $vid);

		del_vpn($vid);
	}
}

sub purge_client($) {
	my ( $cid ) = @_;

	my $client = client_par_id($cid);

	debug "Je purge l'adhérent id ".$cid." de nom ".$client->{CLIENT_NOM};

	# On purge ses infos
	my @clientinfos = clientinfo_par_client_id($cid);
	for my $clientinfo ( @clientinfos ) {
		purge_clientinfo($clientinfo->{INFO_ID});
	}

	# On purge son adresse
	if (defined $client) {
		purge_adresse($client->{'ADRESSE_ID'});
	}

	# On purge ses prelevements
	my @prelevemts = prelevemt_par_client_id($cid);
	for my $prelevemt ( @prelevemts) {
		purge_prelevemt($prelevemt->{'PRELEVEMT_ID'});
	}

	my @creances = creance_par_client_id($cid);
	for my $creance ( @creances ) {
		purge_creance($creance->{CREANCE_ID});
	}

	my @factures = facture_par_client_id($cid);
	for my $facture ( @factures ) {
		purge_facture($facture->{FACTURE_ID});
	}

	my @prlligs = prllig_par_client_id($cid);
	for my $prllig ( @prlligs ) {
		purge_prllig($prllig->{PRLLIG_ID});
	}

	my @prlligsepas = prlligsepa_par_client_id($cid);
	for my $prlligsepa ( @prlligsepas ) {
		purge_prlligsepa($prlligsepa->{PRLLIGSEPA_ID});
	}

	my @paiements = paiement_par_client_id($cid);
	for my $paiement ( @paiements ) {
		purge_paiement($paiement->{PAIEMENT_ID});
	}

	my @payants = payant_par_client_id($cid);
	for my $payant ( @payants ) {
		purge_payant($payant->{PAYANT_ID});
	}

	my @dons = don_par_client_id($cid);
	for my $don ( @dons ) {
		purge_don($don->{DON_ID});
	}

	# On purge ses lignes
	my @ligs = ligne_par_client_id($cid);
	for my $ligne ( @ligs ) {
		purge_ligne($ligne->{'LIGNE_ID'});
	}

	# On purge ses vpns
	my @vpns = vpn_par_client_id($cid);
	for my $vpn ( @vpns ) {
		purge_vpn($vpn->{'VPN_ID'});
	}

	# On purge son accès adhérent
	my $adhacc = adhaccess_par_client_id($cid);
	if ($adhacc) {
		purge_adhaccess($adhacc->{'ADHACCESS_ID'});
	}

	# On purge ses sites
	my @sites = site_par_client_id($cid);
	for my $site ( @sites ) {
		purge_site($site->{SITE_ID});
	}

	# On purge ses domaines
	my @domaines = domaine_par_client_id($cid);
	for my $domaine ( @domaines ) {
		purge_domaine($domaine->{DOMAINE_ID});
	}

	# On purge ses unixuser
	my @unixusers = unixuser_par_client_id($cid);
	for my $unixuser ( @unixusers ) {
		purge_unixuser($unixuser->{UNIXUSER_ID});
	}

	# On purge ses convocations
	my @convocations = convocation_par_client_id($cid);
	for my $convocation ( @convocations ) {
		purge_convocation($convocation->{CONVOCATION_ID});
	}

	# On purge ses votes
	my @votes = vote_par_client_id($cid);
	for my $vote ( @votes ) {
		purge_vote($vote->{VOTE_ID});
	}

	purge_fichier("$stockpdf/prllig/$cid/prllig.pdf");
	purge_fichier("$stockpdf/prllig/$cid/logoFDN.png");
	purge_fichier("$stockpdf/prllig/$cid/fdnfactu.sty");
	rmdir "$stockpdf/prllig/$cid";
	purge_fichier("$stockpdf/adhacc-$cid/adhesion.pdf");
	purge_fichier("$stockpdf/adhacc-$cid/autorisation.pdf");
	purge_fichier("$stockpdf/adhacc-$cid/mandat-sepa.pdf");
	purge_fichier("$stockpdf/adhacc-$cid/fdnforms.sty");
	purge_fichier("$stockpdf/adhacc-$cid/logoFDN.png");
	unlink glob "$stockpdf/adhacc-$cid/ag20[0-9][0-9].pdf";
	rmdir "$stockpdf/adhacc-$cid";

	conduse('FDN::Adsl::Ldap');
	FDN::Adsl::Ldap::ldap_del_client($cid);
	del_client($cid);
}

1;
