package FDN::Adsl::Radius;

use strict;
use parent 'Exporter';

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		create_radius_l2tp
		change_radius_adsl
		create_radius_vpn
		create_radius_adh
		destroy_radius_account
		change_radius_ip
		resilie_radius_account

		update_adhaccess_login
		update_adhaccess_passwd
		control_radius_adh
		);
our @EXPORT_OK = qw ();

use DateTime;
use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Common::CharTable;
use FDN::Sql::Basic;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;
use FDN::Adsl::Ag;

sub get_radius_login($$) { # Test
	my ( $nom, $groupe ) = @_;
	my $grp = radgroup_par_nom($groupe);
	if ( ! defined $grp ) {
		error "Nom de groupe invalide ($groupe)";
		return undef;
	}
	my $gid = $grp->{'RADGROUP_ID'};
	my $suffix = $grp->{'RADGROUP_SUFFIX'};
	if ( ! $suffix ) {
		die "Je ne sais pas quel suffixe utiliser pour ce groupe...";
	}
	
	conduse('FDN::Common::CharTable');
	my $clean = cleanup_chars_as_table($nom, $iso8859_15_to_rfc822);
	$clean =~ s/^(.{0,25}).*$/$1/;
	$clean =~ s/\s/./g;
	my $i = 0;
	my $login;
	if ( ! lock_radius() ) {
		error "Pas reussi a obtenir le lock sur LCK_RADIUS en 10 secondes, que faire...";
		return undef;
	}
	do {
		$login = $clean;
		if ( $i ) {
			$login .= '.'.$i;
		}
		$login .= $suffix;
		$i++;
	} while ( raduser_par_login($login) );
	
	my $uid = new_raduser($login,$gid);
	unlock_radius();
	return $uid;
}

sub get_ip($$) { # Test
	my ( $uid, $type ) = @_;
	if ( ! lock_ip() ) {
		error "Pas reussi a obtenir le lock sur LCK_IP en 10 secondes, que faire...";
		return undef;
	}
	my $ip;
	if ($type =~ /\d+\.\d+\.\d+\.\d+/) {
	  # IP explicite
	  $ip = ipaddr_par_etat_addr('free',$type);
	} else {
	  $ip = ipaddr_par_etat_type('free',$type);
	}
	if ( ! defined ( $ip ) ) {
		error "Il n'y a plus d'IP de type $type libre!";
		unlock_ip();
		return undef;
	}
	my $ipid = $ip->{'IPADDR_ID'};
	set_ipaddr_etat($ipid,'busy');
	set_ipaddr_raduser_id($ipid,$uid);
	unlock_ip();
	return $ipid;
}

sub free_ip($) { # Test
	my ( $ipid ) = @_;
	set_ipaddr_raduser_id($ipid,-1);
	set_ipaddr_etat($ipid,'free');
}

sub ip2num { unpack "N", pack "C4", @_ }

sub get_ipv6_route($) {
	my ( $ip ) = @_;
	my $ipv6_num = 0x1000 + ip2num(split(/\./, $ip)) - ip2num(80,67,176,0);
	return sprintf("2001:910:\%04x::/48", $ipv6_num);
}

# On utilise le dernier /64 pour les IPs sur le lien
sub get_ipv6_link_addr($) {
	my ( $ip ) = @_;
	my $ipv6_num = 0x1000 + ip2num(split(/\./, $ip)) - ip2num(80,67,176,0);
	return sprintf("2001:910:\%04x:ffff::1", $ipv6_num);
}

sub add_uattrs($$) {
	my ( $uid, $attrs ) = @_;
	for my $uattr ( @$attrs ) {
		my ( $type, $name, $op, $value ) = @$uattr;
		new_uattr($uid, $type, $name, $op, $value);
	}
}

sub create_radius_l2tp($$$$;$) { # Test
	my ( $lid, $typeligne, $option, $nom, $typeip ) = @_;
	$typeip //= 'net1';
	my $uid;
	debug "type l2tp ".$typeligne." IP ".$typeip;
	if ($typeligne eq 'liazo') {
		if ($option eq 'LiazoKosc') {
			$uid = get_radius_login($nom,'FTTH IP fixe Liazo');
		} elsif ($option eq 'LiazoBouygues') {
			$uid = get_radius_login($nom,'FTTH IP fixe Liazo Bouygues');
		} else {
			$uid = get_radius_login($nom,'ADSL IP fixe Liazo');
		}
	} else {
		$uid = get_radius_login($nom,'ADSL IP fixe');
	}
	if ( ! defined ( $uid ) ) {
		error "Echec de creation d'un login radius pour $nom (ligne $lid)";
		return undef;
	}

	my $passwd = clef_aleatoire(8);
	add_uattrs($uid, [ [ 'check', 'User-Password', '==', $passwd] ] );

	my $ligne = ligne_par_id($lid);
	my $prixid = $ligne->{PRIX_ID};
	my $prix = prix_par_id($prixid);

	if ($prix->{PRIX_NOM} !~ /^Collecte/) {
		my $ipid = get_ip($uid, $typeip);
		if ( ! defined ( $ipid ) ) {
			error "Echec de reservation d'une IP pour $nom (ligne $lid)";
			del_raduser($uid);
			return undef;
		}

		my $ip = ipaddr_par_id($ipid);
		add_uattrs($uid,
			[ [ 'reply', 'Framed-IP-Address', '=', $ip->{'IPADDR_ADDR'}],
			  [ 'reply', 'Framed-IP-Netmask', '=', $ip->{'IPADDR_MASK'}],
			  [ 'reply', 'Framed-IPv6-Route', '=', get_ipv6_route($ip->{'IPADDR_ADDR'})] ] );
	}

	set_raduser_ligne_id($uid,$lid);
	return $uid;
}

# Changer de compte radius: ici on cre une copie avec le nouveau login
# On dsactivera l'ancienne copie plus tard, pour laisser  l'adhrent le temps
# de migrer
sub change_radius_adsl($$$$;$) {
	my ( $lid, $typeadsl, $nom, $radid, $type ) = @_;
	$type //= 'net1';
	my $uid;
	debug "type adsl ".$typeadsl;
	if ($typeadsl eq 'liazo') {
		$uid = get_radius_login($nom,'ADSL IP fixe Liazo');
	} else {
		$uid = get_radius_login($nom,'ADSL IP fixe');
	}
	if ( ! defined ( $uid ) ) {
		error "Echec de creation d'un login radius pour $nom (ligne $lid)";
		return undef;
	}
	# Copy all radius attributes
	my @attrs = uattr_par_raduser_id($radid);
	for my $uattr ( @attrs ) {
		new_uattr($uid,
			$uattr->{'UATTR_TYPE'},
			$uattr->{'UATTR_ATTR'},
			$uattr->{'UATTR_OP'},
			$uattr->{'UATTR_VALUE'});
	}
	set_raduser_ligne_id($uid,$lid);
	# And set the IP user to the new user
	my $ip = ipaddr_par_raduser_id($radid);
	set_ipaddr_raduser_id($ip->{'IPADDR_ID'},$uid);
	return $uid;
}

sub create_radius_vpn($$;$) { # Test
	my ( $vid, $nom, $type ) = @_;
	$type //= 'vpn1';
	my $uid = get_radius_login($nom,'VPN IP fixe')   if $vid;
	if ( ! defined ( $uid ) ) {
		error "Echec de creation d'un login radius pour $nom (vpn $vid)";
		return undef;
	}
	my $ipid = get_ip($uid, $type);
	if ( ! defined ( $ipid ) ) {
		error "Echec de reservation d'une IP pour $nom (vpn $vid)";
		del_raduser($uid);
		return undef;
	}
	my $ip = ipaddr_par_id($ipid);
	my $passwd = clef_aleatoire(8);
	add_uattrs($uid,
		[ [ 'check', 'User-Password', '==', $passwd ],
		  [ 'reply', 'Framed-IP-Address', '=', $ip->{'IPADDR_ADDR'} ],
		  [ 'reply', 'Framed-IP-Netmask', '=', $ip->{'IPADDR_MASK'} ],
		  [ 'reply', 'Framed-IPv6-Route', '=', get_ipv6_route($ip->{'IPADDR_ADDR'}) ],
		  [ 'reply', 'Framed-IPv6-Address', '=', get_ipv6_link_addr($ip->{'IPADDR_ADDR'})]
		] );
	set_raduser_vpn_id($uid,$vid);
	return $uid;
}

sub create_radius_adh($$$) { # Test
	my ( $adid, $nom, $passwd ) = @_;
	my $uid = get_radius_login($nom,'Adherents');
	if ( ! defined ( $uid ) ) {
		error "Echec de creation d'un login radius pour $nom (adhaccess $adid)";
		return undef;
	}

	add_uattrs($uid,
		[ [ 'check', 'User-Password', '==', $passwd ] ]);
	set_raduser_adhaccess_id($uid,$adid);
	return $uid;
}

sub change_radius_ip($;$) {
	my ( $uid, $type ) = @_;
	if ( ! defined ( $type ) ) {
		$type = 'net1';
	}
	my $ip = ipaddr_par_raduser_id($uid);
	my $ipid = get_ip($uid,$type);
	my $newip = ipaddr_par_id($ipid);
	my @uattrs = uattr_par_raduser_id($uid);
	if ( $#uattrs == -1 ) {
		error "On me demande de changer l'IP d'un.e abonne.e ($uid) qui n'a pas d'UATTR...";
		return;
	}
	for my $a ( @uattrs ) {
		my $aid = $a->{'UATTR_ID'};
		my $atr = $a->{'UATTR_ATTR'};
		if ( $atr eq 'Framed-IP-Address' ) {
			set_uattr_value($aid,$newip->{'IPADDR_ADDR'});
		} elsif ( $atr eq 'Framed-IP-Netmask' ) {
			set_uattr_value($aid,$newip->{'IPADDR_MASK'});
		} elsif ( $atr eq 'Framed-IPv6-Route' ) {
			set_uattr_value($aid,get_ipv6_route($newip->{'IPADDR_ADDR'}));
		} elsif ( $atr eq 'Framed-IPv6-Address' ) {
			set_uattr_value($aid,get_ipv6_link_addr($newip->{'IPADDR_ADDR'}));
		}
	}
	free_ip($ip->{'IPADDR_ID'});
	return $ipid;
}

sub destroy_radius_account($) {
	my ( $uid ) = @_;
	my $rad = raduser_par_id($uid);
	my $ip = ipaddr_par_raduser_id($uid);
	my $login;
	if ( ! defined $rad ) {
		error "On me demande de detruire le compte radius $uid qui n'existe pas.";
		return;
	}
	if ( defined $ip ) {
		free_ip($ip->{'IPADDR_ID'});
	}
	if ( $rad->{'RADGROUP_ID'} < 4 ) { # ADSL = 1, VPN = 3
		$login = $rad->{'RADUSER_LOGIN'};
		$login =~ s/fdn.(fr|nerim|dslnet.fr|ilf.kosc|ilf.bouygues)$/fdn\@INVALID/;
		$login =~ s/apache$/apache\@INVALID/;
		set_raduser_login($uid,$login);
	} else {
		del_uattr_par_raduser_id($uid);
		del_raduser($uid);
	}
	return;
}

sub update_adhaccess_login {
	my ( $adid, $login ) = @_;
	my $rad = raduser_par_adhaccess_id($adid);
	if ( $rad ) {
		my $rid = $rad->{'RADUSER_ID'};
		set_raduser_login($rid, $login.'@apache');
	}
	return set_adhaccess_login($adid, $login);
}

sub update_adhaccess_passwd {
	my ( $adid, $passwd ) = @_;
	my $rad = raduser_par_adhaccess_id($adid);
	my $done = 0;
	if ( $rad ) {
		my $rid = $rad->{'RADUSER_ID'};
		my $uattrs = [ uattr_par_raduser_id($rid) ];
		for my $ua ( @$uattrs ) {
			my $uaid = $ua->{'UATTR_ID'};
			if ( $ua->{'UATTR_ATTR'} eq 'User-Password' ) {
				set_uattr_value($uaid, $passwd);
				$done++;
			}
		}
	}
	return set_adhaccess_passwd($adid, $passwd);
}

sub control_one_radius_adh {
	my ( $adh, $rad, $dry ) = @_;
	my $res = '';
	my $rid = $rad->{'RADUSER_ID'};
	if ( $rad->{'RADUSER_LOGIN'} ne $adh->{'ADHACCESS_LOGIN'}.'@apache' ) {
		if ( ! $dry ) {
			set_raduser_login($rid, $adh->{'ADHACCESS_LOGIN'}.'@apache');
		}
		$res .= "Mise a jour du login pour le raduser $rid\n";
	}
	my $uattrs = [ uattr_par_raduser_id($rid) ];
	for my $ua ( @$uattrs ) {
		my $uaid = $ua->{'UATTR_ID'};
		if ( $ua->{'UATTR_ATTR'} eq 'User-Password' && $ua->{'UATTR_VALUE'} ne $adh->{'ADHACCESS_PASSWD'} ) {
			$res .= "Mise a jour du mot de passe pour le raduser $rid\n";
			if ( ! $dry ) {
				set_uattr_value($uaid, $adh->{'ADHACCESS_PASSWD'});
			}
		}
	}
	return $res;
}

sub __clean_raduser($$$$) {
	my ( $rads, $adid, $login, $dry ) = @_;
	my $res = "";
	if ( exists $rads->{$adid} ) {
		my $uid = $rads->{$adid}->{'RADUSER_ID'};
		$res .= "Il y a un compte radius pour $login qui ne doit pas en avoir. Je le vire.\n";
		if ( ! $dry ) {
			del_uattr_par_raduser_id($uid);
			del_raduser($uid);
		}
		delete $rads->{$adid};
	}
	return $res;
}


sub control_radius_adh {
	my ( $dry ) = @_;
	$dry //= 1;
	my $reply = "";

	my $group = radgroup_par_nom('Adherents');
	if ( ! $group ) {
		Carp::confess "La base n'est pas a jour. Il doit y avoir un RADGROUP qui s'appelle 'Adherents'. Il fait partie de la livraison du SI.";
	}
	my $rgid = $group->{'RADGROUP_ID'};

	# On remonte tous les comptes radius du groupe 'Adherents' et on les index par l'ADHACCESS_ID
	# auquel ils sont rattachs.
	my $tmp = [ raduser_par_radgroup_id($rgid) ];
	my $rads = {};
	for my $rad ( @$tmp ) {
		my $adid = $rad->{'ADHACCESS_ID'};
		if ( $adid ) {
			$rads->{$adid} = $rad;
		} else {
			$reply .= "Le compte radius '', dans le groupe 'Adherents', n'est pas associe a un ADHACCESS, je vire.\n";
			if ( ! $dry ) {
				my $uid = $rad->{'RADUSER_ID'};
				del_uattr_par_raduser_id($uid);
				del_raduser($uid);
			}
		}
	}

	my $adhs = [ tous_adhaccess ];
	my $date = DateTime->now->ymd;
	for my $adh ( @$adhs ) {
		my $cid = $adh->{'CLIENT_ID'};
		my $adid = $adh->{'ADHACCESS_ID'};
		my $login = $adh->{'ADHACCESS_LOGIN'};

		# Recherche et nettoyage des doublons. On prend la methode utilisee pour la convocation
		# a l'AG pour choisir celui de reference.
		my $adh_ag = adhaccess_par_client_id($cid);
		if ( $adh_ag->{'ADHACCESS_ID'} != $adid ) {
			$reply .= __clean_raduser($rads, $adid, $login, $dry);
			if ( ! $dry ) {
				del_adhaccess($adid);
			}
			$reply .= "Je vire le ADHACCESS en doublon pour $login\n";
			next;
		}
		if ( droit_de_vote($cid, $date, 'present') ) {
			# Il a le droit de venir en AG, donc il a le droit d'avoir un compte
			if ( exists $rads->{$adid} ) {
				$reply .= control_one_radius_adh($adh, $rads->{$adid}, $dry);
				delete $rads->{$adid};
				next;
			}
			$reply .= "Il manque le compte radius pour $adh->{'ADHACCESS_LOGIN'}. Je l'ajoute.\n";
			if ( ! $dry ) {
				create_radius_adh($adid, $adh->{'ADHACCESS_LOGIN'}, $adh->{'ADHACCESS_PASSWD'});
			}
			next;
		} else {
			# Il n'a pas le droit de venir en AG, donc il n'a pas le droit d'avoir un compte
			$reply .= __clean_raduser($rads, $adid, $login, $dry);
			next;
		}
	}

	# Tout ce qui reste dans $rads ce sont ces comptes radius pour lesquels il n'y
	# a plus d'entrees dans la table ADHACCESS -> poubelle
	for my $rad ( values %$rads ) {
		my $uid = $rad->{'RADUSER_ID'};
		my $login = $rad->{'RADUSER_LOGIN'};
		$reply .= "Le compte radius '$login' n'est plus rattache a rien (le ADHACCESS a disparu), je le vire\n";
		if ( ! $dry ) {
			del_uattr_par_raduser_id($uid);
			del_raduser($uid);
		}
	}
	# Si tout etait clair et propre, on doit retourner un texte vide.
	return $reply;
}

1;
