package FDN::Adsl::SelL;

# Documentation sur https://portal.ielo-liazo.com/doc/fr/

use strict;
use warnings;

use Math::Trig;

use base 'Exporter';
our @ISA = ( 'Exporter' );
our @EXPORT = qw(
	eligibilite_liazo

	sign
	verify

	get_ville
	get_rue
	get_gps
	gps_to_web
	web_to_gps
	get_immeubles_liste
	get_immeubles_geom
	get_info_immeuble_pto
	get_list_immeuble_pto
	get_info_immeuble
	get_info_addr_pto
	get_nd
	get_nd_nd
	get_nd_typend
	affiche_get_nd
	get_info_nd
	get_info_addr_nd
	get_info_addr_nd_pto
	get_addr_nd_pto
	get_addr_immeuble
	get_addr_from_info_addr_nd
	get_fiber
	get_sdsl
	get_adsl
	get_ftth
	get_contracts
	get_contract
	get_services
	get_service
	get_commande_serviceref
	get_commande_contractref
	get_adsl_delivery_nd
	get_collecte
	commande_liazo
	get_commandes
	get_info_commande
	get_deploiement_commande
	get_ligne_commande
	annuler_commande
	test_adsl
	list_tests_adsl
	reset_adsl

	get_reponse_post
	get_reponse_get

	poll
	poll_resil
	);
our @EXPORT_OK = qw(
	);

use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Web::TemplateParse;
use FDN::Sql::Basic;
use FDN::Sql::DB_Access;
use FDN::Adsl::OrdreTd;
use FDN::Adsl::DBC;
use FDN::Adsl::Basic;
use FDN::Adsl::Wf;
use FDN::Adsl::Infos;
use FDN::Sql::Workflow::Basic;

use HTTP::Headers;
use HTTP::Request;
use LWP::UserAgent;
use JSON qw( decode_json encode_json );
use Encode qw ( encode );
use Text::Iconv;
use Data::Dumper;
use Scalar::Util 'looks_like_number';

use Digest::SHA;
use IPC::Open2;

our $debug = 1;

# Valeurs indexes par l'attnuation
# Mesures plus ou moins empiriques venant de
# https://www.speedguide.net/dsl_speed_calc.php
#
# Ce sont les valeurs de synchro, pas les dbits constats en IP
my @debits = qw(
    100000
    99866
    92874
    88087
    83199
    78311
    73423
    68536
    63648
    58760
    53872
    48985
    44097
    40243
    37165
    34088
    31010
    27933
    24953
    23958
    22962
    21966
    20971
    19975
    19197
    18618
    18038
    17459
    16880
    16301
    15540
    14780
    14020
    13259
    12499
    11742
    11199
    10656
    10113
    9570
    9027
    8656
    8294
    7932
    7570
    7207
    6830
    6432
    6033
    5636
    5237
    4839
    4604
    4387
    4169
    3952
    3735
    3497
    3226
    2954
    2682
    2411
    2139
    2038
    1966
    1893
    1821
    1748
    1628
    1411
    1194
    976
    759
    542
    481
    456
    432
    408
);

# Codes de faisabilit de Liazo:
# 0: En cours de raccordement
# 1: Possible
# 2: Impossible (voir raison)
# 3: Possible sous rserve technique
# 4: Possible sous condition commerciale

my $tbls = 'adsl/tables.cfg';
my $conf = 'adsl/elig/liazo.cfg';

my $conv = Text::Iconv->new("latin1", "ascii//translit");

sub __debug($) {
	my ( $msg ) = @_;
	debug $msg if ( $debug );
}

sub doJSONbool($) {
	my ($b) = @_;
	if ($b eq "true") {
		return JSON::true;
	} else {
		return JSON::false;
	}
}

# Signer un message  poster  Liazo
sub sign($) {
	#  gnrer avec
	# openssl ecparam -genkey -name prime256v1 -out private.pem
	# et envoyer la partie publique obtenue avec
	# openssl ec -in private.pem -pubout -out public.pem
	my $private_file = Config_Key($conf,'ariane','sign_key');
        my ($msg) = @_;
        my $digest = Digest::SHA::sha256($msg);
        my $signature;

        # Idalement
        #use Crypt::OpenSSL::ECDSA;
        #open FILE, $private_file or die "cle non lisible";
        #my $private;
        #{
        #       local $/ = undef;
        #       $private = <FILE>;
        #}
        #close FILE;
        #
        #$signature=Crypt::OpenSSL::ECDSA->sign($digest,$private);

        # Mais les modules perl ne sont pas encore l, donc on utilise le module python
        my ( $in, $out );
        my $pid = open2($out, $in, 'python -c "'.
                "import ecdsa;".
                "import sys;".

                "key=open('$private_file').read();".
                "sk=ecdsa.SigningKey.from_pem(key);".
                "sys.stdout.write(sk.sign(sys.stdin.read()));".
                '"');
        print $in $digest;
        close $in;
        waitpid($pid, 0);
        my $status = $? >> 8;
        die if $status != 0;
        {
                local $/ = undef;
                $signature = <$out>;
        }

	return unpack('H*', $signature);
}

# Vrifier une signature de message  poster  Liazo
sub verify($$) {
        my $public_file = Config_Key($conf,'ariane','sign_pub');
        my ($msg, $signature) = @_;
        my $digest = Digest::SHA::sha256($msg);

        # Idalement
        #use Crypt::OpenSSL::ECDSA;
        #open FILE, $public_file or die "cle non lisible";
        #my $public;
        #{
        #       local $/ = undef;
        #       $public = <FILE>;
        #}
        #close FILE;

        # Mais les modules perl ne sont pas encore l, donc on utilise le module python
        my ( $in, $out );
        my $pid = open2($out, $in, 'python -c "'.
                "import ecdsa;".
                "import sys;".
                "import binascii;".

                "signature='$signature';".
                "signature=binascii.unhexlify(signature);".

                "key=open('$public_file').read();".
                "vk=ecdsa.VerifyingKey.from_pem(key);".
                "vk.verify(signature,sys.stdin.read());".
                '"');
        print $in $digest;
        close $in;
        waitpid($pid, 0);
        my $status = $? >> 8;
        die if $status != 0;
}

sub get_eligibilite($$$) {
	my ($tel, $typend, $refimmeuble) = @_;
	my $inactive = "false";
	my $precab = "false";
	if ($typend eq 'inactif') {
		$inactive = "true";
	} elsif ($typend eq 'precab') {
		$precab = "true";
	}

	my @info = qw();
	my $via_tel = 0;

	if ($tel =~ /^(0[1-5]\d{8}|42C[A-Z][A-Z][0-9]{15})$/) {
		my $via_tel = 1;
		my $info_adsl = get_adsl($tel,$inactive,$precab);
		if ( ! defined $info_adsl || ! defined $info_adsl->[0] ) {
			debug('Erreur serveur Liazo ADSL');
		} else {
			if ( $info_adsl->[0]->{'request'}->{'phone_number'} ne $tel ) {
				debug('Rponse ne portant pas sur le bon NDI. Bug?');
			} else {
				@info = (@info, @$info_adsl);
			}
		}
	}

	if ($precab eq "false") {
		# Il semblerait que l'ligibilit via le numro de tlphone ne soit pas super prcise...
		# Cf https://portal.ielo-liazo.com/private/tickets/?view=open&id=105511#response_1177167
		# Aprs, de toutes faons  terme on va toujours passer par la rfrence d'immeuble a priori?
		my $info_ftth;
		if (0) {
			$info_ftth = get_ftth($tel,$inactive);
		} else {
			$info_ftth = get_ftth(undef,undef,$refimmeuble);
		}
		if ( ! defined $info_ftth || ! defined $info_ftth->[0] ) {
			debug('Erreur serveur Liazo FTTH');
		} else {
			@info = (@info, @$info_ftth);
		}
	}

	if ( scalar(@info) == 0 and $via_tel) {
		debug('Pas de rponse :/');
		return (undef, undef, "Pas de rponse :/", undef);
	}

	return $inactive, $precab, undef, @info;
}

# Tester l'ligibilit d'un numro de tlphone
sub eligibilite_liazo($$$$$) {
	my ( $tel, $typend, $refimmeuble, $zip, $offres ) = @_;
	my $gres = {};
	$gres->{'offres'} = {};
	$gres->{'etat'} = 0;
	$gres->{'body'} = '';
	my $reverse = {};
	for my $offre ( @$offres ) {
		my $debit = get_offre($offre);
		next if ( ! defined $debit );
		$reverse->{$debit} = $offre;
	}

	my $voisin = ($tel =~ /^VOISIN/);
	$tel =~ s/^VOISIN//;
	$tel =~ s/^FTTH//;

	my ( $inactive, $precab, $error, @infos ) = get_eligibilite($tel, $typend, $refimmeuble);

	if (defined($error)) {
		$gres->{'etat'} = 1;
		$gres->{'message'} = [ $error ];
		return $gres;
	}

	$gres->{'body'} = "Adresse ligne selon Liazo:\n".get_addr_nd_pto($zip, $refimmeuble, $tel, $typend);

	for my $offre ( @infos ) {
		my $name = $offre->{'product'}->{'bandwidth_mb'};
		my $reason = '';

		if ($offre->{'product'}->{'code'} =~ /^kosc.ftth.*k1.local/) {
			$name .= "K1LiazoKosc";
		} elsif ($offre->{'product'}->{'code'} =~ /^kosc.ftth.*k2.local/) {
			$name .= "K2LiazoKosc";
		} elsif ($offre->{'product'}->{'code'} =~ /^bouygues.ftth.*.yt1.local/) {
			$name .= "B1LiazoBouygues";
		} elsif ($offre->{'product'}->{'code'} =~ /^bouygues.ftth.*.yt2.local/) {
			$name .= "B2LiazoBouygues";
		} elsif ($offre->{'product'}->{'code'} =~ /^ft.cuivre..dsl/) {

			# On ne s'embte pas avec du bi-vc pour l'instant.
			if (defined $offre->{'product'}->{'vc_type'} and $offre->{'product'}->{'vc_type'} eq 'bi-vc') {
				next;
			}

			if (defined $offre->{'product'}->{'tech'} and $offre->{'product'}->{'tech'} eq 'RE') {
				$name .= 'RE';
			}

			$name .= "LiazoOrange";
			if ($offre->{'product'}->{'dsl_only'}) {
				$name .= 'Nu';
			} else {
				if ($typend eq "precab") {
					# Uh, a n'a pas de sens, un pr-cbl n'a pas encore de ligne tlphonique, virer ce choix  la main
					# rapport sur https://portal.ielo-liazo.com/private/tickets/?view=open&id=48664
					next;
				}
				if ($voisin) {
					# Une commande par voisin est forcment avec dgroupage total
					next;
				}
			}
		} else {
			$reason .= "$offre->{'product'}->{'code'} non support";
			$offre->{'feasibility'} = 0;
		}
		__debug "examen offre $offre->{'product'}->{'code'}, $name";
		if (defined $reverse->{$name}) {
			__debug "reverse ".$reverse->{$name};
		}

		if ($offre->{'product'}->{'code'} =~ /^ft.cuivre/) {
			if (scalar @{$offre->{'response'}->{'reason'}}) {
				for my $one_reason ( @{$offre->{'response'}->{'reason'}} ) {
					$reason .= $one_reason;
				}
			} else {
				$reason = $offre->{'response'}->{'feasibility_res'};
				if ($reason eq "ND non attribu") {
					$reason = "Numro de tlphone actuellement pas attribu";
				}
			}
		}
		$reason =~ s/\x{2019}/'/g;

		if ($offre->{'feasibility'} eq '1') {
			$offre->{'faisable'} = 1;
		} elsif ($offre->{'feasibility'} eq '3') {
			$offre->{'faisable'} = 1;
			$offre->{'reserve'} = $reason;
		} elsif ($offre->{'feasibility'} eq '4') {
			$offre->{'faisable'} = 1;
			$offre->{'reserve'} = $reason;
		} else {
			$offre->{'faisable'} = 0;
		}
		my $atten = 0;
		if (!defined $offre->{'length'} or $offre->{'length'} eq 'unknown' or $offre->{'length'} eq 'n/a') {
			$offre->{'LineLength'} = "inconnue";
		} else {
			my @lengths = split /, /,$offre->{'length'};
			my @calibers = split /, /,$offre->{'caliber'};
			my @sections;
			my $l = 0;
			for (my $i = 0; $i < scalar @lengths; $i++) {
				my $factor = 0;
				$lengths[$i] =~ s/^0*//g;
				$calibers[$i] =~ s/^0*//g;
				push @sections, [$lengths[$i],$calibers[$i]];
				$l += $lengths[$i];
				if (defined $atten) {
					if ($calibers[$i] == 4) { $factor = 15; }
					elsif ($calibers[$i] == 5) { $factor = 12.4; }
					elsif ($calibers[$i] == 6) { $factor = 10.3; }
					elsif ($calibers[$i] == 8) { $factor = 7.9; }
					else { $atten = undef; }
				}
				if (defined $atten) {
					$atten += $factor * $lengths[$i] / 1000;
				}
			}
			$offre->{'LineLength'} = "$l";
			$offre->{'Sections'} = \@sections;
		}
		$offre->{'Attenuation'} = "$atten";
		$atten = int($atten+0.9999);
		if ($atten < $#debits) {
			$offre->{'debit'} = $debits[$atten] * 8 / 10000;
			__debug "debit ".$offre->{'debit'};
		}
		__debug "faisable: ".$offre->{'faisable'};
		if ($offre->{'faisable'} eq 0) {
			$offre->{'infos'} = '';
			$offre->{'infos'} .= $reason;
			__debug "pas faisable: ".$offre->{'infos'};
		}
		my $revname = $reverse->{$name};
		if (not defined $revname) {
			$revname = $offre->{'product'}->{'code'};
		}
		$offre->{'offre'} = $revname;
		$gres->{'offres'}->{$revname} = $offre;
	}

	return $gres;
}

# Usage interne: convertit un code d'offre FDN en notre propre code
sub get_offre($) {
	my ( $offre ) = @_;
	my ( $debit, $option );
	if ( $offre =~ /^(\d+(k|M|G|RE)(K1|K2|B1|B2|))(Liazo(Orange(|Nu)|Kosc|Bouygues))$/ ) {
		( $debit, $option ) = ($1,$4);
	} else {
		__debug "Offre inconnue (syntaxe): $offre";
		return undef;
	}
	my $res; 
	$res = Config_Key($tbls,"$option-liazo",$debit);
	if ( ! defined $res ) {
		__debug "Offre inconnue (config): $offre, $debit, $option";
		return undef;
	}
	$res .= $option;
	__debug "Offre trouvee, $offre se dit $res.";
	return $res; 
}

# Demandes d'informations
sub get_ville($;$$) {
	my ( $query, $zip, $limit ) = @_;
	my $q = "";
	if (defined $query) {
		$q .= "query=".$query;
	}
	if (defined $zip) {
		if (length($zip) == 2) {
			$q .= "&depart_code=$zip";
		} else {
			$q .= "&post_code=$zip";
		}
	}
	if (defined $limit) {
		$q .= "&limit=$limit";
	}
	return get_reponse_get('/geobase/list/city', $q);
}

sub get_rue($$$;$) {
	my ( $query, $ville, $zip, $limit ) = @_;
	$query = "query=".$query;
	if (defined $ville) {
		$query = $query."&city=$ville";
	}
	if (defined $zip) {
		$query = $query."&post_code=$zip";
	}
	if (defined $limit) {
		$query = $query."&limit=$limit";
	}
	return get_reponse_get('/geobase/list/street', $query);
}

sub get_gps($$$$) {
	my ( $numero, $rue, $zip, $ville ) = @_;
	my $query = "city=$ville";
	if (defined $zip) {
		$query = $query."&post_code=$zip";
	}
	if (defined $rue) {
		$rue =~ s/\bAVENUE\b/AV/g;
		$query = $query."&street=$rue";
	}
	if (defined $numero) {
		$query = $query."&street_nb=$numero";
	}
	return get_reponse_get('/geobase/get/gps', $query);
}

my $factor = 6378137;

sub gps_to_web($$) {
	my ( $lat, $lon ) = @_;
	my $x = $lon * pi / 180 * $factor;
	my $y = log(tan(pi/4 + $lat*pi/180 / 2)) * $factor;
	return $x, $y;
}

sub web_to_gps($$) {
	my ( $x, $y ) = @_;
	my $lon = $x / $factor * 180 / pi;
	my $lat = (2*atan(exp($y / $factor)) - pi/2) * 180 / pi;
	return $lat, $lon;
}

sub get_immeubles_liste($$) {
	my ( $zip, $ville ) = @_;
	my $query = "city=$ville&post_code=$zip";
	return get_reponse_get('/geobase/list/building', $query);
}

sub get_immeubles_geom($$$$) {
	my ( $x0, $y0, $x1, $y1 ) = @_;
	my $query = "x0=$x0&y0=$y0&x1=$x1&y1=$y1";
	return get_reponse_get('/geobase/map/building', $query);
}

sub clean_info($);
sub clean_info($) {
	my ( $res ) = @_;
	if (exists($res->{'stair'})) {
		# Pour cohrence plus simple avec le reste du SI
		$res->{'stairs'} = $res->{'stair'};
		delete($res->{'stair'});
	} else {
		for my $k ( keys %$res ) {
			if (ref $res->{$k} eq 'ARRAY') {
				my $arr = $res->{$k};
				for my $e (@$arr) {
					if (ref $e eq 'HASH') {
						clean_info($e);
					}
				}
			}
		}
	}
}

sub get_info_immeuble($) {
	my ( $ref ) = @_;
	my $query = "reference_building=$ref";
	my $res = get_reponse_get('/geobase/get/building', $query);
	clean_info($res);
	return $res;
}

sub get_immeuble_pto_info($$) {
	my ( $info, $pto ) = @_;
	my $list = $info->{pto};
	if ( defined $list ) {
		for my $elem (@$list) {
			if ($elem eq $pto) {
				return {};
			}
		}
	}
	return undef;
}

# This recurses into the different building structure lists to find the $pto,
# and whenever found, records where it was found in $info.
sub get_immeuble_recur($$$$) {
	my ( $info, $pto, $name, $fun ) = @_;
	my $res = get_immeuble_pto_info($info, $pto);
	if (defined $res) {
		return $res;
	}
	my $list = $info->{$name};
	if ( defined $list ) {
		for my $elem ( @$list ) {
			my $res = $fun->($elem, $pto);
			if (defined $res) {
				my $id = $elem->{id};
				if ($id ne '_NA_') {
					$res->{$name} = $id;
				}
				return $res;
			}
		}
	}
	return $fun->($info, $pto);
}

sub get_immeuble_logo_info($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_recur($info, $pto, 'logo', \&get_immeuble_pto_info);
}

sub get_immeuble_door_info($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_recur($info, $pto, 'door', \&get_immeuble_logo_info);
}

sub get_immeuble_floor_info($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_recur($info, $pto, 'floor', \&get_immeuble_door_info);
}

sub get_immeuble_stairs_info($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_recur($info, $pto, 'stairs', \&get_immeuble_floor_info);
}

sub get_immeuble_building_info($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_recur($info, $pto, 'building', \&get_immeuble_stairs_info);
}

sub get_immeuble_ensemble_info($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_recur($info, $pto, 'ensemble', \&get_immeuble_building_info);
}

sub get_info_immeuble_pto($$) {
	my ( $info, $pto ) = @_;
	return get_immeuble_ensemble_info($info, $pto);
}

sub get_list_immeuble_pto($);
sub get_list_immeuble_pto($) {
	my ( $info ) = @_;

	my $res = $info->{pto};
	if ( not defined $res ) {
		$res = qw();
	}

	for my $name (qw(logo door floor stairs building ensemble)) {
		my $list = $info->{$name};
		if ( defined $list ) {
			for my $elem ( @$list ) {
				my $res2 = get_list_immeuble_pto($elem);
				if (defined $res2) {
					push @$res, @$res2;
				}
			}
		}
	}
	return $res;
}

sub get_info_addr_pto($$$) {
	my ( $zip, $ref, $pto ) = @_;
	my $info = get_info_immeuble($ref);
	my $res = get_addr_immeuble($zip, $ref);
	my $res2 = get_info_immeuble_pto($info, $pto);
	debug "pour $zip $ref $pto j'ai recupere ".Dumper($res)." et ".Dumper($res2);
	if (! defined $res and !defined $res2) {
		return undef;
	}
	if (! defined $res) {
		return $res2;
	}
	if (! defined $res2) {
		return $res;
	}
	return {%$res, %$res2};
}

sub mycmp($$) {
	my ( $a, $b ) = @_;
	return ( defined $a and defined $b and
		( (looks_like_number($a) and looks_like_number($b) and $a <=> $b)
		  or ($a cmp $b) ) );
}

sub get_nd($$$;$) {
	my ( $ville, $zip, $rue, $num ) = @_;
	$ville =~ s/-/ /g;
	$rue =~ s/\bAVENUE\b/AV/g;
	my $query = "city=$ville&post_code=$zip&street=$rue";
	if ($num) {
		$query = $query."&street_nb=$num";
	}
	my $ret = get_reponse_get('/eligibility/search/nd', $query);
	if (!defined $ret) {
		return undef;
	}

	for my $item ( @$ret ) {
		if (defined $item->{'ensemble'}) {
			$item->{'ensemble'} =~ s/^0*//;
		}
		if (defined $item->{'building'}) {
			$item->{'building'} =~ s/^0*//;
		}
		if (defined $item->{'stairs'}) {
			$item->{'stairs'} =~ s/^0*//;
		}
		if (defined $item->{'floor'}) {
			$item->{'floor'} =~ s/^0*//;
		}
		if (defined $item->{'door'}) {
			$item->{'door'} =~ s/^0*//;
		}
		#if (defined $item->{'logo'}) {
		#	Il ne vaut probablement mieux pas enlever les 0 du logo
		#}
	}

	$ret = [ sort {
		   mycmp($a->{'street_nb'}, $b->{'street_nb'})
		|| mycmp($a->{'ensemble'}, $b->{'ensemble'})
		|| mycmp($a->{'building'}, $b->{'building'})
		|| mycmp($a->{'stairs'}, $b->{'stairs'})
		|| mycmp($a->{'floor'}, $b->{'floor'})
		|| mycmp($a->{'door'}, $b->{'door'})
		|| ( defined $a->{'logo'} and defined $b->{'logo'} and $a->{'logo'} cmp $b->{'logo'} )
	} @$ret ];
	return $ret;
}

sub get_nd_nd($) {
	my ( $choix ) = @_;
	my $nd = $choix->{'nd'};
	if ($choix->{'precab'}) {
		$nd = $choix->{'ptfid'};
	}
	return $nd;
}

sub get_nd_typend($) {
	my ( $choix ) = @_;
	my $typend = "actif";
	my $inactif = $choix->{'disabled'};
	my $precab = $choix->{'precab'};

	if ($inactif) {
		$typend = "inactif";
	} elsif ($precab) {
		$typend = "precab";
	}

	return $typend;
}

sub affiche_get_nd($;$) {
	my ( $choix, $admin ) = @_;
	my $label = '';
	if (!defined $admin) {
		$admin = 0;
	}

	my $nd = get_nd_nd($choix);
	my $typend = get_nd_typend($choix);
	my $inactif = $choix->{'disabled'};
	my $precab = $choix->{'precab'};

	$label .= $nd;
	if (defined $choix->{'ptfid'}) {
		$label .= " PTFID ".$choix->{'ptfid'};
	}

	if ($typend eq 'inactif') {
		$label .= " inactif";
	} elsif ($typend eq 'precab') {
		$label .= " pr-cbl";
	} else {
		$label .= " actif";
	}

	my $flags = $choix->{flags};
	if (grep (/^LR$/, @$flags)) {
		$label .= " <font color=red>(numro rouge !)</font>";
		if (!$admin) {
			# Uh ??
			error "J'ai essay de retourner un numro rouge ! J'arrtes les frais";
			exit(1);
		}
	}

	$label .= "<br/>";
	my $name = $choix->{'name'};
	if (!$admin) {
		# On vite de montrer les noms des gens...
		$name =~ s/(?!\b).(?!\b)/./g;
	}
	$label .= $name;
	$label .= "<br/>";
	my $complement = "";
	if (defined $choix->{'ensemble'}) {
		$complement .= " ensemble ".$choix->{'ensemble'};
	}
	if (defined $choix->{'building'}) {
		$complement .= " btiment ".$choix->{'building'};
	}
	if (defined $choix->{'stairs'}) {
		$complement .= " escalier ".$choix->{'stairs'};
	}
	if (defined $choix->{'floor'}) {
		$complement .= " tage ".$choix->{'floor'};
	}
	if (defined $choix->{'door'}) {
		$complement .= " porte ".$choix->{'door'};
	}
	if (defined $choix->{'logo'}) {
		$complement .= " logo ".$choix->{'logo'};
	}
	if ($complement ne "") {
		$label .= "$complement<br/>";
	}

	if (defined $choix->{'street_nb'}) {
		$choix->{'street_nb'} =~ s/^0*//;
		$label .= " ".$choix->{'street_nb'};
	}
	if (defined $choix->{'street'}) {
		$label .= " ".$choix->{'street'};
	}
	$label .= "<br/>";
	if (defined $choix->{'city'}) {
		$label .= " ".$choix->{'city'};
	}

	return $label;
}

sub get_info_nd($$$) {
	my ( $nd, $inactive, $precab ) = @_;
	my $query = "nd=$nd";
	$query .= "&inactive=$inactive";
	$query .= "&precab=$precab";
	my $info = get_reponse_get('/eligibility/info/nd', $query);
	if (!defined $info) {
                return undef;
        }
	$info->{'info'}{'street_nb'} =~ s/^0*//;
	return $info;
}

sub get_info_addr_nd($$) {
	my ( $tel, $typend ) = @_;

	$tel =~ s/^VOISIN//;
	$tel =~ s/^FTTH//;

	my $inactif = "false";
	my $precab = "false";
	if ($typend eq 'inactif') {
		$inactif = "true";
	} elsif ($typend eq 'precab') {
		$precab = "true";
	}

	my $nd_info = get_info_nd($tel,$inactif,$precab);
	if (not defined $nd_info) {
		return undef;
	}

	my $liste = get_nd($nd_info->{'info'}{'city'},
	                   $nd_info->{'info'}{'post_code'},
	                   $nd_info->{'info'}{'street'},
	                   $nd_info->{'info'}{'street_nb'});

	my $found = 0;
	for my $choix ( @$liste ) {
		my $choix_tel = $choix->{'nd'};
		my $choix_typend = "actif";
		my $choix_inactif = $choix->{'disabled'};
		my $choix_precab = $choix->{'precab'};

		if ($choix_inactif) {
			$choix_typend = "inactif";
		} elsif ($choix_precab) {
			$choix_tel = $choix->{'ptfid'};
			$choix_typend = "precab";
		}
		if ($choix_tel eq $tel && $choix_typend eq $typend) {
			if ($found) {
				error "oops, trouv le nd $tel $typend deux fois ???";
				return undef;
			} else {
				__debug("retrouv le nd $tel $typend");
				$found = 1;
			}

			if (defined $choix->{'ensemble'}) {
				$nd_info->{'info'}{'ensemble'} = $choix->{'ensemble'};
			}
			if (defined $choix->{'building'}) {
				$nd_info->{'info'}{'building'} = $choix->{'building'};
			}
			if (defined $choix->{'stairs'}) {
				$nd_info->{'info'}{'stairs'} = $choix->{'stairs'};
			}
			if (defined $choix->{'floor'}) {
				$nd_info->{'info'}{'floor'} = $choix->{'floor'};
			}
			if (defined $choix->{'door'}) {
				$nd_info->{'info'}{'door'} = $choix->{'door'};
			}
			if (defined $choix->{'logo'}) {
				$nd_info->{'info'}{'logo'} = $choix->{'logo'};
			}
		}
	}
	return $nd_info;
}

sub get_info_addr_nd_pto($$$$) {
	my ( $zip, $refimmeuble, $tel, $typend ) = @_;

	my $info_nd;
	if ($tel =~ /^(0[1-5]\d{8}|42C[A-Z][A-Z][0-9]{15})$/ ) {
		$info_nd = get_info_addr_nd($tel, $typend);
	} else {
		my $info = get_info_addr_pto($zip, $refimmeuble, $tel);
		if (! defined $info) {
			return undef;
		}
		$info_nd = {};
		$info_nd->{info} = $info;
	}
	return $info_nd;
}

sub get_addr_from_info_addr_nd($) {
	my ( $nd_info ) = @_;

	my $res = "";

	if (defined $nd_info->{'info'}{'name'}) {
		$res .= $nd_info->{'info'}{'name'}."\n";
	}

	my $detail = "";

	if (defined $nd_info->{'info'}{'ensemble'}) {
		$detail .= " ensemble ".$nd_info->{'info'}{'ensemble'};
	}
	if (defined $nd_info->{'info'}{'building'}) {
		$detail .= " btiment ".$nd_info->{'info'}{'building'};
	}
	if (defined $nd_info->{'info'}{'stairs'}) {
		$detail .= " escalier ".$nd_info->{'info'}{'stairs'};
	}
	if (defined $nd_info->{'info'}{'floor'}) {
		$detail .= " tage ".$nd_info->{'info'}{'floor'};
	}
	if (defined $nd_info->{'info'}{'door'}) {
		$detail .= " porte ".$nd_info->{'info'}{'door'};
	}
	if (defined $nd_info->{'info'}{'logo'}) {
		$detail .= " logo ".$nd_info->{'info'}{'logo'};
	}
	if ($detail ne "") {
		$res .= $detail."\n";
	}

	$res .= $nd_info->{'info'}{'street_nb'}." ".
		$nd_info->{'info'}{'street'}."\n";
	if ($nd_info->{'info'}->{'post_code'}) {
		$res .= $nd_info->{'info'}{'post_code'}." ";
	}
	$res .= $nd_info->{'info'}{'city'};

	return $res;
}

sub get_addr_nd_pto($$$$) {
	my ( $zip, $refimmeuble, $tel, $typend ) = @_;

	my $nd_info = get_info_addr_nd_pto($zip, $refimmeuble, $tel, $typend);

	return get_addr_from_info_addr_nd($nd_info);
}

sub get_addr_immeuble($$) {
	my ( $zip, $ref ) = @_;
	my $liste_villes = get_ville(undef, $zip, undef);

	# FIXME: quand l'API Liazo sera disponible, demander directement l'adresse de l'immeuble
	for my $ville ( @$liste_villes ) {
		my $immeubles = get_immeubles_liste($zip, $ville);

		for my $immeuble ( @$immeubles ) {
			if ($immeuble->{reference} eq $ref) {
				return $immeuble;
			}
		}
	}
	return undef;
}

sub get_fiber($$$$) {
	my ( $num, $rue, $zip, $ville ) = @_;
	my $post = "{".
		"\"city\":\"$ville\",".
		"\"street\":\"$rue\",".
		"\"street_nb\":\"$num\",".
		"\"post_code\":\"$zip\"".
	"}";
	return get_reponse_post('/eligibility/send/fiber', $post);
}

sub get_sdsl($$$) {
	my ( $num, $inactive, $precab ) = @_;
	my $post = "{".
		"\"phone_number\":\"$num\",".
		"\"inactive\":$inactive,".
		"\"precab\":$precab".
	"}";
	return get_reponse_post('/eligibility/send/sdsl', $post);
}

sub get_adsl($$$) {
	my ( $num, $inactive, $precab ) = @_;
	my $post = "{".
		"\"phone_number\":\"$num\",".
		"\"inactive\":$inactive,".
		"\"precab\":$precab".
	"}";
	return get_reponse_post('/eligibility/send/adsl', $post);
}

sub get_ftth(;$$$) {
	my ( $num, $inactive, $ref_building ) = @_;
	my $post = "{";
	my $comma = "";
	if (defined $num) {
		$post .= "$comma\"phone_number\":\"$num\"";
		$comma = ",";
	}
	if (defined $inactive) {
		$post .= "$comma\"inactive\":$inactive";
		$comma = ",";
	}
	if (defined $ref_building) {
		$post .= "$comma\"reference_building\":\"$ref_building\"";
		$comma = ",";
	}
	$post .= "}";
	return get_reponse_post('/eligibility/send/ftth', $post);
}

# types:
# 0: global
# 1: DSLAM simple
# 2: DSLAM BitStream
# 3: DSLAM BAS
# 4: test de ligne
sub test_adsl($$) {
	my ( $ref, $type ) = @_;
	my %params = ();
	$params{'reference'} = $ref;
	$params{'test_type'} = $type;
	my $post = encode_json(\%params);
	return get_reponse_post('/service/test/adsl', $post);
}

sub list_tests_adsl() {
	return get_reponse_get('/service/list/test/adsl', '');
}

sub reset_adsl($) {
	my ( $ref ) = @_;
	my %params = ();
	$params{'reference'} = $ref;
	my $post = encode_json(\%params);
	return get_reponse_post('/service/reset/adsl', $post);
}

sub get_contracts(;$) {
	my ( $state ) = @_;
	my $get = '';
	if ( defined($state) ) {
		$get = "last_state=$state";
	}
	my $ret = get_reponse_get('/contract/list', $get);
	for (my $i = 2; $i <= $ret->{num_pages}; $i++) {
		my $get2 = $get."&page=$i";
		my $ret2 = get_reponse_get('/contract/list', $get2);
		push @{$ret->{objs}}, @{$ret2->{objs}};
	}
	return $ret;
}

sub get_contract($) {
	my ( $ref ) = @_;
	my $get = "reference=$ref";
	return get_reponse_get('/contract/get', $get);
}

sub get_services() {
	return get_reponse_get('/service/list', '');
}

sub get_service($) {
	my ( $ref ) = @_;
	my $get = "reference=$ref";
	return get_reponse_get('/service/get', $get);
}

# la rf d'un service pour une rf de commande donne
sub get_commande_serviceref($) {
	# Rfrence de commande
	my ( $ref ) = @_;
	my $commande = get_info_commande($ref);
	return $commande->{'service'};
}

# la ref d'un contrat pour une rf de commande donne
sub get_commande_contractref($) {
	# Rfrence de commande
	my ( $ref ) = @_;
	my $commande = get_info_commande($ref);
	return $commande->{'contract'};
}

sub get_adsl_delivery_nd($) {
	# Rfrence de commande
	my ( $ref ) = @_;
	my $service = get_service(get_commande_serviceref($ref));
	return $service->{'resources'}->{'ADSL/VDSL'}->{'ndi_delivery'};
}

sub get_collecte() {
	return get_reponse_get('/service/list/collect', '');
}

my %product_code_par_debit = (
	"128k" => "160k.96k.dom",
	"512k" => "0.5m.160k",
	"512RE" => "0.5m.160k.re",
	"1M" => "1m.max",
	"2M" => "2m.max",
	"8M" => "10m.max",
	"18M" => "25m.max",
	"80M" => "100m.max",
	"300MK1" => "300m.250m.k1",
	"1GK1" => "1000m.250m.k1",
	"300MK2" => "300m.250m.k2",
	"1GK2" => "1000m.250m.k2",
	"1GB1" => "1000m.500m.yt1",
	"1GB2" => "1000m.500m.yt2"
);

# Usage interne: convertit une paire dbit/option en code Liazo
sub product_code($$) {
	my ($debit, $option) = @_;
	my $pcode = $product_code_par_debit{$debit};

	if ($option eq "LiazoKosc") {
		return "kosc.ftth.$pcode.local";
	} elsif ($option eq "LiazoBouygues") {
		return "bouygues.ftth.$pcode.local";
	} else {
		my $degroupage;
		my $dsl;
		if ($debit eq "80M") {
			$dsl = "vdsl";
		} else {
			$dsl = "adsl";
		}
		if ($option eq "LiazoOrangeNu") {
			$degroupage = "total";
		} else {
			$degroupage = "partial";
		}
		return "ft.cuivre.$dsl.$degroupage.monovc.$pcode.bpincluded";
	}
}

# Commander une ligne, version complte
sub commande($$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$) {
	my (
	     $product,
	     $option,
	     $ndi,
	     $inactive,
	     $precab,
	     $refimmeuble,

	     # contact commande
		$contact_name,
		$contact_phone_number, $contact_mobile_phone_number,
		$contact_email,

	     # install
		# contact sur place
		$install_phone_number,
		$install_post_code,
		$install_street_nb,
		$install_street_code, # optionnel
		$install_street, $install_street_complement,

		# optionnels
		$install_ensemble, $install_building,
		$install_stairs, $install_floor,
		$install_door,
		$install_logo,

		$install_city,
		$install_insee,

		$install_siret,
		$install_name_place,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Contact pour l'installation
		$install_contact_name,
		$install_contact_phone_number, $install_contact_mobile_phone_number,
		$install_contact_email,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$client_reference, $client_comment,

		$test
	) = @_;

	my %params = ();
	$params{'product'} = $product;
	$params{'option'} = $option;

	if ($product =~ /kosc.ftth/ or $product =~ /bouygues.ftth/) {
		my $typend;
		if ($inactive eq "true") {
			$typend = "inactif";
		} else {
			$typend = "actif";
		}

		$params{'reference_building'} = $refimmeuble;
		$params{'commitment_month'} = 12;
	} else {
		if (length($ndi) != 10) {
			error "Commande de $ndi non triviale, Orange ne saura pas faire, il faut commander  la main";
			return undef;
		}
		$params{'ndi'} = $ndi;
		$params{'inactive'} = doJSONbool($inactive);
		$params{'precab'} = doJSONbool($precab);
	}

	my %contact = ();
	$contact{'name'} = $contact_name;
	$contact{'phone_number'} = $contact_phone_number;
	$contact{'mobile_phone_number'} = $contact_mobile_phone_number;
	$contact{'email'} = $contact_email;
	$params{'contact'} = \%contact;

	my %install = ();
	$install{'phone_number'} = $install_phone_number;
	if ($product =~ /kosc.ftth/ or $product =~ /bouygues.ftth/) {
		$install{'post_code'} = $install_post_code;
		$install{'street_nb'} = $install_street_nb;
		if (defined $install_street_code) {
			$install{'street_code'} = $install_street_code;
		}
		$install{'street'} = $install_street;
		if (defined $install_street_complement) {
			$install{'street_complement'} = $install_street_complement;
		}
	}
	if (defined $install_ensemble and $install_ensemble ne 'INVALID') {
		$install{'ensemble'} = $install_ensemble;
	}
	if (defined $install_building and $install_building ne 'INVALID') {
		$install{'building'} = $install_building;
	}
	if (defined $install_stairs and $install_stairs ne 'INVALID') {
		$install{'stairs'} = $install_stairs;
	}
	if (defined $install_floor and $install_floor ne 'INVALID') {
		$install{'floor'} = $install_floor;
	}
	if (defined $install_door and $install_door ne 'INVALID') {
		$install{'door'} = $install_door;
	}
	if ($product =~ /kosc.ftth/ or $product =~ /bouygues.ftth/) {
		if (defined $install_logo and $install_logo ne 'INVALID') {
			$install{'logo'} = $install_logo;
		}
		$install{'city'} = $install_city;
		$install{'insee'} = $install_insee;
	}

	$install{'siret'} = $install_siret;
	$install{'name_place'} = $conv->convert($install_name_place);
	# Orange n'aime pas les lettres pas si courantes
	$install{'name_place'} =~ s/ & / et /g;
	$install{'name_place'} =~ s/&/ et /g;
	$install{'name_place'} =~ s/[^-'a-zA-Z0-9 ]/ /g;
	if ($product =~ /kosc.ftth/ or $product =~ /bouygues.ftth/) {
		$install{'date_available'} = $install_date_available;
		$install{'date_desired_available'} = $install_date_desired_available;
	}
	$install{'contact_name'} = $install_contact_name;
	$install{'contact_phone_number'} = $install_contact_phone_number;
	$install{'contact_mobile_phone_number'} = $install_contact_mobile_phone_number;
	$install{'contact_email'} = $install_contact_email;

	if ($product =~ /kosc.ftth/ or $product =~ /bouygues.ftth/) {
		if ($ndi !~ /0[1-5]\d{8}/) {
			# Livraison par crasement
			$install{'reference_pto'} = $ndi;
		}
	} else {
		# pour l'ADSL seulement
		$install{'rdv_reference'} = $install_rdv_reference;
		$install{'rdv_date'} = $install_rdv_date;
		$install{'rdv_range'} = $install_rdv_range;
	}

	$params{'install'} = \%install;

	my %client = ();
	$client{'reference'} = $client_reference;
	$client{'comment'} = $client_comment;
	$params{'client'} = \%client;

	# pour le SDSL seulement
	#my %collect = ();
	#$collect{'reference'} = $collect_reference;
	#$collect{'vlan_range'} = $client_vlan_range;
	#$collect{'vlan'} = $client_vlan;
	#$params{'collect'} = %collect;

	my $post = encode_json(\%params);

	# resp. /order/send/sdsl
	__debug("commande $post");
	if ($test) {
		return "PR1234-1234";
	} else {
		my $result;
		if ($product =~ /kosc.ftth/ or $product =~ /bouygues.ftth/) {
			$result = get_reponse_post('/order/send/ftth', $post, 1);
		} else {
			$result = get_reponse_post('/order/send/adsl', $post, 1);
		}
		__debug("Liazo a rpondu");
		if (defined($result->{'reference'})) {
			__debug("La reference obtenue est ".$result->{'reference'});
		} else {
			__debug("Erreur Liazo...");
		}
		return $result->{'reference'};
	}
}

# Commander une ligne, version ne demandant que les informations dont on dispose chez FDN
sub do_commande_liazo($$$$$$$$$$$$$$$$$$$$$$$$$) {
	my (
	     $debit,
	     $option_degroupage,
	     $ndi,
	     $inactive,
	     $precab,
	     $refimmeuble,

	     # install
		# contact sur place
		$install_name_place,
		$install_phone_number,
		$adresse,

		# optionnels
		$install_ensemble, $install_building,
		$install_stairs, $install_floor,
		$install_door,
		$install_logo,

		$install_post_code,
		$install_city,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid,

		$validateur,

		$test
	) = @_;

	my $product = product_code($debit, $option_degroupage);
	my $options_liazo = [];
        my $suivi_tel = Config_Key($conf,'ariane','suivi_tel');
	if ($validateur) {
		my $tel = Config_Key('adsl/global.cfg',$validateur,'suivi_tel');
		if ($tel) {
			$suivi_tel = $tel;
		}
	}
	my $mailsuivi = Config_Key('adsl/global.cfg','mail','suivi');

	my ( $install_street_nb, $install_street );
	if ( defined($adresse) ) {
		$adresse =~ /^(\w*) *(.*)$/;
		($install_street_nb, $install_street) = ($1, $2);
	}

	# Pour tester
	# return "PR1705-31054";

	return commande(
		$product,
		$options_liazo,
		$ndi,
		$inactive,
		$precab,
		$refimmeuble,

	     # contact commande, TODO
		'FDN',
		$suivi_tel, $suivi_tel,
		'buro@fdn.fr',

	     # install
		# contact sur place
		$install_phone_number,
		$install_post_code,
		$install_street_nb,
		undef, # street_code, optionnel
		$install_street, '',

		# prcisions adresse optionnelles
		$install_ensemble, $install_building,
		$install_stairs, $install_floor,
		$install_door,
		$install_logo,


		$install_city,
		'',

		'419 897 855 00047',
		$install_name_place,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Contact pour l'installation, TODO
		$install_name_place,
		$install_phone_number, undef,
		$mailsuivi,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		"FDN-".$lid, undef,

		$test
	);
}
sub commande_liazo($$$$$$$$$$$$$$$$$$$$$$$$) {
	my (
	     $debit,
	     $option_degroupage,
	     $ndi,
	     $typend,
	     $refimmeuble,

	     # install
		# contact sur place
		$install_name_place,
		$install_phone_number,
		$adresse,

		# prcisions adresse optionnelles
		$install_ensemble, $install_building,
		$install_stairs, $install_floor,
		$install_door,
		$install_logo,

		$install_post_code,
		$install_city,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid,

		$validateur,

		$test
	) = @_;

	my ( $inactive, $precab, $err, @info ) = get_eligibilite($ndi, $typend, $refimmeuble);
	if (!defined $test && defined $err) {
		debug("oops, erreur $err avant mme la commande\n");
		return undef;
	}

	do_commande_liazo(
	     $debit,
	     $option_degroupage,
	     $ndi,
	     $inactive,
	     $precab,
	     $refimmeuble,

	     # install
		# contact sur place
		$install_name_place,
		$install_phone_number,
		$adresse,

		# prcisions adresse optionnelles
		$install_ensemble, $install_building,
		$install_stairs, $install_floor,
		$install_door,
		$install_logo,

		$install_post_code,
		$install_city,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid,

		$validateur,

		$test
	);
}

# Rcuprer la liste des commandes
sub get_commandes($$) {
	my ( $start, $end ) = @_;
	my $get = "start=$start&end=$end";
	return get_reponse_get('/order/list', $get);
}

# Rcuprer les infos sur une commande
sub get_info_commande($) {
	my ( $ref ) = @_;
	my $get = "reference=$ref";
	return get_reponse_get('/order/get', $get);
}

sub get_deploiement_commande($) {
	my ( $deplid ) = @_;
	my $depl = deploiement_par_id($deplid);
	my $tid = $depl->{TASK_ID};
	workflow_select_db('adsl');
	my $props = get_task_properties($tid);
	return $props->{ordretd};
}

sub get_ligne_commande($) {
	my ( $lid ) = @_;
	my $ligne = ligne_par_id($lid);
	my @depls = deploiement_par_ligne_id($lid);
	@depls = sort { $a->{DEPLOIEMENT_ID} <=> $b->{DEPLOIEMENT_ID} } @depls;
	my $depl = $depls[$#depls];
	return get_deploiement_commande($depl->{DEPLOIEMENT_ID});
}

sub annuler_commande($) {
	my ( $ref ) = @_;
	my $post = "{\"reference\":\"$ref\"}";
	return get_reponse_post('/order/cancel', $post, 1);
}

# Appele de temps en temps par cron pour regarder o en sont les commandes
sub poll() {
	#__debug "commandes nouvelles";
	my $tmp = sql_transaction($dbh, 'select', 'select ORDRETD_ID from ORDRETD where ORDRETD_ETAT="new" and ORDRETD_LIGNE like "PR%"');
	my @list = FDN::Sql::Basic::__return_all($tmp);
	RES:
	for my $res ( @list ) {
		my $oid = $res->{'ORDRETD_ID'};
		my $ordre = ordretd_par_id($oid);
		my $ref = $ordre->{'ORDRETD_LIGNE'};
		my $info = get_info_commande($ref);

		for my $status ( @{$info->{'status'}} ) {
			if ($status->{'status'} == 2 || $status->{'status'} == 4 || $status->{'status'} == 6) {
				# Commande confirme par Liazo, en cours de livraison, voire dj livre
				__debug "commande $oid $ref valide";
				mark_sent( undef, ($ordre) );
				event_sent( ($ordre) );
				# Si elle est dj livre elle ne risque plus d'chouer
				next RES if $status->{'status'} == 6;
			}
		}
		for my $status ( @{$info->{'status'}} ) {
			if ($status->{'status'} == 3 || $status->{'status'} == 10) {
				my $tel = $info->{info}->{product_phone_number};
				my $lid = $info->{info}->{client_reference};
				$lid =~ s/^FDN-//;

				# Commande rejete ou annule par Liazo ou Orange
				workflow_select_db('adsl');
				my @tasks_liazo = task_par_name_state('cdeliazo', 'running');
				foreach my $task ( @tasks_liazo ) {
					my $props = get_task_properties($task->{'TASK_ID'});
					if ($props->{'ordretd-id'} eq $oid) {
						__debug("je vais faire chouer la tche $task->{'TASK_ID'}");
						set_task_state($task->{'TASK_ID'},'failed');
					}
				}

				__debug "commande $oid $ref rejete";
				new_info_ligne($lid, 'inline', 'Rejet commande', 'La commande a ete rejetee lors de la validation');
				set_ordretd_etat($oid, 'ko');
				envoie_mail(undef,'bureau-rejetar',{
						'telephone' => $tel,
						'lid' => $lid,
						'ordretd-id' => $oid,
						'ordretd' => $ref,
					});
			}
		}
	}

	#__debug "commandes envoyes";
	$tmp = sql_transaction($dbh, 'select', 'select ORDRETD_ID from ORDRETD where ORDRETD_ETAT="sent" and ORDRETD_LIGNE like "PR%"');
	@list = FDN::Sql::Basic::__return_all($tmp);
	RES2:
	for my $res ( @list ) {
		my $oid = $res->{'ORDRETD_ID'};
		my $ordre = ordretd_par_id($oid);
		my $ref = $ordre->{'ORDRETD_LIGNE'};
		my $info = get_info_commande($ref);

		for my $status ( @{$info->{'status'}} ) {
			if ($status->{'status'} == 6) {
				# Commande livre par Liazo
				__debug "commande $oid $ref livre";
				set_ordretd_etat($oid, 'ok');
				event_ok( ($ordre) );
				# Ok, fini !
				next RES2;
			}
		}
		for my $status ( @{$info->{'status'}} ) {
			if ($status->{'status'} == 3 || $status->{'status'} == 10) {
				my $tel = $info->{info}->{product_phone_number};
				my $lid = $info->{info}->{client_reference};
				$lid =~ s/^FDN-//;

				# Commande rejete ou annule par Liazo ou Orange
				workflow_select_db('adsl');
				my @tasks_liazo = task_par_name_state('wait_ok_liazo', 'running');
				foreach my $task ( @tasks_liazo ) {
					my $props = get_task_properties($task->{'TASK_ID'});
					if ($props->{'ordretd-id'} eq $oid) {
						__debug("je vais faire chouer la tche $task->{'TASK_ID'}");
						set_task_state($task->{'TASK_ID'},'failed');
					}
				}

				__debug "commande $oid $ref rejete";
				new_info_ligne($lid, 'inline', 'Rejet commande', 'La commande a ete rejetee lors de la livraison');
				set_ordretd_etat($oid, 'ko');
				envoie_mail(undef,'bureau-rejetar',{
						'telephone' => $tel,
						'lid' => $lid,
						'ordretd-id' => $oid,
						'ordretd' => $ref,
					});
			}
		}
	}
}

# Appele de temps en temps par cron pour regarder les contrats rsilis
sub poll_resil() {
	my ( $state ) = @_;
	my $contracts = get_contracts(6);

	for my $c ( @{$contracts->{objs}} ) {
		my $ref = $c->{order__info}->{client_reference};
		next if (!defined $ref);

		next if ($ref !~ /^FDN-(\d*)$/);

		my $lid = $1;
		my $ligne = ligne_par_id($lid);
		next if (!defined $ligne);

		next if ($ligne->{LIGNE_STATE} ne 'ok');

		my $service = $c->{service};
		my $s = get_service($service);
		next if (!defined $s);

		next if (!defined $s->{last_contract}->{reference} || $s->{last_contract}->{reference} ne $c->{reference});

		error "oops, la ligne $lid est ok mais le contrat $c->{reference} est rsili depuis le $c->{end}";
		envoie_mail(undef,'bureau-resil-liazo',{
				'lid' => $lid,
				'ct' => $c->{reference},
				'end' => $c->{end},
			});
	}
}

# Usage interne: rcuprer la base des urls de l'API Liazo
sub get_url() {
	my $protocol = Config_Key($conf,'ariane','protocol');
	my $url      = Config_Key($conf,'ariane','url');
	my $cgi      = Config_Key($conf,'ariane','cgi');
	$url = $protocol."://".$url.$cgi;
	return $url;
}

# Usage interne: rcuprer une rponse http et la dcoder.
sub get_reponse($) {
	my ( $request ) = @_;
	my $ua = LWP::UserAgent->new;
	$ua->timeout(60);
	my $response = $ua->request($request);
	if ( $response->message =~ /timeout/ ) {
		error "Le serveur Liazo n'a pas repondu assez vite.";
		return undef;
	}
	__debug "Reponse HTTP: ".$response->message;
	__debug "Reponse JSON ".$response->content;
	my $res;
	my $worked = eval '$res = decode_json($response->content);';
	if ( !defined($worked)) {
		error "Le serveur Liazo n'a pas rpondu au JSON: ".$@;
		return undef;
	}
	if ( $res->{'error'}{'code'} != 0 ) {
		error "Le serveur Liazo a rpondu ".$res->{'error'}{'msg'};
		return undef;
	}
	if ( !defined($res->{'result'}) ) {
		error "Le serveur Liazo n'a rpondu de section result ";
		return undef;
	}
	return $res->{'result'};
}

# Poster une requte POST  Liazo
sub get_reponse_post($$;$) {
	my ( $cgi, $json, $dosign ) = @_;
	my $url = get_url().$cgi;
	__debug "URL complete: $url";
	__debug "JSON complete: $json";
	my $header = new HTTP::Headers;
	my $key    = Config_Key($conf,'ariane','api_key');
	$header->header('Content-Type' => 'text/plain; charset=utf-8');
	$header->header('X-LAPI-KEY' => $key);
	if ($dosign) {
		my $sign   = sign($json);
		$header->header('X-LAPI-SIGN' => $sign);
	}
	my $request = new HTTP::Request ('POST', $url, $header );
	$request->content($json);
	return get_reponse($request);
}

# Poster une requte GET  Liazo
sub get_reponse_get($) {
	my ( $cgi, $get ) = @_;
	my $url = get_url().$cgi;
	$url = $url.'?'.$get;
	__debug "URL complete: $url";
	my $header = new HTTP::Headers;
	my $key    = Config_Key($conf,'ariane','api_key');
	$header->header('X-LAPI-KEY' => $key);
	my $request = new HTTP::Request ('GET', $url, $header );
	return get_reponse($request);
}

1;
