package FDN::Adsl::Souscription;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		$offres
		$options
		preparation_formulaire_auto
		read_config
		tout_faire_la_ligne
		souscription_fdn
		souscription_vpn
		creation_vpn
		creation_deploiement_vpn
		creation_tache_vpn
		adhesion_fdn

		creation_prelevement_iban
		);
our @EXPORT_OK = qw (creation_deploiement_ligne);

use Carp;

use FDN::Adsl::Basic;
use FDN::Adsl::Infos;
use FDN::Adsl::Utils;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Sql::Basic;
use POSIX qw(strftime);

# Doit rester par ordre de prfrence croissante pour une option donne
our $offres = [
'1GB1LiazoBouygues',
'1GB2LiazoBouygues',
'1GK1LiazoKosc',
'1GK2LiazoKosc',
'300MK1LiazoKosc',
'300MK2LiazoKosc',
'80MLiazoOrangeNu',
'18MLiazoOrangeNu',
'8MLiazoOrangeNu',
'2MLiazoOrangeNu',
'1MLiazoOrangeNu',
'512RELiazoOrangeNu',
'512kLiazoOrangeNu',
#'128kLiazoOrangeNu',
'80MLiazoOrange',
'18MLiazoOrange',
'8MLiazoOrange',
'2MLiazoOrange',
'1MLiazoOrange',
'512RELiazoOrange',
'512kLiazoOrange',
#'128kLiazoOrange',
'18Mopt1',
'8Mopt1',
'2Mopt1',
'1Mopt1',
'512kopt1',
#'6Mopt3',
#'5Mopt3',
#'4Mopt3',
#'3Mopt3',
#'2Mopt3',
#'1Mopt3',
#'512kopt3',
#'18Mopt5',
#'1Mopt5',
#'512REopt5',
#'512kopt5',
'18Mopt0',
'1Mopt0'
];

our $options = {
'opt0' => 'option0',
'opt1' => 'option1',
'opt3' => 'option3',
'opt5' => 'option5',
'LiazoOrange' => 'LiazoOrange',
'LiazoOrangeNu' => 'LiazoOrangeNu',
'LiazoKosc' => 'LiazoKosc',
'LiazoBouygues' => 'LiazoBouygues'
};

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $debug;
}

my $auto;

sub hard_failure($) {
	my ( $texte ) = @_;
	$auto = '' if ! defined $auto;
	if ( $auto eq 'yes' ) {
		eject_page_text($texte);
	} elsif ( $auto eq 'fdn' ) {
		return;
	} else {
		my $page;
		if ( $texte =~ /^OK (.*)$/ ) {
			$page = "<p>La commande est bien prise en compte. ".
				"Le numero de suivi est $1.</p>";
		} elsif ( $texte =~ /^REDO (.*)$/ ) {
			$page = "<p>Vous avez deja passe cette commande! ".
				"Son numero de suivi est $1.</p>";
		} else {
			$page = "<div class=\"error\">".
				"La commande a echoue<br>".
				"Message d'erreur:<br>".
				"<pre>$texte</pre>".
				"</div>";
		}
		eject_page($page);
	}
}

sub deja_client($$$) {
	my ( $adh, $tel, $nom ) = @_;
	my $cl = client_par_numadh($adh);
	my $clid;
	if ( defined $cl ) {
		$clid = $cl->{'CLIENT_ID'};
		new_info_client($clid,'inline','Reprise client existant',
			"Durant une souscription sur le web j'ai repris un client\n".
			"existant. Soit un reload sur la derniere page, soir une\n".
			"ligne pour quelqu'un qui est deja adherent.\n".
			"J'ai retrouve le client par son numero d'adherent.");
	} else {
		my $li = ligne_par_tel($tel);
		if ( defined $li ) {
			$clid = $li->{'CLIENT_ID'};
			my $cl = client_par_id($clid);
			if ($cl->{'CLIENT_NOM'} eq $nom) {
				new_info_client($clid,'inline','Reprise client existant',
					"C'est normalement le fruit d'un reload sur la derniere\n".
					"page d'une souscription.");
			} else {
				$clid = undef;
				info "Pour la ligne $li on a dj un client $clid, mais a ne correspond pas au nom $nom";
			}
		}
	}
	return $clid;
}

sub deja_ligne($) { # Test
	my ( $tel ) = @_;
	my $ligne = ligne_par_tel($tel);
	my $lid;
	if ( defined $ligne ) {
		$lid = $ligne->{'LIGNE_ID'};
		new_info_ligne($lid,'inline','Reprise ligne existante',
			"Durant une souscription sur le web j'ai repris une ligne\n".
			"existante. J'ai retrouve la ligne d'apres son numero de\n".
			"telephone. C'est normalement le fruit d'un reload sur la\n".
			"derniere page.");
	}
	return $lid;
}

sub deja_deploiement($) { # Tests
	my ( $lid ) = @_;
	my $dpl = deploiement_par_ligne_id($lid);
	my $did;
	if ( defined $dpl ) {
		$did = $dpl->{'DEPLOIEMENT_ID'};
		new_info_ligne($lid,'inline','Reprise deploiement existant',
			"Durant une souscription sur le web j'ai repris un deploiement\n".
			"existant. J'ai retrouve ce deploiement d'apres le numero de\n".
			"la ligne ($lid). C'est normalement le fruit d'un reload sur la\n".
			"derniere page.");
	}
	return $did;
}

sub deja_tache($$$) { # Tests
	my ( $did, $lid, $tel ) = @_;
	my $dpl = deploiement_par_id($did);
	my $tid;
	if ( defined $dpl ) {
		$tid = $dpl->{'TASK_ID'};
		if ( $tid != -1 ) {
			new_info_ligne($lid,'inline','Reprise d\'une tache de deploiement',
				"Durant une souscription sur le web j'ai repris une tache\n".
				"de deploiement. J'ai retrouve cette tache parce qu'elle\n".
				"est associee au deploiement lui-meme (que j'ai surement\n".
				"repris aussi. C'est normalement le fruit d'un reload sur\n".
				"la derniere page."
				);
		} else {
			$tid = undef;
		}
	}
	conduse('FDN::Sql::Workflow::Basic');
	workflow_select_db('adsl');
	my $tsk = task_par_key('D/'.$tel);
	if ( !$tid && defined $tsk ) {
		$tid = $tsk->{'TASK_ID'};
		my $props = get_task_properties($tid);
		if ( defined $props->{ligneid} ) {
			my $lid = $props->{ligneid};
			my $lig = ligne_par_id($lid);
			if ( defined $lig ) {
				if ( $lig->{'LIGNE_TEL'} =~ /$tel\.r/ ) {
					# C'est un dmnagement en conservant le numro !
					return undef;
				}
			}
		}
		new_info_ligne($lid,'inline','Reprise d\'une tache de deploiement',
			"Durant une souscription sur le web j'ai repris une tache de\n".
			"deploiement. C'EST PARFAITEMENT ANORMAL.\n".
			"J'ai retrouve cette tache par son identifiant unique:\n".
			"D/$tel, qui colle avec le numero de telephone de la nouvelle\n".
			"souscription.\n".
			"C'EST PROBABLEMENT UNE TENTATIVE DE PIRATAGE (OU UN BUG).");
	}
	return $tid;
}

sub nouvelle_ligne($$) { # Tests
	my ( $clid, $tel ) = @_;
	my $ligne = ligne_par_tel($tel);
	if ( defined ($ligne) ) {
		my $lid = $ligne->{'LIGNE_ID'};
		my $cid = $ligne->{'CLIENT_ID'};
		if ( $cid == $clid ) {
			my $dpl = deploiement_par_ligne_id($lid);
			my $num = $dpl->{'DEPLOIEMENT_NUM'};
			hard_failure("REDO $num");
			new_info_ligne($lid,'inline','Re-commande de la ligne',
				"Le souscripteur ($clid) a re-commande cette ligne\n".
				"Pas inquietant.");
		} else {
			hard_failure("ERR Ligne deja connue chez FDN");
			new_info_ligne($lid,'inline','Vol de ligne',
				"Le souscripteur ($clid) a essaye de piquer une ligne\n".
				"qui ne lui appartient pas.");
			new_info_client($cid,'inline','Vol de ligne',
				"Le souscripteur $clid a essaue de piquer la ligne $lid.\n".
				"Pas normal.");
			new_info_client($clid,'inline','Vol de ligne (VOLEUR!)',
				"Ce client a essaye de piquer la ligne $lid, qui appartient\n".
				"en vrai a $cid.");
		}
		return 0;
	}
	return 1;
}

sub verifie_images($$$$) {
	my ( $path, $option, $ft, $td ) = @_;
	if ( ! -f $path.'/jpeg/'.$ft.'.jpg' ||
	     ! -f $path.'/thumbnails/tn-'.$ft.'.jpg' ) {
		hard_failure("ERR La facture FT est introuvable");
		return 0;
	}
	if ( $option eq 'option1' && (
	     ! -f $path.'/jpeg/'.$td.'.jpg' ||
	     ! -f $path.'/thumbnails/tn-'.$td.'.jpg' )
	     ) {
		hard_failure("ERR Le mandat de degroupage est introuvable");
		return 0;
	}
	return 1;
}

sub verifie_mandat($$) {
	my ( $subs, $num ) = @_;
	my $mandat = mandat_par_numero($num);
	if ( !defined $mandat ) {
		hard_failure("ERR Ce mandat n'existe pas");
		return undef;
	}
	my $mid = $mandat->{'MANDAT_ID'};
	my $det = $mandat->{'MANDAT_DETENTEUR'};
	if ( $det ne $subs ) {
		hard_failure('ERR Ce mandat ne vous appartient pas');
		return undef;
	} else {
		return $mid;
	}
}

sub creation_client($$$$$$$$) { # Test
	my ( $nom, $adr, $cp, $vil, $mail, $pgp, $tel, $tarif ) = @_;
	my $aid = new_adresse( $nom, $adr, $cp, $vil );
	if ( $mail ) {
		set_adresse_mail($aid,$mail);
	}
	if ( $pgp ) {
		set_adresse_pgp($aid,$pgp);
	}
	if ( $tel ) {
		set_adresse_tel($aid,$tel);
	}
	my $clid = new_client($nom,$tarif);
	set_client_adresse($clid,$aid);
	return $clid;
}

sub creation_prelevement_rib($$$$$$$$$$) { # Test
	my ( $clid, $bq, $gu, $cte, $rib, $nom, $nombq, $adrbq, $cpbq, $vilbq ) = @_;
	if ( $bq && $gu && $cte && $rib ) {
		info("Creation du prelevemt");
		my $pid = new_prelevemt($clid,$bq,$gu,$cte,$rib,$nom,$nombq);
		my $aid = new_adresse($nombq,$adrbq,$cpbq,$vilbq);
		set_prelevemt_adresse($pid,$aid);
		return $pid;
	}
	return 0;
}

sub creation_prelevement_iban($$$$$) { # Test
	my ( $clid, $bic, $iban, $nom, $nombq ) = @_;
	if ( $bic && $iban ) {
		my $pid = new_prelevemt_iban($clid, $bic, $iban, $nom, $nombq);
		my $rum = 'FDN-001-'.$clid.'-'.$pid;
		set_prelevemt_mandat($pid,$rum);
		set_prelevemt_datemandat($pid,'now');
		return $pid;
	}
	return 0;
}

sub creation_ligne($$$$$$$$$$$$) { # Tests
	my ( $nom, $adr, $cp, $ville, $mail, $pgp, $tel, $telcontact, $debit, $option, $prix, $clid ) = @_;
	info("Creation de la ligne");
	my $aid = new_adresse(
		$nom,
		$adr,
		$cp,
		$ville);
	if ( ! $aid ) {
		hard_failure("ERR Echec de creation de l'adresse. Alerter FDN.");
		return undef;
	}
	if ( $mail ) {
		set_adresse_mail($aid,$mail);
	}
	if ( $pgp ) {
		set_adresse_pgp($aid,$pgp);
	}
	set_adresse_tel($aid,$telcontact);
	my $lid = new_ligne(
		$tel,
		$debit,
		$option,
		$prix,
		$clid,
		$aid);
	if ( ! $lid ) {
		hard_failure("ERR Echec de creation de la ligne. Alerter FDN.");
		return undef;
	}
	if ($tel eq "NOUVEAU") {
		set_ligne_tel($lid, "NOUVEAU-$lid");
	}
	new_info_ligne($lid,'inline','Creation de la ligne',
		'Creation de la ligne par souscription en ligne sur le site web');
	return $lid;
}

sub creation_vpn($$$$$$$$) {
	my ( $nom, $adr, $cp, $ville, $mail, $pgp, $prix, $clid ) = @_;
	info "Creation du vpn";
	my $aid = new_adresse(
		$nom,
		$adr,
		$cp,
		$ville);
	if ( ! $aid ) {
		hard_failure("ERR Echec de creation de l'adresse. Alerter FDN.");
		return undef;
	}
	if ( $mail ) {
		set_adresse_mail($aid,$mail);
	}
	if ( $pgp ) {
		set_adresse_pgp($aid,$pgp);
	}
	my $vpnid = new_vpn($prix, $clid, $aid);
	set_vpn_dtin($vpnid);
	return $vpnid;
}

sub __creation_deploiement($) { # Tests
	my ( $links ) = @_;
	my $dplnum;
	my $deploiement;
	info("Creation du deploiement");
	do {
		$dplnum = clef_aleatoire(10);
		$deploiement = deploiement_par_num($dplnum);
	} while ( defined $deploiement );
	my $dplid = new_deploiement($dplnum);
	if ( ! $dplid ) {
		hard_failure("ERR Echec de la creation du deploiement. Alerter FDN.");
		return undef;
	}
	if ( $links->{'ligne'} ) {
		set_deploiement_ligne_id($dplid,$links->{'ligne'});
	}
	if ( $links->{'vpn'} ) {
		set_deploiement_vpn_id($dplid,$links->{'vpn'});
	}
	return $dplid;
}

sub creation_deploiement_ligne($;$) { # Tests
	my ( $lid, $mid ) = @_;
	my $dplid = __creation_deploiement({'ligne' => $lid});
	return undef if ! $dplid;
	if ( defined $mid ) {
		set_mandat_deploiement_id($mid,$dplid);
	}
	new_info_ligne($lid,'inline','Creation du deploiement',
		"Creation du deploiement numero $dplid pour\n".
		"cette ligne.");
	return $dplid;
}

sub creation_deploiement_vpn($) {
	my ( $vpnid ) = @_;
	my $dplid = __creation_deploiement({'vpn' => $vpnid});
	new_info_vpn($vpnid, 'inline', 'Creation du deploiement',
		"Creation du deploiement numero $dplid pour\n".
		"ce vpn.");
	return $dplid;
}

sub creation_tache_vpn($$$) {
	my ( $nom, $vpnid, $dplid ) = @_;
	conduse('FDN::Sql::Workflow::Run');
	workflow_select_db('adsl');
	workflow_select_config('adsl/wf.cfg');
	info("Creation de la tache de deploiement");
	my $task_vars = {
		'nom' => $nom,
		'vpnid' => $vpnid,
		'deplid' => $dplid
	};
	my $tid = launch_task('depl-vpn',$task_vars);
	if ( ! $tid ) {
		new_info_vpn($vpnid,'inline','Echec lancement de la tache',
			"Echec du lancement de la tache de deploiement de\n".
			"ce VPN.");
		hard_failure("ERR Echec de lancement de la tache de creation du VPN. Alerter FDN.");
		return undef;
	}
	set_deploiement_task_id($dplid,$tid);
	new_info_vpn($vpnid,'inline','Creation de la tache de deploiement',
		"Creation de la tache de deploiement du VPN, numero $tid");
	return $tid;
}

sub creation_tache_adsl($$$$$$$$$$$$$$$$$$$$$$) { # Tests
	my ( $nom, $adresse, $refimmeuble, $ensemble, $building, $stairs, $floor, $door, $logo, $cp, $ville, $email, $tel, $telcontact, $typend, $debit, $option, $prix, $login, $passwd, $lid, $dplid ) = @_;
	conduse('FDN::Sql::Workflow::Run');
	workflow_select_db('adsl');
	workflow_select_config('adsl/wf.cfg');
	info("Creation de la tache de deploiement (debit $debit)");
	$debit =~ s/opt(0|1|3|5)$//;
	$debit =~ s/Liazo.*$//;
	my $task_vars = {
		'nom' => $nom,
		'adresse' => $adresse,
		'cp' => $cp,
		'ville' => $ville,
		'refimmeuble' => $refimmeuble,
		'ensemble' => $ensemble,
		'building' => $building,
		'stairs' => $stairs,
		'floor' => $floor,
		'door' => $door,
		'logo' => $logo,
		'email' => $email,
		'telephone' => $tel,
		'telcontact' => $telcontact,
		'typend' => $typend,
		'debit' => $debit,
		'option' => $option,
		'tarif' => $prix,
		'login' => $login,
		'passwd' => $passwd,
		'ligneid' => $lid,
		'deplid' => $dplid,
		'date' => (strftime "%Y-%m-%d %H:%M:%S", localtime)
	};
	my $tid;
	if ($option =~ /Liazo/) {
		$tid = launch_task('depl-adslL',$task_vars);
	} else {
		$tid = launch_task('depl-adslN',$task_vars);
	}
	if ( ! $tid ) {
		new_info_ligne($lid,'inline','Echec lancement de la tache',
			"Echec du lancement de la tache de deploiement de cette\n".
			"ligne. C'est inquietant, le client n'aura jamais satisfaction.");
		hard_failure("ERR Echec du lancement de la tache de creation. Alerter FDN.");
		return undef;
	}
	set_deploiement_task($dplid,$tid);
	new_info_ligne($lid,'inline','Creation de la tache de deploiement',
		"Creation de la tache de deploiement de cette ligne, numero $tid");
	return $tid;
}

sub creation_infos($$$$$) {
	my ( $lid, $mid, $option, $imgft, $imgtd ) = @_;
	new_info_ligne($lid,'fichier','img:Justificatif ligne telephonique',$imgft);
	if ( $option eq 'option1' ) {
		new_info_ligne($lid,'fichier','img:Mandat de degroupage',$imgtd);
		set_mandat_image($mid,$imgtd);
	}
}

sub termine_bien($) {
	my ( $dplid ) = @_;
	my $dpl = deploiement_par_id($dplid);
	my $num = $dpl->{'DEPLOIEMENT_NUM'};
	hard_failure("OK $num");
}

sub preparation_formulaire_auto($$) {
	my ( $subs, $vars ) = @_;
	conduse('FDN::Web::Forms');
	my $form = form_read ( 'adsl/subs/subs.cfg', 'subs'.$subs );
	my $tarif = Config_Key('adsl/subs.cfg',$subs,'tarif');
	$auto = $vars->{auto};
	if ( $vars->{do} ne 'yes' ) {
		for my $o ( @$offres ) {
			next if ! exists $form->{'byname'}->{$o};
			$o =~ /^(\d+(?:k|M|G|RE)(K1|K2|B1|B2|))(opt\d|Liazo.*)$/;
			my ( $debit, $option ) = ( $1, $3 );
			$option =~ s/opt/option/;
			my $d = debit_id_par_debit($debit);
			my $p = prix_id_par_debit_id_type_option($d, $tarif, $option);
			my $pr = prix_par_id($p);
			my $prix = sprintf(" %.2f ETTC d'installation puis %.2f ETTC par mois",$pr->{'PRIX_FASTTC'}/100,$pr->{'PRIX_ABOTTC'}/100);
			$form->{'byname'}->{$o}->{'input_label'} .= $prix;
		}
		eject_page(affiche_form($form,$vars));
		return 0;
	}
	if ( !check_transition($form,$vars) ) {
		if ( $auto eq 'yes' ) {
			hard_failure("ERR Faute de syntaxe");
		} else {
			eject_page(affiche_form($form,$vars));
		}
		return 0;
	}
	return 1;	
}

sub tout_faire_la_ligne($$) {
	my ( $subs, $vars ) = @_;
	my $etat = {};
	my $clid  = Config_Key('adsl/subs.cfg',$subs,'client-id');
	my $tarif = Config_Key('adsl/subs.cfg',$subs,'tarif');
	my $login = Config_Key('adsl/subs.cfg',$subs,'fakelogin');
	my $psswd = Config_Key('adsl/subs.cfg',$subs,'fakepsswd');
	#my $path_img = Config_Key('adsl/subs.cfg',$subs,'image-path');
	
	my $client = client_par_id($clid);
	my $auto   = $vars->{'auto'};
	debug "On a recupere le client $client a partir de l'id $clid.";

	if ( $vars->{debit} =~ /^(1G|300M)/ and $vars->{tel} =~ /^0[1-5]\d{8}$/ and ! $vars->{tellig}) {
		eject_page_error "Pour une livraisons FTTH sans numro de PTO connu, il est ncessaire d'avoir un numro de contact pour pouvoir tablir le rendez-vous de livraison.";
		return;
	}

	if ( $vars->{debit} =~ /^(1G|300M)/ and $vars->{tel} =~ /^0[1-5]\d{8}$/ and ! $vars->{refimmeuble}) {
		eject_page_error "Pour une livraisons FTTH sans numro de PTO connu, il est ncessaire de renseigner le champ d'identifiant d'immeuble IPE";
		return;
	}

	my $d = $vars->{debit};
	$d =~ s/opt\d$//;
	$d =~ s/Liazo.*//;
	my $debit = debit_id_par_debit($d);
	debug "On a recupere le debit $debit a partir du debit $d.";
	
	my $option = $vars->{debit};
	$option =~ s/^\d+(k|M|G|RE)(K1|K2|B1|B2|)//;
	$option = $options->{$option};
	debug "On a l'option $option.";
	
	my $prix = prix_id_par_debit_id_type_option($debit,$tarif,$option);
	debug "Du coup le prix est $prix.";

	my $typend = $vars->{typend};
	if ( $vars->{tel} =~ /^VOISIN\d+$/ ) {
		# Le voisin est forcment actif
		$typend = "actif";
	}
	
	if ( ! nouvelle_ligne($clid,$vars->{tel}) ) {
		error "La ligne n'est pas nouvelle, on abandonne.";
		return;
	}
	__debug "La ligne est bien nouvelle.";
	#if ( ! verifie_images($path_img, $option, $vars->{factureft}, $vars->{mandatdgrp}) ) {
	#	error "Manque une ou plusieurs images, on abandonne.";
	#	return;
	#}
	#__debug "Les images sont presentes.";
	#my $mid = verifie_mandat($subs,$vars->{nummandat});
	#if ( ! $mid ) {
	#	error "Le mandat n'appartient pas  $subs, on abandonne.";
	#	return;
	#}
	#__debug "Le mandat $mid appartient effectivement a $subs.";
	my $lid = creation_ligne(
			$vars->{nom},
			$vars->{adresse},
			$vars->{cp},
			$vars->{ville},
			$vars->{email},
			$vars->{pgp},
			$vars->{tel},
			$vars->{tellig},
			$debit,
			$option,
			$prix,
			$clid);
	if ( ! $lid ) {
		error "Pas reussi a creer la ligne, on abandonne.";
		return;
	}
	__debug "On a bien reussi a creer la ligne $lid.";
	my $dplid = creation_deploiement_ligne($lid);
	if ( ! $dplid ) {
		error "Pas reussi a creer le deploiement, on abandonne.";
		return;
	}
	__debug "On a bien reussi a creer le deploiement $dplid.";
	my $contactlig = $vars->{tellig};
	if (!defined $contactlig || length($contactlig) != 10) {
		my $adresse = adresse_par_id($client->{ADRESSE_ID});
		$contactlig = $adresse->{ADRESSE_TEL};
	}
	my $tid = creation_tache_adsl(
			$vars->{nom},
			$vars->{adresse},
			$vars->{refimmeuble},
			$vars->{ensemble},
			$vars->{building},
			$vars->{stairs},
			$vars->{floor},
			$vars->{door},
			$vars->{logo},
			$vars->{cp},
			$vars->{ville},
			$vars->{email},
			$vars->{tel},
			$contactlig,
			$typend,
			$vars->{debit},
			$option,
			$prix,
			$login,
			$psswd,
			$lid,
			$dplid
			);
	if ( ! $tid ) {
		error "Pas reussi a creer la tache de deploiement, on abandonne.";
		return;
	}
	__debug "On a bien reussi a creer la tache de deploiement $tid.";
	__debug "Tout va pour le mieux, on termine bien cette demande de $subs.";
	termine_bien($dplid);
}

sub creation_adhaccess($) {
	my ( $clid ) = @_;
	my $login = 'adhacc-'.$clid;
	my $passwd= clef_aleatoire(7);
	my $adhid = new_adhaccess($clid,$login,$passwd);
	return $adhid;
}

sub adhesion_fdn($) {
	my ( $vars ) = @_;
	$auto = 'fdn';
	my $clid = creation_client(
		$vars->{nomdflt},
		$vars->{adressedflt},
		$vars->{cpdflt},
		$vars->{villedflt},
		$vars->{email},
		$vars->{pgp},
		$vars->{tel},
		$vars->{tarif}
		);
	if ( ! $clid ) {
		die "Horribly";
		# PANIC
	}

	my $adhid = creation_adhaccess(
			$clid
			);
	my $acc = adhaccess_par_id($adhid);
	my $dir = 'adhacc-'.$clid;
	
	my $client = client_par_id($clid);
	my $plid;
#	if ( $vars->{'banque'} ) {
#		$plid = creation_prelevement_rib (
#				$clid,
#				$vars->{banque},
#				$vars->{guichet},
#				$vars->{compte},
#				$vars->{rib},
#				$vars->{nomdflt},
#				$vars->{nombq},
#				$vars->{adressebq},
#				$vars->{cpbq},
#				$vars->{villebq}
#			);
#	}
	if ( $vars->{'bic'} ) {
		$plid = creation_prelevement_iban (
				$clid,
				$vars->{'bic'},
				$vars->{'iban'},
				$vars->{'nomdflt'},
				$vars->{'nombq'}
			);
	}
	if ( $plid ) {
		if ( $client->{'PRELEVEMT_ID'} == -1 ) {
			set_client_prelevemt($clid,$plid);
		} else {
			info "Ce client est deja en prelevement. Reprise du prl existant.";
		}
	} else {
		info "Pas d'info bancaires dans la commande"
	}
	
	conduse('FDN::TeX::TeX');
	if ( ! set_tex_config('adsl/tex.cfg') ) {
		die "Horribly";
		# PANIC
	}
#	my $prl = make_pdf_file('autorisation',$dir, {
#			'NOM'       => $vars->{nomdflt},
#			'ADRESSE'   => $vars->{adressedflt},
#			'CP'        => $vars->{cpdflt},
#			'VILLE'     => $vars->{villedflt},
#			'NOMBQ'     => $vars->{nombq},
#			'ADRESSEBQ' => $vars->{adressebq},
#			'CPBQ'      => $vars->{cpbq},
#			'VILLEBQ'   => $vars->{villebq},
#			'BANQUE'    => $vars->{banque},
#			'GUICHET'   => $vars->{guichet},
#			'COMPTE'    => $vars->{compte},
#			'RIB'       => $vars->{rib}
#			});
	my $prel = prelevemt_par_id($plid);
	my $prl = make_pdf_file('mandat-sepa',$dir.'/prel'.$plid, {
			'CID'       => tex_encode($clid),
			'NUMADH'    => tex_encode($vars->{numadh}),
			'NOM'       => tex_encode($vars->{nomdflt}),
			'ADRESSE'   => tex_encode($vars->{adressedflt}),
			'CP'        => tex_encode($vars->{cpdflt}),
			'VILLE'     => tex_encode($vars->{villedflt}),
			'NOMBQ'     => tex_encode($vars->{nombq}),
			'BIC'       => tex_encode($vars->{bic}),
			'IBAN'      => tex_encode($vars->{iban}),
			'RUM'       => tex_encode($prel->{'PRELEVEMT_MANDAT'}),
			});
	my $adh = make_pdf_file('adhesion',$dir,{
			'CID'     => tex_encode($clid),
			'NUMADH'  => tex_encode($vars->{numadh}),
			'NOM'     => tex_encode($vars->{nomdflt}),
			'ADRESSE' => tex_encode($vars->{adressedflt}),
			'CP'      => tex_encode($vars->{cpdflt}),
			'VILLE'   => tex_encode($vars->{villedflt}),
			'TEL'     => tex_encode($vars->{tel}),
			'EMAIL'   => tex_encode($vars->{email}),
			});
	return $adhid;

}


sub souscription_fdn($) {
	my ( $vars ) = @_;
	$auto = 'fdn';

	my $debit = $vars->{debit};
	__debug("souscription a un debit $debit");
	$debit =~ s/opt\d$//;
	$debit =~ s/Liazo.*//;
	my $debitid = debit_id_par_debit($debit);
	__debug("cad debit $debit d'id $debitid");

	my $option = $vars->{debit};
	$option =~ s/^\d+(k|M|G|RE)(K1|K2|B1|B2|)//;
	__debug("cad option $option");
	$option = $options->{$option};
	__debug("et donc option $option");

	my $prix = prix_id_par_debit_id_type_option($debitid,$vars->{tarif},$option);
	
	my $clid = deja_client($vars->{numadh},$vars->{tel},$vars->{nomdflt});
	if ( ! $clid ) {
		$clid = creation_client(
				$vars->{nomdflt},
				$vars->{adressefac},
				$vars->{cpfac},
				$vars->{villefac},
				$vars->{email},
				$vars->{pgp},
				$vars->{telfac},
				$vars->{tarif}
				);
	}
	if ( ! $clid ) {
		die "Horribly."
		# PANIC
	}
	my $client = client_par_id($clid);

	my $adhid = creation_adhaccess(
			$clid
			);
	
#	my $plid = creation_prelevement_rib(
#			$clid,
#			$vars->{banque},
#			$vars->{guichet},
#			$vars->{compte},
#			$vars->{rib},
#			$vars->{nomdflt},
#			$vars->{nombq},
#			$vars->{adressebq},
#			$vars->{cpbq},
#			$vars->{villebq}
#			);
	my $plid = creation_prelevement_iban (
			$clid,
			$vars->{'bic'},
			$vars->{'iban'},
			$vars->{'nomdflt'},
			$vars->{'nombq'}
			);
	if ( $plid ) {
		if ( $client->{'PRELEVEMT_ID'} == -1 ) {
			set_client_prelevemt($clid,$plid);
		} else {
			info "Ce client est deja en prelevement. Reprise du prl existant.";
		}
	} else {
		info "Pas d'info bancaires dans la commande"
	}

	my $lid = deja_ligne($vars->{tel});
	my $ligne;

	if ( $lid ) {
		$ligne = ligne_par_id($lid);
		if ( $ligne->{'CLIENT_ID'} != $clid && $ligne->{'LIGNE_STATE'} eq "new" ) {
			new_info_ligne($lid,'inline','Rcupration de ligne',
				"Cette ligne tait prvue pour ".$ligne->{'CLIENT_ID'}."\n".
				"mais $clid a re-command la mme ligne.\n".
				"On oublie la premire commande.");
			set_ligne_tel($lid, $ligne->{'LIGNE_TEL'}.'.d'.$ligne->{'CLIENT_ID'});
			$lid = undef;
		}
	}
	if ( ! $lid ) {
		$lid = creation_ligne(
				$vars->{nomlig},
				$vars->{adressedflt},
				$vars->{cpdflt},
				$vars->{villedflt},
				$vars->{email},
				$vars->{pgp},
				$vars->{tel},
				$vars->{tellig},
				$debitid,
				$option,
				$prix,
				$clid);
		$ligne = ligne_par_id($lid);
		# il peut l'avoir chang (e.g. cas NOUVEAU)
		$vars->{tel} = $ligne->{LIGNE_TEL};
	} else {
		$ligne = ligne_par_id($lid);
		if ( $ligne->{'CLIENT_ID'} != $clid ) {
			new_info_ligne($lid,'inline','Tentative d\'usurpation',
				"Tentative d'usurpationde cette ligne par quelqu'un\n".
				"via le site de souscription en ligne.");
		}
	}
	if ( ! $lid ) {
		# PANIC
	}

	my $did = deja_deploiement($lid);
	if ( ! $did ) {
		$did = creation_deploiement_ligne($lid);
	}
	if ( ! $did ) {
		# PANIC
	}
	my $dpl = deploiement_par_id($did);
	my $dplnum = $dpl->{'DEPLOIEMENT_NUM'};

### Pas de mandat chez Nerim
###	conduse('FDN::Adsl::Mandat');
###	my $mid = create_mandat(
###			$vars->{tel},
###			$vars->{nomlig},
###			$vars->{adressedflt},
###			$vars->{cpdflt},
###			$vars->{villedflt},
###			'fdn');
###	if ( ! $mid ) {
###		# PANIC
###	}
###	set_mandat_deploiement_id($mid, $did);
	
	my $tid = deja_tache($did, $lid, $vars->{tel});
	if ( ! $tid ) {
		my $contactlig = $vars->{tellig};
		if (!defined $contactlig || length($contactlig) != 10) {
			my $adresse = adresse_par_id($client->{ADRESSE_ID});
			$contactlig = $adresse->{ADRESSE_TEL};
		}
		$tid = creation_tache_adsl(
				$vars->{nomlig},
				$vars->{adressedflt},
				$vars->{refimmeuble},
				$vars->{ensemble},
				$vars->{building},
				$vars->{stairs},
				$vars->{floor},
				$vars->{door},
				$vars->{logo},
				$vars->{cpdflt},
				$vars->{villedflt},
				$vars->{email},
				$vars->{tel},
				$contactlig,
				$vars->{typend},
				$vars->{debit},
				$option,
				$prix,
				'',
				'',
				$lid,
				$did
				);
	}
	if ( ! $tid ) {
		# PANIC
	}
	
	conduse('FDN::TeX::TeX');
	if ( ! set_tex_config('adsl/tex.cfg') ) {
		# PANIC
		confess "Panic";
	}
#	my $prl = make_pdf_file('autorisation',$dplnum, {
#			'NOM'       => tex_encode($vars->{nomdflt}),
#			'ADRESSE'   => tex_encode($vars->{adressefac}),
#			'CP'        => tex_encode($vars->{cpfac}),
#			'VILLE'     => tex_encode($vars->{villefac}),
#			'NOMBQ'     => tex_encode($vars->{nombq}),
#			'ADRESSEBQ' => tex_encode($vars->{adressebq}),
#			'CPBQ'      => tex_encode($vars->{cpbq}),
#			'VILLEBQ'   => tex_encode($vars->{villebq}),
#			'BANQUE'    => tex_encode($vars->{banque}),
#			'GUICHET'   => tex_encode($vars->{guichet}),
#			'COMPTE'    => tex_encode($vars->{compte}),
#			'RIB'       => tex_encode($vars->{rib}),
#			});
	my $prel = prelevemt_par_id($plid);
	info "Production de mandat SEPA";
	my $prl = make_pdf_file('mandat-sepa',$dplnum, {
			'CID'       => tex_encode($clid),
			'NUMADH'    => tex_encode($vars->{numadh}),
			'NOM'       => tex_encode($vars->{nomdflt}),
			'ADRESSE'   => tex_encode($vars->{adressefac}),
			'CP'        => tex_encode($vars->{cpfac}),
			'VILLE'     => tex_encode($vars->{villefac}),
			'NOMBQ'     => tex_encode($vars->{nombq}),
			'BIC'       => tex_encode($vars->{bic}),
			'IBAN'      => tex_encode($vars->{iban}),
			'RUM'       => tex_encode($prel->{'PRELEVEMT_MANDAT'}),
			});
	info "Production du bulletin de souscription";
	my $abo = make_pdf_file('souscription',$dplnum,{
			'CID'          => tex_encode($clid),
			'NUMADH'       => tex_encode($vars->{numadh}),
			'NOM'          => tex_encode($vars->{nomdflt}),
			'ADRESSE'      => tex_encode($vars->{adressefac}),
			'CP'           => tex_encode($vars->{cpfac}),
			'VILLE'        => tex_encode($vars->{villefac}),
			'ADRESSEDFLT'  => tex_encode($vars->{adressedflt}),
			'CPDFLT'       => tex_encode($vars->{cpdflt}),
			'VILLEDFLT'    => tex_encode($vars->{villedflt}),
			'TEL'          => tex_encode($vars->{tel}),
			'OPT'          => tex_encode($vars->{debit}),
			'DEBIT'        => tex_encode($debit),
			'OPTION'       => tex_encode($option),
			$vars->{tarif} => 1
			});
	info "Production du bulletin d'adhesion";
	my $adh = make_pdf_file('adhesion',$dplnum,{
			'CID'     => tex_encode($clid),
			'NUMADH'  => tex_encode($vars->{numadh}),
			'NOM'     => tex_encode($vars->{nomdflt}),
			'ADRESSE' => tex_encode($vars->{adressefac}),
			'CP'      => tex_encode($vars->{cpfac}),
			'VILLE'   => tex_encode($vars->{villefac}),
			'TEL'     => tex_encode($vars->{telfac}),
			'EMAIL'   => tex_encode($vars->{email}),
			'LIGNEID' => tex_encode($lid),
			});

	return $dplnum;
}

sub souscription_vpn($) {
	my ( $vars ) = @_;

	my $tarif = tarif_par_qui_quoi($vars->{'tarif'}, 'vpn');
	if ( ! defined $tarif ) {
		die "Failed to retrieve TARIF";
	}

	my $prix = $tarif->{'PRIX_ID'};
	if ( ! $prix ) {
		die "Failed to retrive PRIX_ID";
	}

	my $clid = creation_client(
			$vars->{nom},
			$vars->{adresse},
			$vars->{cp},
			$vars->{ville},
			$vars->{email},
			$vars->{pgp},
			$vars->{tel},
			$vars->{tarif}
			);
	if ( ! $clid ) {
		die "Failed to create CLIENT";
		# PANIC
	}
	my $client = client_par_id($clid);

	my $adhid = creation_adhaccess( $clid);
	
#	my $plid = creation_prelevement_rib(
#			$clid,
#			$vars->{banque},
#			$vars->{guichet},
#			$vars->{compte},
#			$vars->{rib},
#			$vars->{nom},
#			$vars->{nombq},
#			$vars->{adressebq},
#			$vars->{cpbq},
#			$vars->{villebq}
#			);
	my $plid = creation_prelevement_iban(
			$clid,
			$vars->{'bic'},
			$vars->{'iban'},
			$vars->{'nom'},
			$vars->{'nombq'}
			);
	if ( ! $plid ) {
		die "Failed to create PRELEVEMT";
		# PANIC
	}
	set_client_prelevemt($clid,$plid);

	my $vpnid = creation_vpn(
			$vars->{'nomvpn'},
			$vars->{'adressevpn'},
			$vars->{'cpvpn'},
			$vars->{'villevpn'},
			$vars->{'emailvpn'},
			$vars->{'pgpvpn'},
			$prix,
			$clid);
	my $dplid = creation_deploiement_vpn($vpnid);
	my $dpl = deploiement_par_id($dplid);
	my $dplnum = $dpl->{'DEPLOIEMENT_NUM'};
	my $tid = creation_tache_vpn($vars->{'nomvpn'},$vpnid, $dplid);
	conduse('FDN::TeX::TeX');
	if ( ! set_tex_config('adsl/tex.cfg') ) {
		# PANIC
	}
#	my $prl = make_pdf_file('autorisation',$dplnum, {
#			'NOM'       => tex_encode($vars->{nom}),
#			'ADRESSE'   => tex_encode($vars->{adresse}),
#			'CP'        => tex_encode($vars->{cp}),
#			'VILLE'     => tex_encode($vars->{ville}),
#			'NOMBQ'     => tex_encode($vars->{nombq}),
#			'ADRESSEBQ' => tex_encode($vars->{adressebq}),
#			'CPBQ'      => tex_encode($vars->{cpbq}),
#			'VILLEBQ'   => tex_encode($vars->{villebq}),
#			'BANQUE'    => tex_encode($vars->{banque}),
#			'GUICHET'   => tex_encode($vars->{guichet}),
#			'COMPTE'    => tex_encode($vars->{compte}),
#			'RIB'       => tex_encode($vars->{rib}),
#			});
	my $prel = prelevemt_par_id($plid);
	my $prl = make_pdf_file('mandat-sepa',$dplnum, {
			'CID'       => tex_encode($clid),
			'NUMADH'    => '',
			'NOM'       => tex_encode($vars->{nom}),
			'ADRESSE'   => tex_encode($vars->{adresse}),
			'CP'        => tex_encode($vars->{cp}),
			'VILLE'     => tex_encode($vars->{ville}),
			'NOMBQ'     => tex_encode($vars->{nombq}),
			'BIC'       => tex_encode($vars->{bic}),
			'IBAN'      => tex_encode($vars->{iban}),
			'RUM'       => tex_encode($prel->{'PRELEVEMT_MANDAT'}),
			});
	my $adh = make_pdf_file('adhesion',$dplnum,{
			'CID'     => tex_encode($clid),
			'NUMADH'  => '',
			'NOM'     => tex_encode($vars->{nom}),
			'ADRESSE' => tex_encode($vars->{adresse}),
			'CP'      => tex_encode($vars->{cp}),
			'VILLE'   => tex_encode($vars->{ville}),
			'EMAIL'   => tex_encode($vars->{email}),
			'VPNID'   => tex_encode($vpnid),
			});
	my $abo = make_pdf_file('abovpn',$dplnum,{
			'CID'          => tex_encode($clid),
			'NOM'          => tex_encode($vars->{nomvpn}),
			'ADRESSE'      => tex_encode($vars->{adressevpn}),
			'CP'           => tex_encode($vars->{cpvpn}),
			'VILLE'        => tex_encode($vars->{villevpn}),
			$vars->{tarif} => 1
			});
	return ( $dplnum, $vpnid );
}

1;






















