package FDN::Adsl::Stats;

use strict;
use warnings;

use base 'Exporter';

our @EXPORT = qw (
	$STATS
	compute_stats
	retrieve_stats
	stats_nb_adh
	stats_nb_vpn
);

use DateTime;
use FDN::Common::Logger;
use FDN::Adsl::Basic;
use FDN::Adsl::Ag;

our $STATS = {};
sub compute_stats {
	my ( $date ) = @_;
	$date //= DateTime->now->ymd;
	for my $name ( keys %$STATS ) {
		my $st = $STATS->{$name};
		if ( ref($st->{'code'}) ne 'CODE' ) {
			error "Je ne sais pas calculer des stats sans un CODEREF associe.";
			next;
		}
		my $res = &{$st->{'code'}}($date);
		my $sid;
		if ( $st->{'type'} eq 'count' ) {
			$sid = new_stat($date,$name,$res);
			if ( ! $sid ) {
				error "Echec d'ecriture en base pour new_stat avec les valeurs <$date> <$name> <$res>.";
				next;
			}
		} elsif ( $st->{'type'} eq 'abo' ) {
			if ( ref($res) ne 'HASH' || ! exists $res->{'count'} || ! exists $res->{'total'} || ! exists $res->{'duration'} ) {
				error "Le CODEREF pour calculer la stat '$name', de type 'abo', renvoie des donnees que je ne sais pas traiter";
				next;
			}
			$sid = new_stat($date,$name."-count",$res->{'count'});
			if ( ! $sid ) {
				error "Echec d'ecriture en base pour new_stat avec les valeurs <$date> <$name-count> <$res->{'count'}>.";
				next;
			}
			$sid = new_stat($date,$name."-total",$res->{'total'});
			if ( ! $sid ) {
				error "Echec d'ecriture en base pour new_stat avec les valeurs <$date> <$name-total> <$res->{'total'}>.";
				next;
			}
			$sid = new_stat($date,$name."-duration",$res->{'duration'});
			if ( ! $sid ) {
				error "Echec d'ecriture en base pour new_stat avec les valeurs <$date> <$name-duration> <$res->{'duration'}>.";
				next;
			}
		} else {
			error "La stat <$name> est du type <$st->{'type'}> que je ne sais pas traiter.";
		}
	}
}
our $DAILY = 0;
BEGIN {
	for my $name ( keys %$STATS ) {
		my $st = $STATS->{$name};
		if ( $st->{'type'} eq 'count' ) {
			$DAILY++;
		} elsif ( $st->{'type'} eq 'abo' ) {
			$DAILY+=3;
		} else {
			die "La stat '$name' est mal declaree, je ne connais pas le type <$st->{'type'}>.";
		}
	}
}

sub retrieve_stats {
	my ( $date ) = @_;
	$date //= DateTime->now->ymd;
	my $sts = [ stat_par_date($date) ];
	if ( ! @$sts ) {
		compute_stats($date);
		$sts = [ stat_par_date($date) ];
	}
	if ( ! @$sts ) {
		die "Echec du calcul des stats... Ca ne devrait jamais arriver";
	}
	my $res = {};
	for my $st ( @$sts ) {
		my $name = $st->{'STAT_NAME'};
		my $val  = $st->{'STAT_VALUE'};
		my $key;
		if ( $name =~ /^(.*)-(.*)$/ ) {
			$name = $1;
			$key = $2;
		} else {
			$key = 'count';
		}
		if ( ! exists $STATS->{$name} ) {
			die "Je ne sais pas afficher les stats '$name'";
		}
		my $st = $STATS->{$name};
		my $type = $st->{'type'};
		$res->{$name} //= {};
		$res->{$name}->{'show'} = $st->{'nom'};
		$res->{$name}->{'type'} = $st->{'type'};
		if ( $type eq 'count' ) {
			$res->{$name}->{'count'} = $val;
		} elsif ( $type eq 'abo' ) {
			$res->{$name}->{$key} = $val;
		}	
	}
	return $res;
}
	

sub stats_nb_adh {
	my ( $date ) = @_;
	$date //= DateTime->now->ymd;

	my $count = 0;
	my $adhs = [ tous_clients() ];
	for my $adh ( @$adhs ) {
		my $nom = $adh->{'CLIENT_NOM'};
		if ( droit_de_vote($adh, $date, 'present') ) {
			debug "$nom a le droit de venir le $date\n";
			$count++;
		} else {
			debug "$nom n'a pas le droit de venir le $date\n";
		}
	}
	return $count;
}
$STATS->{'nbadh'} = {
	nom => "Nombre d'adhrent.es",
	code => \&stats_nb_adh,
	type => 'count'
};

sub __stats_is_active_stuff {
	my ( $date, $item, $IN, $OUT ) = @_;

	my $in = $item->{$IN};
	$in =~ s/( |T)\d\d:\d\d:\d\d$//;
	if ( $in gt $date ) {
		# Debut apres la date, donc non;
		return 0;
	}
	if ( $in eq '0000-00-00' ) {
		# Pas de dbut, donc non;
		return 0;
	}
	$in =~ /(\d{4})-(\d\d)-(\d\d)/;
	$in = DateTime->new(year => $1, month => $2, day => $3)->epoch;

	my $out = $item->{$OUT};
	$out =~ s/( |T)\d\d:\d\d:\d\d$//;
	if ( $out eq '0000-00-00' ) {
		# Pas de fin, donc oui.
		return 1;
	}
	if ( $out lt $date ) {
		# Fini avant la date, donc non;
		return 0;
	}
}

sub __stats_nb_stuff {
	my ( $date, $items, $IN, $OUT ) = @_;
	$date =~ /(\d{4})-(\d\d)-(\d\d)/;
	my $epoch = DateTime->new(year => $1, month => $2, day => $3)->epoch;

	my $count = 0;
	my $total = 0;
	my $duration = 0;

	for my $item ( @$items ) {
		my $in = $item->{$IN};
		$in =~ s/( |T)\d\d:\d\d:\d\d$//;
		if ( $in gt $date ) {
			# Debut apres la date, donc non;
			next;
		}
		if ( $in eq '0000-00-00' ) {
			# Pas de dbut, donc non;
			next;
		}
		$in =~ /(\d{4})-(\d\d)-(\d\d)/;
		$in = DateTime->new(year => $1, month => $2, day => $3)->epoch;

		my $out = $item->{$OUT};
		$out =~ s/( |T)\d\d:\d\d:\d\d$//;
		if ( $out eq '0000-00-00' ) {
			# Pas de fin, donc oui.
			$duration += $epoch - $in;
			$count++;
			$total++;
			next;
		}
		if ( $out lt $date ) {
			# Fini avant la date, donc non;
			$out =~ /(\d{4})-(\d\d)-(\d\d)/;
			$out = DateTime->new(year => $1, month => $2, day => $3)->epoch;
			$duration += $out - $in;
			$total++;
			next;
		}
		# Date de fin connue, mais apres la date interessante, donc oui.
		$duration += $epoch - $in;
		$count++;
		$total++;
	}
	if ( $total ) {
		$duration = $duration/86400;
		$duration = $duration/$total;
		$duration =~ s/\.\d+$//;
	} else {
		$duration = 0;
	}
	return {
		count => $count,
		total => $total,
		duration => $duration
	};
}


sub stats_nb_adh_abonne {
	my ( $date ) = @_;
	$date //= DateTime->now->ymd;

	my $count = 0;
	my $adhs = [ tous_clients() ];
	for my $adh ( @$adhs ) {
		my $nom = $adh->{'CLIENT_NOM'};
		if ( droit_de_vote($adh, $date, 'present') ) {
			debug "$nom a le droit de venir le $date\n";
			my $lignes = [ ligne_par_client_id($adh->{'CLIENT_ID'}) ];
			for my $ligne ( @$lignes ) {
				if (__stats_is_active_stuff($date, $ligne, 'LIGNE_DTFAC', 'LIGNE_DTOUT')) {
					debug "$nom a au moins un abonnement xDSL/FTTH\n";
					$count++;
					next;
				}
			}
			my $vpns = [ vpn_par_client_id($adh->{'CLIENT_ID'}) ];
			for my $vpn ( @$vpns ) {
				if (__stats_is_active_stuff($date, $vpn, 'VPN_DTFAC', 'VPN_DTOUT')) {
					debug "$nom a au moins un abonnement VPN\n";
					$count++;
					next;
				}
			}
			debug "$nom n'a pas d'abonnement en cours\n";
		} else {
			debug "$nom n'a pas le droit de venir le $date\n";
		}
	}
	return $count;
}
$STATS->{'nbadh_abonne'} = {
	nom => "Nombre d'adhrent.es abonn.es  Internet",
	code => \&stats_nb_adh_abonne,
	type => 'count'
};


sub stats_nb_ligne_xdsl {
	my ( $date ) = @_;
	$date //= DateTime->now->ymd;
	my $lignes = [ toutes_lignes() ];
	my $items = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} =~ /^300M/ or
		    $debit->{'DEBIT_DEBIT'} =~ /^1G/ ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl'} = {
	nom => "Lignes xDSL",
	code => \&stats_nb_ligne_xdsl,
	type => 'abo'
};

sub stats_nb_vpn {
	my ( $date ) = @_;
	$date //= DateTime->now->ymd;
	my $vpns = [ tous_vpn() ];
	my $res = __stats_nb_stuff($date, $vpns, 'VPN_DTFAC', 'VPN_DTOUT');
	return $res;
}
$STATS->{'nbvpn'} = {
	nom => "VPN",
	code => \&stats_nb_vpn,
	type => 'abo'
};

sub stats_nb_ligne_xdsl_adh {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} =~ /^300M/ or
		    $debit->{'DEBIT_DEBIT'} =~ /^1G/ ) {
			next;
		}
		my $client = client_par_id($lig->{'CLIENT_ID'});
		if ( $client->{'CLIENT_TYPE'} =~ /blanche|collecte/ ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl_adh'} = {
	nom => "Lignes xDSL (adherent.es)",
	code => \&stats_nb_ligne_xdsl_adh,
	type => 'abo'
};

sub stats_nb_ligne_xdsl_coll {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} =~ /^300M/ or
		    $debit->{'DEBIT_DEBIT'} =~ /^1G/ ) {
			next;
		}
		my $client = client_par_id($lig->{'CLIENT_ID'});
		if ( $client->{'CLIENT_TYPE'} !~ /blanche|collecte/ ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl_coll'} = {
	nom => "Lignes xDSL (collecte)",
	code => \&stats_nb_ligne_xdsl_coll,
	type => 'abo'
};

sub stats_nb_ligne_xdsl_nerim {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} =~ /^300M/ or
		    $debit->{'DEBIT_DEBIT'} =~ /^1G/ ) {
			next;
		}
		if ($lig->{'LIGNE_OPT'} !~ /^option/) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl_nerim'} = {
	nom => "Lignes xDSL Nerim",
	code => \&stats_nb_ligne_xdsl_nerim,
	type => 'abo'
};

sub stats_nb_ligne_xdsl_liazo {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} =~ /^300M/ or
		    $debit->{'DEBIT_DEBIT'} =~ /^1G/ ) {
			next;
		}
		if ($lig->{'LIGNE_OPT'} !~ /^LiazoOrange/) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl_liazo'} = {
	nom => "Lignes xDSL Liazo",
	code => \&stats_nb_ligne_xdsl_liazo,
	type => 'abo'
};

sub stats_nb_ligne_adsl {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} =~ /^300M/ or
		    $debit->{'DEBIT_DEBIT'} =~ /^1G/ ) {
			next;
		}
		if ($debit->{'DEBIT_DEBIT'} eq "80M") {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl_adsl'} = {
	nom => "Lignes ADSL",
	code => \&stats_nb_ligne_adsl,
	type => 'abo'
};

sub stats_nb_ligne_vdsl {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} ne "80M" ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbadsl_vdsl'} = {
	nom => "Lignes VDSL",
	code => \&stats_nb_ligne_vdsl,
	type => 'abo'
};

sub stats_nb_ligne_ftth {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} !~ /^300M/ and
		    $debit->{'DEBIT_DEBIT'} !~ /^1G/ ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbftth'} = {
	nom => "Lignes FTTH",
	code => \&stats_nb_ligne_ftth,
	type => 'abo'
};

sub stats_nb_ligne_ftth_adh {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} !~ /^300M/ and
		    $debit->{'DEBIT_DEBIT'} !~ /^1G/ ) {
			next;
		}
		my $client = client_par_id($lig->{'CLIENT_ID'});
		if ( $client->{'CLIENT_TYPE'} =~ /blanche|collecte/ ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbftth_adh'} = {
	nom => "Lignes FTTH (adherent.es)",
	code => \&stats_nb_ligne_ftth_adh,
	type => 'abo'
};

sub stats_nb_ligne_ftth_coll {
	my ( $date ) = @_;
	$date //=DateTime->now->ymd;
	my $lignes = [ toutes_lignes ];
	my $items  = [];
	for my $lig ( @$lignes ) {
		my $debit = debit_par_id($lig->{'DEBIT_ID'});
		if ($debit->{'DEBIT_DEBIT'} !~ /^300M/ and
		    $debit->{'DEBIT_DEBIT'} !~ /^1G/ ) {
			next;
		}
		my $client = client_par_id($lig->{'CLIENT_ID'});
		if ( $client->{'CLIENT_TYPE'} !~ /blanche|collecte/ ) {
			next;
		}
		push @$items, $lig;
	}
	my $res = __stats_nb_stuff($date, $items, 'LIGNE_DTFAC', 'LIGNE_DTOUT');
	return $res;
}
$STATS->{'nbftth_coll'} = {
	nom => "Lignes FTTH (collecte)",
	code => \&stats_nb_ligne_ftth_coll,
	type => 'abo'
};

1;
