package FDN::Adsl::Torrent;

use strict;
use warnings;

use parent 'Exporter';
use Bencode qw(bdecode);
use FDN::Common::Logger;
use FDN::Common::Config;

our @EXPORT = qw (
	check_torrent
	list_torrents_files
	add_torrent_file
	del_torrent_file
);

my $fdn_user;
my $fdn_dir;
my $fdn_server;
my $fdn_tracker;
my $fdn_port;
my $fdn_announce;

our $CONFIG_FILE = 'adsl/torrents.cfg';

sub __read_config {
	$fdn_user    = Config_Key($CONFIG_FILE, 'main', 'user');
	$fdn_dir     = Config_Key($CONFIG_FILE, 'main', 'dir');
	$fdn_server  = Config_Key($CONFIG_FILE, 'main', 'server');
	$fdn_tracker = Config_Key($CONFIG_FILE, 'main', 'tracker');
	$fdn_port    = Config_Key($CONFIG_FILE, 'main', 'port');
	$fdn_announce = 'http://'.$fdn_tracker.':'.$fdn_port.'/announce';
	unless ( $fdn_user && $fdn_dir && $fdn_server && $fdn_tracker && $fdn_port ) {
		die "Invalid configuration file";
	}
}

sub add_torrent_file($$$) {
	my ( $cid, $name, $content ) = @_;
	__read_config();
	$name = $cid.'_'.$name;

	my $target = $fdn_user.'@'.$fdn_server;
	debug "Sending torrent ssh $target \"cat > $fdn_dir/$name\"";
	open OUT, "| ssh $target \"cat > $fdn_dir/$name\"";
	print OUT $content;
	close OUT;
}

sub del_torrent_file($$) {
	my ( $cid, $name ) = @_;
	__read_config();
	my $prefix = $cid.'_';
	if ( $name !~ /^$prefix/ ) {
		error "$name n'est pas un torrent pour le client $cid";
		return 0;
	}
	my $target = $fdn_user.'@'.$fdn_server;
	my $res = `ssh $target "cd $fdn_dir; rm $name"`;
}

sub list_torrents_files($) {
	my ( $cid ) = @_;
	__read_config();

	my $target = $fdn_user.'@'.$fdn_tracker;
	my $files  = $cid."_*";
	my $list = "ssh $target \"cd $fdn_dir; ls $files\"";
	# print STDERR "Listing files with $list";
	$list = `$list`;
	$list = [ split /\n/, $list ];
	return $list;
}

sub check_torrent($) {
	my ( $file ) = @_;
	__read_config();

	my $res = bdecode($file, 1, 5);
	if ( exists $res->{'announce'} && !ref($res->{'announce'}) ) {
		if ( $res->{'announce'} eq $fdn_announce ) {
			debug "FDN is the main announce, file is valid and for us.";
			return 1;
		}
		debug "FDN is not the main announce in the file.";
		# print STDERR "Expected $fdn_announce, got $res->{'announce'}";
	} else {
		debug "Invalid torrent file, no announce.";
		return 0;
	}
	if ( exists $res->{'announce-list'} && ref($res->{'announce-list'}) eq 'ARRAY' ) {
		for my $list ( @{$res->{'announce-list'}} ) {
			next if ref($list) ne 'ARRAY';
			for my $ann ( @$list ) {
				if ( $ann eq $fdn_announce ) {
					return 1;
				}
				# print STDERR "Expected $fdn_announce, got $ann";
			}
		}
		debug "FDN is neither any other in announce-list";
	}
	debug "FDN is not a tracker for that torrent. Please discard.";
	return 0;
}

1;
