package FDN::Adsl::Utils;

use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		prt_tva
		prt_euro
		date_to_ts
		ts_to_date
		calcul_factu
		ligne_factu
		$errors
		check_transition
		add_form_error
		reset_form_errors
		affiche_form
		ajoute_liste_selval
		ajoute_separateur
		eject_page
		eject_page_text
		eject_page_error
		eject_page_index
		send_png_file
		send_jpeg_file
		send_pdf_file
		back_from
		forward_to
		ajoute_liste_clients
		ajoute_liste_clients_actifs
		ajoute_liste_clients_facture
		ajoute_liste_factures
		ajoute_liste_factures_dues
		ajoute_liste_factures_envoyees
		ajoute_liste_factures_payees
		ajoute_liste_prelevements
		ajoute_liste_prelevements_dus
		ajoute_liste_prelevements_envoyes
		ajoute_liste_prelevements_payes
		ajoute_liste_comptas
		ajoute_liste_debits
		ajoute_liste_prix
		ajoute_liste_scanners
		ajoute_liste_banques
		ajoute_liste_deploiement
		ajoute_liste_lignes
		ajoute_liste_raduser
		ajoute_liste_unixgroup
		ajoute_champ_config
		array_has_no_rows
		view_task
		clef_aleatoire
		get_locked_tasks
		verif_clef_rib
		verif_clef_iban
                );
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Common::Constantes;
use FDN::Web::TemplateParse;
use FDN::Etebac::Rib;
use FDN::Adsl::Basic;
use CGI;

our $errors = {};
my $ftpl = '/usr/share/fdn/templates/forms/form-part.html';

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

my @mois  = (
	'janvier',
	'fevrier',
	'mars',
	'avril',
	'mai',
	'juin',
	'juillet',
	'aout',
	'septembre',
	'octobre',
	'novembre',
	'decembre');

sub date_to_ts($) {
        my ( $date ) = @_;
        $date =~ s/(\d{2})\/(\d{2})\/(\d{4})/$3$2${1}000000/;
        return $date;
}

sub ts_to_date($) {
	my ( $date ) = @_;
	$date =~ /^(\d{4})(\d{2})(\d{2})\d{6}$/;
	return $3." ".$mois[$2-1]." ".$1;
}

sub prt_tva($) {
        my ( $tva ) = @_;
        if ( $tva eq '200' ) {
                return "20,00 %";
        } elsif ( $tva eq '196' ) {
                return "19,60 %";
        } elsif ( $tva eq '55' ) {
                return "5,50 %";
        } else {
                return "N/A";
        }
}

sub prt_euro($) {
        my ($ct)=@_;
	$ct =~ s/^$/0/;
	$ct =~ s/^(\d)$/0$1/;
	$ct =~ s/^(\d\d)$/0$1/;
        $ct =~ s/(\d{2})$/,$1/;
        return $ct;
}

sub ligne_factu($$$$$) {
        my ( $factu, $ht, $ttc, $tva, $libelle ) = @_;

        my $ligne = {};
        $ligne->{ht} = prt_euro($ht);
        $ligne->{ttc} = prt_euro($ttc);
        $ligne->{tva} = $tva;
        $ligne->{dttva} = prt_euro($ttc-$ht);
        $ligne->{lib} = $libelle;
        push @{$factu->{lignes}},$ligne;
        $factu->{totalht} += $ht;
        $factu->{totalttc} += $ttc;
        $factu->{totaltva} += $ttc-$ht;
}

sub calcul_factu($$$) {
        my ($cid, $debut, $fin) = @_;

        my $factu = {};

        my $client = client_par_id($cid);
        $factu->{nom} = $client->{CLIENT_NOM};
        $factu->{totalht} = 0;
        $factu->{totalttc} = 0;
        $factu->{totaltva} = 0;
        $factu->{lignes} = [];
        $factu->{debut} = ts_to_date($debut);
        $factu->{fin} = ts_to_date($fin);

        my @cls = clientligne_par_client_id($cid);
        for my $cl ( @cls ) {
                my $lid = $cl->{LIGNE_ID};
                my $ligne = ligne_par_id($lid);
                next if (!defined($ligne->{LIGNE_ID}));

                my $prix = prix_par_id($ligne->{PRIX_ID});
                my $tva = prt_tva($prix->{PRIX_TVA});
                my $ttc;
                my $ht;
                my $lib;

                if (if_fas($ligne,$debut,$fin)) {
                        $ttc = $prix->{PRIX_FASTTC};
                        $ht  = $prix->{PRIX_FASHT};
                        $lib = "FAS ".$ligne->{LIGNE_TEL};
                        ligne_factu($factu,$ht,$ttc,$tva,$lib);
                }
                my $prorata=prorata($ligne,$debut,$fin);
                if ($prorata>0) {
                        $ttc = $prorata * $prix->{PRIX_ABOTTC};
			$ttc =~ s/\..*$//;
                        $ht  = $prorata * $prix->{PRIX_ABOHT};
			$ht  =~ s/\..*$//;
                        $lib = "Abonnement ".$ligne->{LIGNE_TEL};
                        ligne_factu($factu,$ht,$ttc,$tva,$lib);
                }
        }
        $factu->{totalht} = prt_euro($factu->{totalht});
        $factu->{totalttc} = prt_euro($factu->{totalttc});
        $factu->{totaltva} = prt_euro($factu->{totaltva});
        return $factu;
}

# Verifications des formulaires

sub check_transition($$) {
        my ( $form, $vars ) = @_;
	conduse('FDN::Web::Forms');
        my $errs = {};
        if ( ! form_check_val($form,$vars,$errs) ) {
                get_form_errors($errs);
                return FALSE;
        }
        return TRUE;
}

sub affiche_form($$;$) {
        my ( $form, $vars, $template ) = @_;
	$template = '/page-forms.tpl' if ! defined $template;
	conduse('FDN::Web::Forms');
        form_keep_val($form,$vars);
        my $glob = form_print_parts($form,$ftpl);
        $glob = { %$glob, %$errors };
	conduse('FDN::Common::Config');
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
        remplace($dir.$template,$glob,1);
}

# Reprend les erreurs du formulaire et les range comme il faut pour
# les afficher.

sub get_form_errors($) {
        my ( $errs ) = @_;
        if ( defined $errs->{mandatory} ) {
                info "Vu des mandatory manquants...";
                my $e = $errs->{mandatory};
                $errors->{mandatory} = [];
                my $e2 = $errors->{mandatory};
                for my $f ( keys %$e ) {
                        push @$e2, {
                                err => $e->{$f}
                                };
                }
                if ( $#$e2 == -1 ) {
                        delete $errors->{mandatory};
                }
        }
        if ( defined $errs->{invalid} ) {
                info "Vu des parametres invalides...";
                my $e = $errs->{invalid};
                $errors->{syntax} = [];
                my $e2 = $errors->{syntax};
                for my $f ( keys %$e ) {
                        push @$e2, {
                                err => $e->{$f}
                                };
                }
                if ( $#$e2 == -1 ) {
                        delete $errors->{syntax};
                }
        }
}

# Ajoute une erreur traitee cote CGI pour les formulaires

sub add_form_error($$) {
	my ( $type, $message ) = @_;
	if ( ! exists $errors->{$type} ) {
		$errors->{$type} = [];
	}
	push @{$errors->{$type}}, {
		'err' => $message
		};
}

sub reset_form_errors {
	$errors = {};
}

# Re-ecrit le resultat d'une requete SQL sous forme d'une liste
# de selval pour un champs select.
sub ajoute_liste_selval($$$$$) {
        my ( $form, $rewrite_file, $rewrite, $field, $liste ) = @_;
	conduse('FDN::Common::Config');
	conduse('FDN::Web::Forms','dynamics');
        my $cfg = {};
        my @lst;
        for my $elt ( @$liste ) {
                my $res;
                $res = Rewrite_Table($rewrite_file,$rewrite,$elt);
                my $c = {};
                for my $e ( @{$res->{valeurs}} ) {
                        $c->{$e->{nom}} = $e->{valeur};
                }
                $c->{value} = $c->{value};
                my $key = $c->{value};
                $cfg->{"selval-".$key} = $c;
                push @lst, $key;
        }
        my $sel = $form->{byname}->{$field};
        for my $k ( @lst ) {
                my $sv = form_new_selval($cfg,$k);
                form_add_selval($sel,$sv);
        }
}

sub ajoute_separateur($$$$) {
	my ( $form, $field, $texte,$nom ) = @_;
	conduse('FDN::Web::Forms','dynamics');
	my $cfg = {
		'selval-'.$nom => {
			'label' => $texte,
			'value' => ''
		}
	};
	my $sel = $form->{byname}->{$field};
	my $sv = form_new_selval($cfg,$nom);
	form_add_selval($sel,$sv);
}

# Pour simplifier l'ecriture des CGI

sub eject_page($;$) {
	my ( $page, $template ) = @_;
	$template = '/base.tpl' if ! defined $template;
	print "Content-Type: text/html; charset=ISO-8859-1\n\n";
	conduse('FDN::Common::Config');
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	$dir = '' if ! defined $dir;
	my $res = remplace($dir.$template,{'PAGE'=>$page},);
	$res = '' if ! defined $res;
	print $res;
}

sub eject_page_text($) {
	my ( $page ) = @_;
	print "Content-Type: text/plain\n\n";
	print $page;
}

sub eject_page_error($) {
	my ( $text ) = @_;
	eject_page("<div class=\"error\">$text</div>");
}

sub eject_page_index() {
	conduse('FDN::Common::Config');
	my $index = Config_Key('adsl/global.cfg','web','index');
	print "Location: $index\n\n";
}

sub send_png_file($) {
	my ( $file ) = @_;
	print "Content-Type: image/png\n\n";
	local $/;
	my $ref = ref($file);
	if ( ($ref eq 'IO') or ($ref eq 'GLOB') ) {
		__debug "Sending content as png";
		print <$file>;
	} else {
		__debug "Sending as png file $file.";
		open FILE, $file;
		print <FILE>;
		close FILE;
	}
}

sub send_jpeg_file($) {
	my ( $file ) = @_;
	print "Content-Type: image/jpeg\n\n";
	__debug "Sending as jpeg file $file.";
	local $/;
	open FILE, $file;
	print <FILE>;
	close FILE;
}

sub send_pdf_file($;$) {
	my ( $file, $asname ) = @_;
	if ( ! defined $asname ) {
		$asname = $file;
		$asname =~ s/^(.*)?\/([^\/]+)$/$2/;
	}
	print "Content-Type: application/pdf\n".
	      "Content-Disposition: inline; filename=\"$asname\"\n\n";
	__debug "Sending as pdf file $file.";
	local $/;
	open FILE, $file;
	print <FILE>;
	close FILE;
}

our $froms = {
	'client' => {
		'url' => '/adsl/client/view',
		'id'  => 'cid',
	},
	'client-compta' => {
		'url' => '/adsl/client/view',
		'id'  => 'cid',
	},
	'client-technique' => {
		'url' => '/adsl/client/view',
		'id'  => 'cid',
	},
	'ligne' => {
		'url' => '/adsl/ligne/view',
		'id'  => 'lid',
	},
	'payant' => {
		'url' => '/adsl/payant/view',
		'id'  => 'pid',
	},
	'adresse' => {
		'url' => '/adsl/adresse/view',
		'id'  => 'aid',
	},
	'prelevemt' => {
		'url' => '/adsl/prelevemt/view',
		'id'  => 'plid',
	},
	'prix' => {
		'url' => '/adsl/view-prix.cgi',
		'id'  => 'pid',
	},
	'prllig' => {
		'url' => '/adsl/prllig/view',
		'id'  => 'pid',
	},
	'prlligsepa' => {
		'url' => '/adsl/prlligsepa/view',
		'id'  => 'pid',
	},
	'creance' => {
		'url' => '/adsl/creance/view',
		'id'  => 'cid',
	},
	'paiement' => {
		'url' => '/adsl/paiement/view',
		'id'  => 'pid',
	},
	'uattr' => {
		'url' => '/adsl/view-uattr.cgi',
		'id'  => 'uid',
	},
	'gattr' => {
		'url' => '/adsl/view-gattr.cgi',
		'id'  => 'gid',
	},
	'raduser' => {
		'url' => '/adsl/view-raduser.cgi',
		'id'  => 'uid',
	},
	'compta' => {
		'url' => '/adsl/compta/view',
		'id' => 'cid',
	},
	'adhaccess' => {
		'url' => '/adsl/adhaccess/view',
		'id'  => 'adhid'
	},
	'site' => {
		'url' => '/adsl/view-site.cgi',
		'id'  => 'sid'
	},
	'domaine' => {
		'url' => '/adsl/view-domaine.cgi',
		'id'  => 'did'
	},
	'val' => {
		'url' => '/adsl/view-validation.cgi',
		'id' => 'id'
	},
	'vpn' => {
		'url' => '/adsl/vpn/view',
		'id' => 'vid'
	},
	'unixuser' => {
		'url' => '/adsl/view-unixuser.cgi',
		'id'  => 'id'
	},
	'unixgroup' => {
		'url' => '/adsl/view-unixgroup.cgi',
		'id'  => 'id'
	},
	'don' => {
		'url' => '/adsl/don/view',
		'id' => 'did'
	},
};

sub back_from($$) {
	my ( $where, $frominfo ) = @_;
	$frominfo =~ /^(\D*)(\d*)$/;
	my ( $from, $fid ) = ( $1, $2 );
	my $cgi = new CGI;
	my $url = '/adsl/index.cgi';

	if ( ! defined $froms->{$from} ) {
		__debug "From $from n'est pas defini.";
		print $cgi->redirect($url);
		return;
	}

	my $frm = $froms->{$from};
	$url = $frm->{'url'};
	$url .= '?'.$frm->{'id'}.'='.$fid.'&do=yes';
	__debug "Back to url $url";
	print $cgi->redirect($url);
}

my $forwards = {
	'client' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/client/new'
	},
	'client-normal' => {
		'id'  => 'do=yes&cid',
		'txt' => 'Infos gnrales',
		'url' => '/adsl/client/view'
	},
	'client-compta' => {
		'id'  => 'do=yes&cid',
		'txt' => 'Infos comptables',
		'url' => '/adsl/client/view_compta'
	},
	'client-technique' => {
		'id'  => 'do=yes&cid',
		'txt' => 'Infos techniques',
		'url' => '/adsl/client/view_tech'
	},
	'numadh' => {
		'id'  => 'do=yes&cid',
		'txt' => 'Donner numro d\'adhrent',
		'url' => '/adsl/new-numadh.cgi'
	},
	'ligne' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/ligne/new'
	},
	'adresse' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/adresse/new'
	},
	'payant' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/payant/new'
	},
	'npayant' => {
		'id' => 'client',
		'txt' => 'Ajouter',
		'url' => '/adsl/payant/new'
	},
	'creance' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/creance/new'
	},
	'ncreance' => {
		'id' => 'client',
		'txt' => 'Ajouter',
		'url' => '/adsl/creance/new'
	},
	'prelevemt' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/prelevemt/new'
	},
	'nprelevemt' => {
		'id' => 'cid',
		'txt' => 'Ajouter',
		'url' => '/adsl/prelevemt/new'
	},
	'prix' => {
		'id' => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-prix.cgi'
	},
	'paiement' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/paiement/new'
	},
	'npaiement' => {
		'id'  => 'client',
		'txt' => 'Ajouter',
		'url' => '/adsl/paiement/new'
	},
	'uattr' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-uattr.cgi'
	},
	'nuattr' => {
		'id'  => 'uid',
		'txt' => 'Ajouter',
		'url' => '/adsl/new-uattr.cgi'
	},
	'gattr' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-gattr.cgi'
	},
	'ngattr' => {
		'id'  => 'gid',
		'txt' => 'Ajouter',
		'url' => '/adsl/new-gattr.cgi'
	},
	'ipaddr' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-ipaddr.cgi'
	},
	'raduser' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-raduser.cgi'
	},
	'lclient' => {
		'txt' => 'Tous',
		'url' => '/adsl/client/all'
	},
	'lclient-act' => {
		'txt' => 'Restreint aux actifs',
		'url' => '/adsl/client/act'
	},
	'lclient-resil' => {
		'txt' => 'Restreint aux rsilis',
		'url' => '/adsl/client/resil'
	},
	'lval' => {
		'id'  => 'tous',
		'txt' => 'Tous',
		'url' => '/adsl/liste-validations.cgi'
	},
	'lval-new' => {
		'id'  => 'new',
		'txt' => 'Nouvelles',
		'url' => '/adsl/liste-validations.cgi'
	},
	'lval-done' => {
		'id'  => 'done',
		'txt' => 'Faites',
		'url' => '/adsl/liste-validations.cgi'
	},
	'adhaccess' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/adhaccess/new'
	},
	'validation' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-validation.cgi'
	},
	'site' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-site.cgi'
	},
	'domaine' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-domaine.cgi'
	},
	'nsite' => {
		'id'  => 'client',
		'txt' => 'Ajouter',
		'url' => '/adsl/new-site.cgi'
	},
	'ndomaine' => {
		'id'  => 'client',
		'txt' => 'Ajouter',
		'url' => '/adsl/new-domaine.cgi'
	},
	'unixuser' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-unixuser.cgi'
	},
	'nunixuser' => {
		'id'  => 'client',
		'txt' => 'Ajouter',
		'url' => '/adsl/new-unixuser.cgi'
	},
	'unixgroup' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-unixgroup.cgi'
	},
	'nunixgroup' => {
		'id'  => 'rien',
		'txt' => 'Ajouter',
		'url' => '/adsl/new-unixgroup.cgi'
	},
	'padresse-tiers' => {
		'id'  => 'type=tiers&aid',
		'txt' => 'Imprimer 1/3',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-tiers0' => {
		'id'  => 'type=tiers&aid',
		'txt' => 'Imprimer env. vierge 1/3',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-demi' => {
		'id'  => 'type=demi&aid',
		'txt' => 'Imprimer 1/2',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-demi0' => {
		'id' => 'type=demi&aid',
		'txt' => 'Imprimer env. vierge 1/2',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-a4' => {
		'id'  => 'type=a4&aid',
		'txt' => 'Imprimer A4 portrait',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-a40' => {
		'id'  => 'type=a4&aid',
		'txt' => 'Imprimer env. vierge A4 portrait',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-a4scape' => {
		'id'  => 'type=a4scape&aid',
		'txt' => 'Imprimer A4 paysage',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'padresse-a4scape0' => {
		'id'  => 'type=a4scape&aid',
		'txt' => 'Imprimer env. vierge A4 paysage',
		'url' => '/adsl/print-enveloppe.cgi'
	},
	'nscan' => {
		'id'  => 'rien',
		'txt' => 'Numriser',
		'url' => '/adsl/new-scan.cgi'
	},
	'nvpn' => {
		'id'  => 'id',
		'txt' => 'Modifier',
		'url' => '/adsl/new-vpn.cgi'
	},
	'ninfo' => {
		'id' => 'id',
		'txt' => 'Ajouter',
		'url' => '/adsl/infos/new'
	},
};

sub forward_to($$$) {
	my ( $to, $toid, $from ) = @_;
	my $fwd = $forwards->{$to};
	my $fld = $fwd->{'id'};
	my $txt = $fwd->{'txt'};
	my $url = $fwd->{'url'};
	return "(<a href=\"$url?$fld=$toid&from=$from\">$txt</a>)";
}

# Pour ajouter un select sur l'ensemble des clients

sub ajoute_liste_clients($) {
        my ( $form ) = @_;
        my @clts = tous_clients();
        @clts = sort { $a->{'CLIENT_NUMADH'} cmp $b->{'CLIENT_NUMADH'} } @clts;
        ajoute_liste_selval($form,'adsl/rewrite.tbl','lstclt','client',\@clts);
}

sub ajoute_liste_clients_actifs($) {
        my ( $form ) = @_;
        my @clts = tous_clients_actifs();
        @clts = sort { $a->{'CLIENT_NUMADH'} cmp $b->{'CLIENT_NUMADH'} } @clts;
        ajoute_liste_selval($form,'adsl/rewrite.tbl','lstclt','client',\@clts);
}

sub ajoute_liste_clients_facture($) {
        my ( $form ) = @_;
        my @clts = tous_clients_facture();
        @clts = sort { $a->{'CLIENT_NUMADH'} cmp $b->{'CLIENT_NUMADH'} } @clts;
        ajoute_liste_selval($form,'adsl/rewrite.tbl','lstclt','client',\@clts);
}

sub ajoute_separateur_factures($$$) {
	my ( $form, $texte, $nom ) = @_;
	ajoute_separateur($form,'factu',$texte,$nom);
}

sub ajoute_liste_factures_etat($$) {
	my ( $form, $etat ) = @_;
	my @factus = facture_par_etat($etat);
	if ( ! array_has_no_rows(\@factus) ) {
		ajoute_liste_selval($form,'adsl/rewrite.tbl','lstfac','factu',\@factus);
	}
}

sub ajoute_liste_factures_dues($) {
	ajoute_liste_factures_etat($_[0],'due');
}

sub ajoute_liste_factures_envoyees($) {
	ajoute_liste_factures_etat($_[0],'envoyee');
}

sub ajoute_liste_factures_payees($) {
	ajoute_liste_factures_etat($_[0],'payee');
}

sub ajoute_liste_factures($) {
	my ( $form ) = @_;
	ajoute_separateur_factures($form,'Factures dues','dues');
	ajoute_liste_factures_dues($form);
	ajoute_separateur_factures($form,'Factures envoyes','envoyees');
	ajoute_liste_factures_envoyees($form);
	ajoute_separateur_factures($form,'Factures payes','payees');
	ajoute_liste_factures_payees($form);
}

sub ajoute_separateur_prelevements($$$) {
	my ( $form, $texte, $nom ) = @_;
	ajoute_separateur($form,'prlbq',$texte,$nom);
}

sub ajoute_liste_prelevements_etat($$) {
	my ( $form, $etat ) = @_;
	my @prls = prlbq_par_etat($etat);
	if ( ! array_has_no_rows(\@prls) ) {
		ajoute_liste_selval($form,'adsl/rewrite.tbl','lstprl','prlbq',\@prls);
	}
}

sub ajoute_liste_prelevements_dus($) {
	ajoute_liste_prelevements_etat($_[0],'du');
}

sub ajoute_liste_prelevements_envoyes($) {
	ajoute_liste_prelevements_etat($_[0],'envoye');
}

sub ajoute_liste_prelevements_payes($) {
	ajoute_liste_prelevements_etat($_[0],'paye');
}

sub ajoute_liste_prelevements($) {
	my ( $form ) = @_;
	ajoute_separateur_prelevements($form,'Prlvements dus','dus');
	ajoute_liste_prelevements_dus($form);
	ajoute_separateur_prelevements($form,'Prlvements envoys','envoyes');
	ajoute_liste_prelevements_envoyes($form);
	ajoute_separateur_prelevements($form,'Prlvements pays','payes');
	ajoute_liste_prelevements_payes($form);
}

sub ajoute_liste_debits($) {
	my ( $form ) = @_;
	my @debits = tous_debits();
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstdbt','debit',\@debits);
}

sub ajoute_liste_prix($) {
	my ( $form ) = @_;
	my @prix = tous_prix();
	@prix = sort { $a->{'PRIX_NOM'} cmp $b->{'PRIX_NOM'} } @prix;
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lsttrf','prix',\@prix);
}

sub ajoute_liste_comptas($) {
	my ( $form ) = @_;
	my @tmp = tous_comptas();
	my @comptas;
	for my $c ( @tmp ) {
		if ( exists $c->{'COMPTA_ETAT'} && defined $c->{'COMPTA_ETAT'} && $c->{'COMPTA_ETAT'} eq 'closed' ) {
			next;
		}
		push @comptas, $c;
	}
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstcompta','compta',\@comptas);
}

sub ajoute_liste_scanners($) {
	my ( $form ) = @_;
	conduse('FDN::Common::Config');
	my $cfg = Config_Section('adsl/global.cfg','scan');
	my @scans;
	for my $k ( keys %$cfg ) {
		push @scans, { 'key' => $k, 'val' => $cfg->{$k} };
	}
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstscan','scanner',\@scans);
	
}

sub ajoute_liste_banques($) {
	my ( $form ) = @_;
	conduse('FDN::Common::Config');
	my $cfg = Config_Section('adsl/global.cfg','banque');
	my @banques;
	for my $k ( keys %$cfg ) {
		push @banques, { 'key' => $k, 'val' => $cfg->{$k} };
	}
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstbq','banque',\@banques);
}

sub ajoute_liste_deploiement($) {
	my ( $form ) = @_;
	my @ligs = ligne_par_state('new');
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstdpl','ligne',\@ligs);
}

sub ajoute_liste_lignes($;$) {
	my ( $form, $field ) = @_;
	my @ligs = toutes_lignes();
	@ligs = sort { $a->{'LIGNE_ID'} <=> $b->{'LIGNE_ID'} } @ligs;
	$field = 'ligne' if (!defined $field);
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstdpl',$field,\@ligs);
}

sub ajoute_liste_raduser($;$) {
	my ( $form, $field ) = @_;
	my @rads = tous_raduser();
	@rads = sort { $a->{'RADUSER_ID'} <=> $b->{'RADUSER_ID'} } @rads;
	$field = 'raduser' if (!defined $field);
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstradu',$field,\@rads);
}

sub ajoute_liste_unixgroup($) {
	my ( $form ) = @_;
	my @groupes = tous_unixgroup();
	ajoute_liste_selval($form,'adsl/rewrite.tbl','lstugrp','groupe',\@groupes);
}

# Pour ajouter un champs qui est deja en config

sub ajoute_champ_config($$$$;$) {
	my ( $form, $cfg_file, $block, $champ, $placement ) = @_;
	conduse('FDN::Common::Config');
	conduse('FDN::Web::Forms','dynamics');
	my $cfg = Load_Config($cfg_file);
	my $fld = form_new_field($cfg,$champ);
	my $blk = $form->{'blocksbyname'}->{$block};
	form_add_field($blk,$fld,$placement);
}

# Pour voir une tache

sub view_task($) {
	my ( $tid ) = @_;
	conduse('FDN::Sql::Workflow::Basic');
	conduse('FDN::Web::Array');
	workflow_select_db('adsl');
	my $page = "";

	# Moi
	my $task = task_par_id($tid);
	$page .= tableau_1d('adsl/rewrite.tbl','taskprint',$task,"Tche courante");
	# Papa
	my $ts = taskself_par_fils($tid);
	if ( defined($ts) ) {
		my $task = task_par_id($ts->{'TASK_ID'});
		$page .= tableau_1d('adsl/rewrite.tbl','taskprint',$task,"Tche d'origine");
	} else {
		$page .= "<h1>Pas de tche d'origine</h1>\n";
	}
	
	# Mes fils
	my @ts = taskself_par_parent($tid);
	if ( array_has_no_rows(\@ts) ) {
		$page .= "<h1>Pas de sous-tche connue</h1>\n";
	} else {
		my @tasks = ();
		for $ts ( @ts ) {
			my $t = task_par_id($ts->{SELF_ID});
			if ( defined($t) ) {
				push @tasks, $t;
			}									    
		}
		@tasks = sort { $a->{TASK_ID} <=> $b->{TASK_ID} } @tasks;
		$page .= tableau_2d('adsl/rewrite.tbl','taskview',\@tasks,"Sous-tche(s) connue(s)");
	}
	# Mon etat
	my @runs = run_par_task($tid);
	if ( array_has_no_rows(\@runs) ) {
		$page .= "<h1>Pas d'execution attendue</h1>\n";
	} else {
		$page .= "<h1>Prochaine execution prevue sur vnement ".$runs[0]->{'RUN_EVENT'}."</h1>";
	}

	my @waits = waiting_par_task($tid);
	my @rwts;
	if ( array_has_no_rows(\@waits) ) {
		$page .= "<h1>Pas d'vnement attendu</h1>\n";
	} else {
		$page .= tableau_2d('adsl/rewrite.tbl','waiting',\@waits,
			"vnements attendus (cliquer sur l'vnement pour le creer)");
		for my $wait ( @waits ) {
			next if ( ! defined $wait );
			next if ( $wait->{'WAITING_EVENT'} =~ /^EOT/ );
			push @rwts, $wait;
		}
	}

	if ( array_has_no_rows(\@runs) &&
	     $#rwts == -1 ) {
		$page .= "<h1>Pas d'autre vnement attendu que des fins de sous-taches</h1>";
		$page .= "Pour m'envoyer un vnement: <a href=\"new-run.cgi?taskid=$tid\">ici</a>";
	}
	# Mes proprietes
	my $props = get_task_properties($tid);
	conduse('FDN::Common::Config');
	my $dir = Config_Key('adsl/global.cfg','dirs','htmltpl');
	$page .= Rewrite_Props('adsl/props.tbl',$task->{'TASK_NAME'},$dir.'/arr1.tpl',$props,
		'Proprits de la tche courante <a href="modif-taskprop.cgi?taskid='.$tid.'">(modifier)</a>');
	return $page;
}

sub get_locked_tasks($) {
	my ( $type ) = @_;
	conduse('FDN::Sql::Workflow::Basic');
	workflow_select_db('adsl');
	my @tasks = task_par_name_not_state($type,'done');
	my @res;
	for my $t ( @tasks ) {
		my $lock = is_locked_task($t);
		if ( $lock ) {
			push @res, $t;
			$t->{'LOCK'} = $lock;
		}
	}
	return @res;
}

sub is_locked_task($);
sub is_locked_task($) {
	my ( $task ) = @_;

	if ( $task->{'TASK_STATE'} eq 'failed' ) {
		return $task->{'TASK_KEY'}.' failed';
	}
	my $tid = $task->{'TASK_ID'};
	my $run  = run_par_task($tid);
	if ( defined $run ) {
		# La tache va s'executer a la prochaine occasion
		return 0;
	}
	# Pas d'execution prevue
	my @wait = waiting_par_task($tid);
	if ( array_has_no_rows(\@wait) ) {
		# La tache ne sera pas reveillee par un vnement
		return 'noevent';
	}
	my $lock;
	for my $w ( @wait ) {
		if ( $w->{'WAITING_EVENT'} =~ /^EOT-(.*)$/ ) {
			# La tache attend la fin d'une tache
			my $skey = $1;
			my $subtask = task_par_key($1);
			if ( !defined $subtask ) {
				$lock .= 'endofphantom';
				next;
			}
			my $sublock = is_locked_task($subtask);
			if ( $sublock) {
				# qui est bloquee
				$lock .= 'at '.$subtask->{'TASK_KEY'}.' on '.$sublock;
			} else {
				# qui n'est pas bloquee, on n'est donc pas bloque
				return 0;
			}
		} else {
			# La tache attend un vrai vnement
			return 0;
		}
	}
	# On attend des fin de taches qui sont toutes bloquees
	return $lock;
}

# Pour generer des clefs aleatoires:
# On vite les lettres homographes 0O 1Iil 8B 

sub clef_aleatoire($) {
	my ( $taille ) = @_;
	my @chars = ( '2'..'7', '9', 'A', 'C'..'H', 'J'..'N', 'P'..'Z', 'a'..'h', 'j'..'k', 'm'..'z');
	my $nbchars = $#chars+1;

	my $str;
	for my $i (0..$taille) {
		my $n = int(rand($nbchars));
		$str.=$chars[$n];
	}
	return $str;
}

### # Pour la gestion du RIB
### 
### sub verif_clef_rib($$$$) {
### 	my ( $banque, $guichet, $compte, $clef ) = @_;
### 	return FALSE if ( $banque == 0 );
### 	return FALSE if ( $compte == 0 );
### 	$compte =~ tr/ABCDEFGHIJKLMNOPQRSTUVWXYZ/12345678912345678923456789/;
### 	conduse('Math::BigInt');
### 	my $id = Math::BigInt->new($banque.$guichet.$compte.$clef);
### 	$id->bmod(97);
### 	if ( $id->is_zero() ) {
### 		return TRUE;
### 	} else {
### 		return FALSE;
### 	}
### }
### 





1;

