#!/usr/bin/perl

use strict;
use CGI;
use FDN::Common::Config;
use FDN::Common::Logger;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;
use FDN::Adsl::Infos;
use FDN::Sql::Workflow::Run;
use FDN::Sql::Workflow::Basic;
use POSIX qw(strftime mktime);

my $cgi = new CGI;
my $vars = $cgi->Vars;
my $page = "";

workflow_select_db('adsl');

if ( ! defined $vars->{'ligneid'} ) {
	my @tasks_liazo = task_par_name_state('cdeliazo', 'failed');
	my @cdeman_lignes;
        foreach my $task ( @tasks_liazo ) {
		my $props = get_task_properties($task->{'TASK_ID'});
		my $ligne = ligne_par_id($props->{'ligneid'});
		my $did = $props->{'deplid'};
		if ($props->{'ordretd-id'}) {
			next;
		}
		if ($ligne->{'LIGNE_STATE'} eq 'del') {
			next;
		}
		$ligne->{'DEPLOIEMENT_ID'} = $did;
		$ligne->{'CDE_TASK_ID'} = $task->{'TASK_ID'};
		$ligne->{'WAIT_TASK_ID'} = "";
		push @cdeman_lignes, $ligne;
	}

	my @tasks_liazo = task_par_name_state('wait_ok_liazo', 'failed');
        foreach my $task ( @tasks_liazo ) {
		my $props = get_task_properties($task->{'TASK_ID'});
		my $ptid = $props->{'ptid'};
		my $pprops = get_task_properties($ptid);
		my $ligne = ligne_par_id($pprops->{'ligneid'});
		my $ctask = task_par_key($pprops->{'cde-key'});
		my $did = $props->{'deplid'};
                if ($ligne->{'LIGNE_STATE'} ne 'building') {
                        next;
                }
		$ligne->{'DEPLOIEMENT_ID'} = $did;
		$ligne->{'CDE_TASK_ID'} = $ctask->{'TASK_ID'};
		$ligne->{'WAIT_TASK_ID'} = $task->{'TASK_ID'};
		push @cdeman_lignes, $ligne;
	}

	$page .= tableau_2d('adsl/rewrite.tbl','ligne-cdeman',\@cdeman_lignes);

	eject_page($page);
	exit(0);
}

my $lid = $vars->{'ligneid'};
my $dpl;
my $did = $vars->{'deplid'};
if (defined($did) && $did ne '') {
	$dpl = deploiement_par_id($did);
} else {
	$dpl = deploiement_par_ligne_id($lid);
	$did = $dpl->{'DEPLOIEMENT_ID'};
}
my $tid = $dpl->{'TASK_ID'};
my $cde_tid = $vars->{'tid'};
my $wait_tid = $vars->{'wtid'};
my $props = get_task_properties($tid);
my $eid = $props->{'eligibilite-id'};
my $elig = eligibilite_par_id($eid);
if ( $vars->{'do'} eq 'delligne' ) {
	set_ligne_state($lid,'del');
	print "Location: cdeman-lig.cgi\n\n";
	exit(0);
}
if ( $vars->{'do'} ne 'yes' ) {
	my $ligne = ligne_par_id($lid);
	my $cid = $ligne->{'CLIENT_ID'};
	my $client = client_par_id($cid);
	my $adresse = adresse_par_id($ligne->{'ADRESSE_ID'});
	my $page;
	my @infos;

	$page  = tableau_1d('adsl/rewrite.tbl','ligne',$ligne,"Description de la ligne");
	$page .= "<p>Option demande: ".$props->{'debit'}." ".$props->{'option'}."</p>";
	if ($elig) {
		$page .= "<p>Retour ligibilit: ".$elig->{'ELIGIBILITE_REPONSE'}."</p>";
	}
	$page .= tableau_1d('adsl/rewrite.tbl','adresse',$adresse,"Adresse de la ligne");
	$page .= tableau_1d('adsl/rewrite.tbl','client',$client,"Description du client");
	if ( $client->{'PRELEVEMT_ID'} ) {
		my $prel = prelevemt_par_id($client->{'PRELEVEMT_ID'});
		$page .= tableau_1d('adsl/rewrite.tbl','prelevemt',$prel,"Compte bancaire pour prelevement");
	} else {
		$page .= '<div class="error">Pas de compte bancaire pour les prelevements. Paiement par facture ?</div>'."\n\n";
	}

	@infos = toutes_infos_ligne($lid);
	$page .= tableau_2d('adsl/rewrite.tbl','infos',\@infos,"Informations sur la ligne");

	@infos = toutes_infos_client($cid);
	$page .= tableau_2d('adsl/rewrite.tbl','infos',\@infos,"Informations sur le client");

	$page .= "<p>Il faut commander  la main <a href=\"https://portal.ielo-liazo.com/private/eligibility/?view=default\">chez Liazo</a>, et quand on obtient le numro de commande PR1234-12345, l'entrer ci-dessous pour informer le SI de la commande.</p>";
	$page .= '<form method="get" action="cdeman-lig.cgi">';
	$page .= '<input type="text" name="ordretd" />';
	$page .= '<input type="hidden" name="ligneid" value="'.$lid.'"/>';
	$page .= '<input type="hidden" name="deplid" value="'.$did.'"/>';
	$page .= '<input type="hidden" name="tid" value="'.$vars->{'tid'}.'"/>';
	$page .= '<input type="hidden" name="wtid" value="'.$vars->{'wtid'}.'"/>';
	$page .= '<input type="hidden" name="do" value="yes"/>';
	$page .= '<input type="submit" value="Valider"/> ou <a href="cdeman-lig.cgi?ligneid='.$lid.'&do=delligne">Annuler la demande</a></p>';
	$page .= "</form>";
	eject_page($page);
	exit(0);
}

if ( ! ( $vars->{'ordretd'} =~ /PR[1-9][0-9]*-[1-9][0-9]*/ ) ) {
	eject_page_error("Le numro de commande '".$vars->{'ordretd'}."' est bizarre.");
	exit(0);
}

# On a besoin que l'ajout d'ordretd et la mise en attente de la tche de commande soit atomique
# pour viter que la cration de l'vnement qui va terminer cette tche se fasse entre les deux.
my $lck = lock_event();
if (!$lck) {
	eject_page_error("les vnements sont bloqus ?!");
	exit(0);
}

# Noter qui a valid la commande
$props->{'commandeur'} = $cgi->remote_user();
update_task_properties($tid, $props);

# Crer un ordretd
my $oid = new_ordretd();
set_ordretd_ligne($oid,$vars->{'ordretd'});
set_deploiement_cde($vars->{'deplid'});
$props->{'ordretd-id'} = $oid;
$props->{'ordretd'} = $vars->{'ordretd'};
update_task_properties($tid, $props);
my $cde_props = get_task_properties($cde_tid);
$cde_props->{'ordretd-id'} = $oid;
$cde_props->{'ordretd'} = $vars->{'ordretd'};
update_task_properties($cde_tid, $cde_props);
info "J'ai cr l'ordretd $oid pour l'ordre $vars->{'ordretd'} et la tche $tid";

# annuler l'chec de la tche

# Supprimer la tche d'attente ventuelle
if (defined $wait_tid && $wait_tid ne '') {
	purge_task($wait_tid);
}

# Supprimer le run init qui a chou
my $run = run_par_task($cde_tid);
if ($run) {
	del_run($run->{'RUN_ID'});
}
# La remettre running
set_task_state($cde_tid, 'running');
# et la faire attendre la validation de la commande
new_waiting($cde_tid, "ARFIC-$oid");

info "J'ai relanc la tche $vars->{'tid'}";

unlock_event();


print "Location: cdeman-lig.cgi\n\n";

