#!/usr/bin/perl

use strict;
use CGI;
#use FDN::Config;
use FDN::Common::Logger;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::Sql::Workflow::Basic;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;
#use FDN::Workflow::Wf;

workflow_select_db('adsl');
my $cgi = new CGI;

my $page = "";

my @ips = toutes_ips();

$page .= "<h1>Vrifier que seul un abonnement actif possde une IP donne</h1>";

for my $ip ( @ips ) {
	my @uattrs = uattr_par_value($ip->{'IPADDR_ADDR'});
	my @radusers;
	my $abo = (-1,-1); # ligne_id, vpn_id
	my $differents_abos = 0;
	for my $uattr ( @uattrs ) {
		my $raduser = raduser_par_id($uattr->{'RADUSER_ID'});
		if ($raduser->{RADUSER_LOGIN} !~ /\/VALID$/) {
			my $autre_abo = (
				$raduser->{'LIGNE_ID'},
				$raduser->{'VPN_ID'}
			);
			if ($abo == (-1,-1)) {
				$abo = $autre_abo;
			} elsif ($abo != $autre_abo) {
				$differents_abos = 1;
			}
			push @radusers, $raduser;
		}
	}
	if ($differents_abos) {
		$page .= tableau_2d('adsl/rewrite.tbl','raduser',\@radusers, "Doublons pour ".$ip->{'IPADDR_ADDR'});
	}
}

$page .= "<h1>Vrifier que les abonnements des IPs occupes sont actifs</h1>";

my @bogus_ips;
for my $ip ( @ips ) {
	next if $ip->{'IPADDR_ADDR'} eq '80.67.176.0'; # C'est une IP qui ne "fonctionne pas bien" sur Internet, on la garde de ct exprs
	next if ($ip->{'IPADDR_ETAT'} eq 'free');

	my $raduser = raduser_par_id($ip->{'RADUSER_ID'});
	if ($raduser->{'LIGNE_ID'} != -1) {
		my $ligne = ligne_par_id($raduser->{'LIGNE_ID'});
		if ($ligne->{'LIGNE_DTOUT'} ne '0000-00-00 00:00:00') {
			# Ligne inactive !?
			push @bogus_ips, $ip;
		}
	}
	if ($raduser->{'VPN_ID'} != -1) {
		my $vpn = vpn_par_id($raduser->{'VPN_ID'});
		if ($vpn->{'VPN_DTOUT'} ne '0000-00-00 00:00:00') {
			# VPN inactif !?
			push @bogus_ips, $ip;
		}
	}
}

sub dtout_par_raduserid($) {
	my ( $rid ) = @_;
	my $raduser = raduser_par_id($rid);
	my $lid = $raduser->{'LIGNE_ID'};
	if ($lid != -1) {
		my $ligne = ligne_par_id($lid);
		return $ligne->{'LIGNE_DTOUT'};
	}
	my $vid = $raduser->{'VPN_ID'};
	if ($vid != -1) {
		my $vpn = vpn_par_id($vid);
		return $vpn->{'VPN_DTOUT'};
	}
	return -1;
}

@bogus_ips = sort {
	dtout_par_raduserid($a->{'RADUSER_ID'})
	cmp
	dtout_par_raduserid($b->{'RADUSER_ID'})
} @bogus_ips;

if (scalar @bogus_ips) {
	$page .= tableau_2d('adsl/rewrite.tbl','ipaddr-inactive',\@bogus_ips,"");
}

$page .= "<h1>Vrifier les lignes ou vpns actifs pour un adhrent rsili</h1>";

my @clients = tous_clients();
my @bogus_lignes;
my @bogus_vpns;
for my $client ( @clients ) {
	next if ($client->{'CLIENT_ETAT'} eq 'actif');
	my @lignes = ligne_par_client_id($client->{'CLIENT_ID'});
	for my $ligne ( @lignes ) {
		if ($ligne->{'LIGNE_DTOUT'} eq '0000-00-00 00:00:00') {
			push @bogus_lignes, $ligne;
		}
	}
	my @vpns = vpn_par_client_id($client->{'CLIENT_ID'});
	for my $vpn ( @vpns ) {
		if ($vpn->{'VPN_DTOUT'} eq '0000-00-00 00:00:00') {
			push @bogus_vpns, $vpn;
		}
	}
}
if (scalar @bogus_lignes) {
	$page .= tableau_2d('adsl/rewrite.tbl','ligne',\@bogus_lignes,"");
}
if (scalar @bogus_vpns) {
	$page .= tableau_2d('adsl/rewrite.tbl','vpn',\@bogus_vpns,"");
}

my @lignes = toutes_lignes();
@bogus_lignes = ();
for my $ligne ( @lignes ) {
	next if $ligne->{'LIGNE_DTFAC'} ne '0000-00-00 00:00:00';
	my @rus = raduser_par_ligne_id($ligne->{'LIGNE_ID'});
	for my $raduser (@rus) {
		# Personne ne paie pour cette ligne, est-ce qu'elle monopolise une IP?
		my $ipaddr = ipaddr_par_raduser_id($raduser->{'RADUSER_ID'});
		next if ! defined $ipaddr;
		next if $ipaddr->{'IPADDR_ETAT'} eq 'free';
		push @bogus_lignes, $ligne;
	}
}
@bogus_lignes = sort { $a->{'LIGNE_ID'} <=> $b->{'LIGNE_ID'} } @bogus_lignes;

$page .= tableau_2d('adsl/rewrite.tbl','ligne',\@bogus_lignes,"Vrifier les lignes ayant un login radius et non finies de construire");

@bogus_lignes = ();
for my $ligne ( @lignes ) {
	my $prixid = $ligne->{PRIX_ID};
	my $prix = prix_par_id($prixid);
	next if ($prix->{PRIX_NOM} !~ /^Collecte/);

	my @rus = raduser_par_ligne_id($ligne->{'LIGNE_ID'});
	for my $raduser (@rus) {
		# Cette ligne est en collecte, est-ce qu'elle monopolise une IP?
		my $ipaddr = ipaddr_par_raduser_id($raduser->{'RADUSER_ID'});
		next if ! defined $ipaddr;
		next if $ipaddr->{'IPADDR_ETAT'} eq 'free';
		push @bogus_lignes, $ligne;
	}
}
@bogus_lignes = sort { $a->{'LIGNE_ID'} <=> $b->{'LIGNE_ID'} } @bogus_lignes;

$page .= tableau_2d('adsl/rewrite.tbl','ligne',\@bogus_lignes,"Vrifier les lignes en collecte n'ont pas d'adresse IP");

# TODO: vrifier plusieurs radius pour une ligne

eject_page($page);

