#!/usr/bin/perl

use strict;
use CGI;
#use FDN::Config;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::Sql::DBC;
use FDN::Sql::Basic;
use FDN::Sql::DB_Access;
use FDN::Sql::Workflow::Basic;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;
#use FDN::Workflow::Wf;

workflow_select_db('adsl');
my $dbh = do_connect('adsl');
my $cgi = new CGI;

my $page = "";

sub run_sql($) {
	my ( $req ) = @_;
	debug "requte SQL $req";
	my $res = sql_transaction($dbh, 'select', $req);
	if ( !$res->{'ok'} ) {
		debug "erreur sql ".$res->{'err_mess'};
		return ($res->{'err_mess'});
	}
	return '';
}

sub get_sql($$) {
	my ( $req, $champ ) = @_;
	debug "requte SQL $req";
	my $res = sql_transaction($dbh, 'select', $req);
	if ( !$res->{'ok'} ) {
		debug "erreur sql ".$res->{'err_mess'};
		return ($res->{'err_mess'});
	}
	return FDN::Sql::Basic::__return_all_col($res, $champ);
}

# Rcupre une liste d'orphelins
sub list_parent($$$$$$$) {
	my (
		$table_enfant,	# Table o on trouve les enfants
		$id_enfant,	# Identifiant qu'on va retourner
		$concat,	# ventuel appel  concat pour ajouter un prfixe au pointeur vers le parent,
		$like,		# ventuel "and like" pour filtrer les pointeurs
		$id_parent_enfant, # Nom de la colonne dans la table enfant o on trouve le pointeur vers le parent
		$table_parent,	# Table o on esprait trouver le parent
		$id_parent	# Identifiant du parent, utilis pour le pointeur
	) = @_;
	return get_sql(
		"select distinct $table_enfant.$id_enfant from $table_enfant ".
		"left join $table_parent ".
		"on $table_enfant.$id_parent_enfant = $concat $table_parent.$id_parent ) ".
		"where $table_parent.$id_parent is null ".$like, "$id_enfant");
}

# Affiche une bte liste d'identifiants
sub show_list($) {
	my ( $ids ) = @_;
	for my $id ( @$ids ) {
		$page .= " $id";
	}
}

# Affiche une bte liste d'identifiants avec un titre
sub show_triv($$) {
	my ( $ids, $title ) = @_;
	$page .= "<h2>$title</h2>";
	show_list($ids);
}

# Affiche une jolie liste d'identifiants, en utilisant la fonction get pour rcuprer tous les champs, et une section de rewrite.tbl pour faire un joli tableau
sub show($$$$) {
	my ( $ids, $get, $rewrite_label, $title ) = @_;

	my @orphelins;
	for my $id ( @$ids ) {
		my $orphelin = &$get($id);
		push @orphelins, $orphelin;
	}

	$page .= tableau_2d('adsl/rewrite.tbl', $rewrite_label, \@orphelins, $title);

	show_list($ids);
}

sub verif_parent2($$$$$$$$) {
	my ( $table_enfant, $id_enfant, $id_parent_enfant, $table_parent, $id_parent, $get, $rewrite_label, $title ) = @_;

	my @ids = list_parent($table_enfant, $id_enfant, '(', '', $id_parent_enfant, $table_parent, $id_parent);

	show(\@ids, $get, $rewrite_label, $title);
}

# Vrifie une relation entre parent et enfant, et affiche joliment le rsultat
sub verif_parent($$$$$$$) {
	my ( $table_enfant, $id_enfant, $table_parent, $id_parent, $get, $rewrite_label, $title ) = @_;
	verif_parent2($table_enfant, $id_enfant, $id_parent, $table_parent, $id_parent, $get, $rewrite_label, $title);
}

# Vrifie que quand un parent a un lien vers un enfant, cet enfant pointe bien vers ce parent
sub list_backlink($$$$$$) {
	my (
		$table_parent,	# Table o on trouve les parents
		$id_parent,	# Nom de la colonne o on y trouve l'identifiant d'un parent
		$id_enfant_parent, # Nom de la colonne dans la table parent o on trouve le pointeur vers l'enfant
		$table_enfant,	# Table o on trouve les enfants
		$id_enfant,	# Nom de la colonne o on y trouve l'identifiant d'un enfant
		$id_parent_enfant )  # Nom de la colonne dans la table enfant o on trouve le pointeur vers le parent
	= @_;
	return get_sql(
		"select distinct $table_parent.$id_parent from $table_enfant, $table_parent ".
		"where $table_parent.$id_enfant_parent = $table_enfant.$id_enfant ".
		"and $table_enfant.$id_parent_enfant != $table_parent.$id_parent ", "$id_parent");
}

sub check_backlink($$$$$$$$$) {
	my ( $table_parent, $id_parent, $id_enfant_parent, $table_enfant, $id_enfant, $id_parent_enfant, $get, $rewrite_label, $title ) = @_;
	my @list = list_backlink( $table_parent, $id_parent, $id_enfant_parent, $table_enfant, $id_enfant, $id_parent_enfant );
	show(\@list, $get, $rewrite_label, $title);
}


# Affiche une jolie liste d'infos, en utilisant la fonction get pour rcuprer l'info prcise.
sub show_info($$$) {
	my ( $ids, $get, $title ) = @_;

	my @orphelins;
	for my $id ( @$ids ) {
		my $orphelin = info_par_id($id);
		my $info = &$get($id);
		my $all = {%$orphelin, %$info};
		push @orphelins, $all;
	}

	$page .= tableau_2d('adsl/rewrite.tbl', 'infos', \@orphelins, $title);

	show_list($ids);
}

# Vrifie une relation d'info, et affiche joliment le rsultat
sub verif_info($$$$$$) {
	my ( $table_enfant, $id_enfant, $table_parent, $id_parent, $get, $title ) = @_;

	my @ids = list_parent($table_enfant, $id_enfant, '(', '', $id_parent, $table_parent, $id_parent);

	show_info(\@ids, $get, $title);
}

# Vrifie une relation entre parent et enfant, et affiche une bte liste d'identifiants
sub verif_parent_triv($$$$$) {
	my ( $table_enfant, $id_enfant, $table_parent, $id_parent, $title ) = @_;
	my @ids = list_parent($table_enfant, $id_enfant, '(', '', $id_parent, $table_parent, $id_parent);
	show_triv(\@ids, $title);
}

# Vrifie une relation d'event, et affiche une bte liste d'identifiants
sub verif_event($$$$) {
	my ( $prefix, $table_parent, $id_parent, $title ) = @_;
	my @ids = list_parent('EVENT', 'EVENT_ID', "CONCAT('$prefix',", "and EVENT.EVENT_EVENT like '$prefix%'", 'EVENT_EVENT', $table_parent, $id_parent);
	show_triv(\@ids, $title);
}

# Lister les tches d'un certain nom
sub list_task($$) {
	my ( $nom, $title ) = @_;

	my @eventids = get_sql("select TASK_ID from TASK where TASK.TASK_NAME = '$nom'", 'TASK_ID');
	show_triv(\@eventids, $title);
}

# Vrifie une relation entre parent et une tche d'un certain nom contenant un pointeur vers le parent dans une proprit
sub verif_task($$$$$) {
	my ( $nom, $prop, $table_parent, $id_parent, $title ) = @_;

	my @ids = get_sql(
		"select distinct TASK.TASK_ID from TASK, PROPTASK ".
		"left join $table_parent ".
		"on PROPTASK.PROPTASK_VAL = $table_parent.$id_parent ".
		"where $table_parent.$id_parent is null ".
		"and TASK.TASK_NAME = '$nom' ".
		"and PROPTASK.PROPTASK_NOM = '$prop' ".
		"and TASK.TASK_ID = PROPTASK.TASK_ID ", 'TASK_ID');
	show_triv(\@ids, $title);
}

# Vrifie une relation entre parent et un plutar d'une certaine action contenant un pointeur vers le parent dans une proprit
sub verif_plutar($$$$$) {
	my ( $nom, $prop, $table_parent, $id_parent, $title ) = @_;

	my @ids = get_sql(
		"select distinct PLUTAR.PLUTAR_ID from PLUTAR ".
		"left join PROPPLUTAR ".
		"on PLUTAR.PLUTAR_ID = PROPPLUTAR.PLUTAR_ID ".
		"where PROPPLUTAR.PLUTAR_ID is null ".
		"and PLUTAR.PLUTAR_ACTION = '$nom' ".
		"and PROPPLUTAR.PROPPLUTAR_NOM = '$prop' ", 'PLUTAR_ID');
	show_triv(\@ids, $title. ' sans proprit de lien');

	@ids = get_sql(
		"select distinct PLUTAR.PLUTAR_ID from PLUTAR, PROPPLUTAR ".
		"left join $table_parent ".
		"on PROPPLUTAR.PROPPLUTAR_VAL = $table_parent.$id_parent ".
		"where $table_parent.$id_parent is null ".
		"and PLUTAR.PLUTAR_ACTION = '$nom' ".
		"and PROPPLUTAR.PROPPLUTAR_NOM = '$prop' ".
		"and PLUTAR.PLUTAR_ID = PROPPLUTAR.PLUTAR_ID ", 'PLUTAR_ID');
	show_triv(\@ids, $title);
}


# On vrifie table par table, dans le mme ordre que adsl.sql


verif_parent('LIGNE', 'LIGNE_ID', 'CLIENT', 'CLIENT_ID', \&ligne_par_id, 'ligne', 'Lignes orphelines de leur adhrent.e');
verif_parent('VPN', 'VPN_ID', 'CLIENT', 'CLIENT_ID', \&vpn_par_id, 'vpn', 'VPNs orphelins de leur adhrent.e');

## ADRESSE
# mysql 5.5 trop vieux pour supporter l'optimisation de matrialistion d'un in, ou de la syntaxe with :/
$page .= run_sql("create temporary table client_adresse ".
		"( primary key (ADRESSE_ID) ) ".
		"select distinct ADRESSE_ID from CLIENT");
$page .= run_sql("create temporary table prelevemt_adresse ".
		"( primary key (ADRESSE_ID) ) ".
		"select distinct ADRESSE_ID from PRELEVEMT");
$page .= run_sql("create temporary table ligne_adresse ".
		"( primary key (ADRESSE_ID) ) ".
		"select distinct ADRESSE_ID from LIGNE");
$page .= run_sql("create temporary table vpn_adresse ".
		"( primary key (ADRESSE_ID) ) ".
		"select distinct ADRESSE_ID from VPN");
my @aids = get_sql(
	"select ADRESSE_ID from ADRESSE where ".
		"ADRESSE_ID not in (select * from prelevemt_adresse) and ".
		"ADRESSE_ID not in (select * from client_adresse) and ".
		"ADRESSE_ID not in (select * from ligne_adresse) and ".
		"ADRESSE_ID not in (select * from vpn_adresse)", 'ADRESSE_ID');
show(\@aids, \&adresse_par_id, 'adresse', "Adresses orphelines de leur adhrent.e/prlvement/ligne/vpn");

verif_parent('ADHACCESS', 'ADHACCESS_ID', 'CLIENT', 'CLIENT_ID', \&adhaccess_par_id, 'adhaccess', 'Adhaccess orphelins de leur adhrent.e');

## DEPLOIEMENT
$page .= run_sql("create temporary table deploiement_ligne ".
		"( primary key (LIGNE_ID) ) ".
		"select distinct LIGNE_ID from LIGNE");
$page .= run_sql("create temporary table deploiement_vpn ".
		"( primary key (VPN_ID) ) ".
		"select distinct VPN_ID from VPN");
my @dids = get_sql(
	"select DEPLOIEMENT_ID from DEPLOIEMENT where ".
		"LIGNE_ID not in (select * from deploiement_ligne) and ".
		"VPN_ID not in (select * from deploiement_vpn);", 'DEPLOIEMENT_ID');
show(\@dids, \&deploiement_par_id, 'deploiement', "Dploiements orphelins de leur ligne ou VPN");
verif_parent('MANDAT', 'MANDAT_ID', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', \&mandat_par_id, 'mandat', 'Mandats orphelins de leur dploiement');


# ELIGIBILITE
$page .= run_sql("create temporary table task_eligibilite ".
		"select distinct PROPTASK_VAL from PROPTASK where PROPTASK_NOM='eligibilite-id'");
my @eids = get_sql(
	"select ELIGIBILITE_ID from ELIGIBILITE where ELIGIBILITE_ID not in (select * from task_eligibilite);", 'ELIGIBILITE_ID');
show(\@eids, \&eligibilite_par_id, 'eligibilite', 'ligibilits orphelines de leur tche');

verif_parent('CREANCE', 'CREANCE_ID', 'CLIENT', 'CLIENT_ID', \&creance_par_id, 'creance', 'Crances orphelines de leur adhrent.e');
verif_parent('FACTURE', 'FACTURE_ID', 'CLIENT', 'CLIENT_ID', \&facture_par_id, 'facture', 'Factures orphelines de leur adhrent.e');
verif_parent('PAIEMENT', 'PAIEMENT_ID', 'CLIENT', 'CLIENT_ID', \&paiement_par_id, 'paiement', 'Paiements orphelins de leur adhrent.e');
verif_parent('PRELEVEMT', 'PRELEVEMT_ID', 'CLIENT', 'CLIENT_ID', \&prelevemt_par_id, 'prelevemt', 'Comptes bancaires orphelins de leur adhrent.e');
verif_parent('PRLLIG', 'PRLLIG_ID', 'CLIENT', 'CLIENT_ID', \&prllig_par_id, 'prllig', 'Prlvement ETEBAC orphelins de leur adhrent.e');
verif_parent('PRLLIG', 'PRLLIG_ID', 'PRLBQ', 'PRLBQ_ID', \&prllig_par_id, 'prllig', 'Prlvement ETEBAC orphelins de leur fichier de prlvement');
verif_parent('PRLLIGSEPA', 'PRLLIGSEPA_ID', 'CLIENT', 'CLIENT_ID', \&prlligsepa_par_id, 'prlligsepa', 'Prlvement SEPA orphelins de leur adhrent.e');
verif_parent('PRLLIGSEPA', 'PRLLIGSEPA_ID', 'PRLBQ', 'PRLBQ_ID', \&prlligsepa_par_id, 'prlligsepa', 'Prlvement SEPA orphelins de leur fichier de prlvement');

# INFO
$page .= run_sql("create temporary table client_info ".
		"( primary key (INFO_ID) ) ".
		"select distinct INFO_ID from CLIENTINFO");
$page .= run_sql("create temporary table compta_info ".
		"( primary key (INFO_ID) ) ".
		"select distinct INFO_ID from COMPTAINFO");
$page .= run_sql("create temporary table ligne_info ".
		"( primary key (INFO_ID) ) ".
		"select distinct INFO_ID from LIGNEINFO");
$page .= run_sql("create temporary table new_info ".
		"( primary key (INFO_ID) ) ".
		"select distinct INFO_ID from NEWINFO");
$page .= run_sql("create temporary table vpn_info ".
		"( primary key (INFO_ID) ) ".
		"select distinct INFO_ID from VPNINFO");
my @iids = get_sql(
	"select INFO_ID from INFO where ".
		"INFO_ID not in (select * from client_info) and ".
		"INFO_ID not in (select * from compta_info) and ".
		"INFO_ID not in (select * from ligne_info) and ".
		"INFO_ID not in (select * from new_info) and ".
		"INFO_ID not in (select * from vpn_info);", 'INFO_ID');
show(\@iids, \&info_par_id, 'infos', 'Infos orphelines de leur info adhrent.e/compta/ligne/new/vpn');

verif_info('LIGNEINFO', 'INFO_ID', 'LIGNE', 'LIGNE_ID', \&ligneinfo_par_info_id, 'Infos de lignes orphelines de leur ligne');
verif_info('CLIENTINFO', 'INFO_ID', 'CLIENT', 'CLIENT_ID', \&clientinfo_par_info_id, 'Infos adhrent.e orphelines de leur adhrent.e');
verif_info('COMPTAINFO', 'INFO_ID', 'COMPTA', 'COMPTA_ID', \&comptainfo_par_info_id, 'Infos compta orphelines de leur compta');
verif_info('VPNINFO', 'INFO_ID', 'VPN', 'VPN_ID', \&vpninfo_par_info_id, 'Infos VPN orphelines de leur VPN');

# ORDRETD
$page .= run_sql("create temporary table task_ordretd ".
		"select distinct PROPTASK_VAL from PROPTASK where PROPTASK_NOM='ordretd-id'");
my @oids = get_sql(
	"select ORDRETD_ID from ORDRETD where ORDRETD_ID not in (select * from task_ordretd);", 'ORDRETD_ID');
show(\@oids, \&ordretd_par_id, 'ordretd', 'Ordres TD orphelins de leur tche');


verif_parent('PAYANT', 'PAYANT_ID', 'CLIENT', 'CLIENT_ID', \&payant_par_id, 'payant', 'Payants orphelins de leur adhrent.e');
verif_parent('DON', 'DON_ID', 'CLIENT', 'CLIENT_ID', \&don_par_id, 'don', 'Dons orphelins de leur adhrent.e');


# RADUSER
$page .= run_sql("create temporary table raduser_ligne ".
		"( primary key (LIGNE_ID) ) ".
		"select distinct LIGNE_ID from LIGNE");
$page .= run_sql("create temporary table raduser_adhaccess ".
		"( primary key (ADHACCESS_ID) ) ".
		"select distinct ADHACCESS_ID from ADHACCESS");
$page .= run_sql("create temporary table raduser_vpn ".
		"( primary key (VPN_ID) ) ".
		"select distinct VPN_ID from VPN");
my @radids = get_sql(
	"select RADUSER_ID from RADUSER where ".
		"LIGNE_ID     not in (select * from raduser_ligne) and ".
		"ADHACCESS_ID not in (select * from raduser_adhaccess) and ".
		"VPN_ID       not in (select * from raduser_vpn) and ".
		"RADUSER_LOGIN != 'test2\@fdn.fr' and ".
		"RADUSER_LOGIN != 'test3\@fdn.fr' and ".
		"RADUSER_LOGIN != 'nagios.isengard\@vpn.fdn.fr';", 'RADUSER_ID');
show(\@radids, \&raduser_par_id, 'raduser', 'Utilisateurs RADIUS orphelins de leur ligne/adhaccess/vpn');

verif_parent('UATTR', 'UATTR_ID', 'RADUSER', 'RADUSER_ID', \&uattr_par_id, 'uattr', 'Uattr orphelins de leur raduser');


# EVENT

my @eventids = get_sql(
	'select EVENT_ID from EVENT where not ( '.
	'EVENT_EVENT like "NEW-IPADDR" or '.
	'EVENT_EVENT like "ELIG-%" or '.
	'EVENT_EVENT like "EOT-%" or '.
	'EVENT_EVENT like "FAS-%" or '.
	'EVENT_EVENT like "ARFIC-%" or '.
	'EVENT_EVENT like "ARTD-%" or '.
	'EVENT_EVENT like "CRTD-%" or '.
	'EVENT_EVENT like "ACK-%" or '.
	'EVENT_EVENT like "OK-%")', 'EVENT_ID');
show_triv(\@eventids, 'vnements avec noms inconnus');

# Tous les vnements n'ont pas forcment d'infos. Regardons seulement ceux qui en ont, mme si idalement on traquerait tout

# ELIG- : contient juste si a passe ou pas, sans numro de tlphone etc.
# EOT- : pas de proprits
# FAS- : pas de proprit
# ARFIC- : pas de proprit

verif_event('ARTD-', 'ORDRETD', 'ORDRETD_ID', 'vnements accus de rception TD orphelins de leur ordre TD');
verif_event('CRTD-', 'ORDRETD', 'ORDRETD_ID', 'vnements compte-rendu TD orphelins de leur ordre TD');

# ACK- : seulement le rsultat normal/erreur
# OK- : pas de proprits


verif_parent_triv('PROPEVENT', 'EVENT_ID', 'EVENT', 'EVENT_ID', 'EVENT_ID des Propevent orphelins de leur event');


# TASK

# git grep launch_task
my @eventids = get_sql(
	"select TASK.TASK_ID from TASK ".
	"left join TASKSELF ".
	"on TASK.TASK_ID = TASKSELF.SELF_ID ".
	"where TASKSELF.SELF_ID is null and not ( ".
	"TASK_NAME = 'mail-bur' or ".
	"TASK_NAME = 'notif-num' or ".
	"TASK_NAME = 'notif-perte' or ".
	"TASK_NAME = 'depl-vpn' or ".
	"TASK_NAME = 'depl-adsl' or ".
	"TASK_NAME = 'depl-adsl9' or ".
	"TASK_NAME = 'depl-adslL' or ".
	"TASK_NAME = 'depl-adslN' or ".
	"TASK_NAME = 'perte-ligne-normal' or ".
	"TASK_NAME = 'perte-ligne-anormal' or ".
	"TASK_NAME = 'change-adsl9' or ".
	"TASK_NAME = 'change-adslN' or ".
	"TASK_NAME = 'change-adslL' or ".
	"TASK_NAME = 'resil-vpn' or ".
	"TASK_NAME = 'resil-adsl')", 'TASK_ID');
show_triv(\@eventids, 'Tches avec noms inconnus');

# il ne devrait pas y en avoir
list_task('mail-bur', 'Tches mail-bur');
# on ne devrait plus en avoir
list_task('notif-num', 'Tches notif-num');

verif_task('depl-vpn', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de dploiement de VPN');
verif_task('depl-adsl', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de dploiement ADSL TD');
verif_task('depl-adsl9', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de dploiement ADSL 9');
verif_task('depl-adslL', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de dploiement ADSL Liazo');
verif_task('depl-adslN', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de dploiement ADSL Nerim');
verif_task('notif-perte', 'ligneid', 'LIGNE', 'LIGNE_ID', 'Tches de perte de ligne');
verif_task('perte-ligne-normal', 'ligneid', 'LIGNE', 'LIGNE_ID', 'Tches de perte de ligne normal');
verif_task('perte-ligne-anormal', 'ligneid', 'LIGNE', 'LIGNE_ID', 'Tches de perte de ligne anormal');
verif_task('change-adsl9', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de changement de ligne 9');
verif_task('change-adslN', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de changement de ligne Nerim');
verif_task('change-adslL', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Tches de changement de ligne Liazo');
verif_task('resil-vpn', 'vpnid', 'VPN', 'VPN_ID', 'Tches de rsiliation de VPN');
verif_task('resil-adsl', 'ligneid', 'LIGNE', 'LIGNE_ID', 'Tches de rsiliation de ligne ADSL');

verif_parent_triv('PROPTASK', 'TASK_ID', 'TASK', 'TASK_ID', 'TASK_ID de Proptask orphelins de leur tche');
my @ids = list_parent('TASKSELF', 'TASK_ID', '(', '', 'TASK_ID', 'TASK', 'TASK_ID');
show_triv(\@ids, 'Taskself orphelins de la tche parente');
my @ids = list_parent('TASKSELF', 'SELF_ID', '(', '', 'SELF_ID', 'TASK', 'TASK_ID');
show_triv(\@ids, 'Taskself orphelins de la tche enfante');
verif_parent_triv('RUN', 'RUN_ID', 'TASK', 'TASK_ID', 'Run orphelins de leur tche');
verif_parent_triv('WAITING', 'WAITING_ID', 'TASK', 'TASK_ID', 'Waiting orphelins de leur tche');


# PLUTAR
my @eventids = get_sql(
	"select PLUTAR.PLUTAR_ID from PLUTAR ".
	"where not ( ".
	"PLUTAR_ACTION = 'elig' or ".
	"PLUTAR_ACTION = 'elig9' or ".
	"PLUTAR_ACTION = 'eligN' or ".
	"PLUTAR_ACTION = 'eligL' or ".
	"PLUTAR_ACTION = 'commandeL' or ".
	"PLUTAR_ACTION = 'gpgp' or ".
	"PLUTAR_ACTION = 'resil_change_radius' or ".
	"PLUTAR_ACTION = 'attends')", 'PLUTAR_ID');
show_triv(\@eventids, 'Plutars avec noms inconnus');

verif_plutar('elig', 'eid', 'ELIGIBILITE', 'ELIGIBILITE_ID', 'Plutar d\'ligibilit');
verif_plutar('elig9', 'eid', 'ELIGIBILITE', 'ELIGIBILITE_ID', 'Plutar d\'ligibilit 9');
verif_plutar('eligN', 'eid', 'ELIGIBILITE', 'ELIGIBILITE_ID', 'Plutar d\'ligibilit Nerim');
verif_plutar('eligL', 'eid', 'ELIGIBILITE', 'ELIGIBILITE_ID', 'Plutar d\'ligibilit Liazo');
verif_plutar('commandeL', 'deplid', 'DEPLOIEMENT', 'DEPLOIEMENT_ID', 'Plutar de commande de ligne Liazo');
verif_plutar('gpgp', 'adresse-id', 'ADRESSE', 'ADRESSE_ID', 'Plutar de rcupration PGP');
verif_plutar('resil_change_radius', 'radid', 'RADUSER', 'RADUSER_ID', 'Plutar de changement/rsiliation Radius');
# pas d'informations autre que le tid, flemme de nettoyer
#verif_plutar('attends', 'tid', 'TASK', 'TASK_ID', 'Plutar d\'attente');

verif_parent_triv('PROPPLUTAR', 'PROPPLUTAR_ID', 'PLUTAR', 'PLUTAR_ID', 'Propplutar orphelins de leur plutar');

verif_parent('SITE', 'SITE_ID', 'CLIENT', 'CLIENT_ID', \&site_par_id, 'site', 'Sites orphelins de leur adhrent.e');
verif_parent('DOMAINE', 'DOMAINE_ID', 'CLIENT', 'CLIENT_ID', \&domaine_par_id, 'domaine', 'Domaines orphelins de leur adhrent.e');
verif_parent('NOTIF', 'NOTIF_ID', 'LIGNE', 'LIGNE_ID', \&notif_par_id, 'notif', 'Notifications orphelines de leur ligne');
verif_parent('UNIXUSER', 'UNIXUSER_ID', 'CLIENT', 'CLIENT_ID', \&unixuser_par_id, 'unixuser', 'Utilisateurs Unix orphelins de leur adhrente.');
verif_parent_triv('UNIXUSERUNIXGROUP', 'UNIXUSER_ID', 'UNIXUSER', 'UNIXUSER_ID', 'Relations user/group Unix orphelines de l\'user');
verif_parent_triv('UNIXUSERUNIXGROUP', 'UNIXUSER_ID', 'UNIXGROUP', 'UNIXGROUP_ID', 'Relations user/group Unix orphelines du groupe');
verif_parent('VALIDATION', 'VALIDATION_ID', 'LIGNE', 'LIGNE_ID', \&validation_par_id, 'validation', 'Validations orphelines de leur ligne');
verif_parent('VALIDATION', 'VALIDATION_ID', 'CLIENT', 'CLIENT_ID', \&validation_par_id, 'validation', 'Validations orphelines de leur adhrent.e');
verif_parent('CONVOCATION', 'CONVOCATION_ID', 'CLIENT', 'CLIENT_ID', \&convocation_par_id, 'convocation', 'Convocations orphelines de leur adhrent.e');
verif_parent('VOTE', 'VOTE_ID', 'CLIENT', 'CLIENT_ID', \&vote_par_id, 'vote', 'Votes orphelins de leur adhrent.e');


sub check_dir_image_info($$) {
	my ( $dir, $title ) = @_;
	debug "looking in $dir";
	$page .= "<h2>$title ($dir)</h2>";
	opendir DIR, $dir or $page .= "<p>Pas pu ouvrir: $!</p>";
	while ( my $name = readdir(DIR) ) {
		next if $name eq '.';
		next if $name eq '..';
		$name =~ s/.jpg$//;
		my $info = info_par_val($name);
		if (!defined ($info) or $info->{INFO_TYPE} eq 'inline') {
			$page .= "$name<br/>";
		}
	}
	closedir DIR;
}

my $imgdir = Config_Key('adsl/global.cfg','dirs','imgdir');
my $prldir = Config_Key('adsl/global.cfg','dirs','prldir');
my $stockpdf = Config_Key('adsl/global.cfg','dirs','stockpdf');
my $tdout = Config_Key('adsl/global.cfg','dirs','tdout');
my $tdoutdone = Config_Key('adsl/global.cfg','dirs','tdoutdone');
my $ntout = Config_Key('adsl/global.cfg','dirs','9tout');
my $ntoutdone = Config_Key('adsl/global.cfg','dirs','9toutdone');
my $nerimout = Config_Key('adsl/global.cfg','dirs','nerimout');
my $nerimoutdone = Config_Key('adsl/global.cfg','dirs','nerimoutdone');

check_dir_image_info("$imgdir/test", "Images test sans info");

check_dir_image_info("$imgdir/jpeg", "Images jpeg sans info");

$page .= "<h2>Thumbnail sans image ($imgdir/thumbnails)</h2>";
opendir DIR, "$imgdir/thumbnails" or $page .= "<p>Pas pu ouvrir: $!</p>";
while ( my $name = readdir(DIR) ) {
	next if $name eq '.';
	next if $name eq '..';
	$name =~ s/^tn-//;
	if (! -e "$imgdir/jpeg/$name") {
		$page .= "$name<br/>";
	}
}
closedir DIR;

sub list_dir($$) {
	my ( $dir, $title ) = @_;
	$page .= "<h2>$title ($dir)</h2>";
	debug "looking in $dir";
	opendir DIR, $dir or $page .= "<p>Pas pu ouvrir: $!</p>";
	while ( my $name = readdir(DIR) ) {
		next if $name eq '.';
		next if $name eq '..';
		next if $name eq 'logoFDN.png';
		next if $name eq 'fdnfactu.sty';
		next if $name eq 'prelevement.tex';
		next if $name eq 'prelevement.log';
		next if $name eq 'prelevement.aux';
		next if $name eq 'prelevement.pdf';
		$page .= "$name<br/>";
	}
	closedir DIR;
}
list_dir("$imgdir/wrong", "Wrong");

sub check_dir_fichier($$$$$) {
	my ( $dir, $prefix, $suffix, $table_par_fichier, $title ) = @_;
	$page .= "<h2>$title ($dir)</h2>";
	debug "looking in $dir";
	opendir DIR, "$dir" or $page .= "<p>Pas pu ouvrir: $!</p>";
	while ( my $name = readdir(DIR) ) {
		next if $name eq '.';
		next if $name eq '..';
		next if $name eq 'logoFDN.png';
		next if $name eq 'fdnfactu.sty';
		next if $name eq 'factprl.tex';
		next if $name eq 'factprl.log';
		next if $name eq 'factprl.aux';
		if ($name =~ /$prefix(.*)$suffix/) {
			my $num = $1;
			my $e = &$table_par_fichier($num);
			if (!defined ($e)) {
				$page .= "$name<br/>";
			}
		} else {
			$page .= "$name<br/>";
		}
	}
	closedir DIR;
}
check_dir_fichier("$tdout", "", "", \&ordretd_par_fichier, 'Fichiers ordretd TD sans ordretd');
check_dir_fichier("$tdoutdone", "", "", \&ordretd_par_fichier, 'Fichiers ordretd TD sans ordretd');
check_dir_fichier("$ntout", "", "", \&ordretd_par_fichier, 'Fichiers ordretd 9T sans ordretd');
check_dir_fichier("$ntoutdone", "", "", \&ordretd_par_fichier, 'Fichiers ordretd 9T sans ordretd');
check_dir_fichier("$nerimout", "", "", \&ordretd_par_fichier, 'Fichiers ordretd Nerim sans ordretd');
check_dir_fichier("$nerimoutdone", "", "", \&ordretd_par_fichier, 'Fichiers ordretd Nerim sans ordretd');

check_dir_fichier("$prldir", "", "", \&prlbq_par_fichier, 'Fichiers de prlvement sans prlvement');

sub check_glob_client($$$) {
	my ( $glob, $prefix, $title ) = @_;
	$page .= "<h2>$title ($glob)</h2>";
	debug "looking in $glob";
	for my $name ( glob $glob ) {
		next if $name eq '.';
		next if $name eq '..';
		if ($name =~ /^$prefix(.*)/) {
			my $id = $1;
			my $client = client_par_id($id);
			if (!defined $client) {
				$page .= "$id ";
			}
		}
	}
}

check_glob_client("$stockpdf/adhacc-*", "$stockpdf/adhacc-", "Stock pdf adhsion sans adhrent");
check_glob_client("$stockpdf/prllig/*", "$stockpdf/prllig/", "Stock pdf prlvement sans adhrent");

sub check_glob_path($$$) {
	my ( $glob, $table_par_fichier, $title ) = @_;
	$page .= "<h2>$title ($glob)</h2>";
	debug "looking in $glob";
	for my $name ( glob $glob ) {
		next if $name eq '.';
		next if $name eq '..';
		my $e = &$table_par_fichier("$name");
		if (!defined ($e)) {
			$page .= "$name<br/>";
		}
	}
}
check_glob_path("$stockpdf/mandat/*/*.pdf", \&mandat_par_fichier, 'Fichiers mandats sans mandats');

check_dir_fichier("$stockpdf/factures", "fact-", ".pdf", \&facture_par_num, 'Fichiers factures sans facture');

#  voir si on veut virer les fichiers  mesure, ou bien virer les prlbq  mesure.
# Cf mail SI Date: Wed, 15 Apr 2020 11:24:19 +0200
#check_dir_fichier("$stockpdf/prlbq", "prlbq-", "-[0-9]*.pdf", \&prlbq_par_date, 'Fichiers prlvement sans prlvement');
list_dir("$stockpdf/prlbq", "Vieux prlbq");

$page .= "<h2>Documents dploiement sans dploiement ($stockpdf/*)</h2>";
opendir DIR, "$stockpdf" or $page .= "<p>Pas pu ouvrir: $!</p>";
while ( my $name = readdir(DIR) ) {
	next if $name eq '.';
	next if $name eq '..';
	next if $name =~ '^adhacc-';
	next if $name eq 'prllig';
	next if $name eq 'mandat';
	next if $name eq 'factures';
	next if $name eq 'prlbq';

	my $deploiement = deploiement_par_num($name);
	if (!defined $deploiement) {
		$page .= "$name<br/>";
	}
}
closedir DIR;


check_backlink("CLIENT", "CLIENT_ID", "PRELEVEMT_ID", "PRELEVEMT", "PRELEVEMT_ID", "CLIENT_ID", \&client_par_id, 'client', "Adhrents dont le compte bancaire actif ne leur appartient pas");

eject_page($page);
