#!/usr/bin/perl

use strict;
use CGI;
use FDN::Common::Config;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;
use FDN::Adsl::Infos;
use FDN::Adsl::SelL;
use FDN::Sql::Workflow::Run;
use FDN::Sql::Workflow::Basic;
use POSIX qw(strftime mktime);

my $cgi = new CGI;
my $vars = $cgi->Vars;

workflow_select_db('adsl');

if ( ! defined $vars->{'ligneid'} ) {
	my @lignes = ligne_par_state('new');
	@lignes = sort { $a->{'LIGNE_ID'} <=> $b->{'LIGNE_ID'} } @lignes;
	my @leslignes;
	foreach my $ligne ( @lignes ) {
		my $event = 'FAS-'.$ligne->{'LIGNE_ID'};
		if (event_par_event($event)) {
			# Dj valid, juste pas encore trait par le cron
			next;
		}
		push @leslignes, $ligne;
	}
	my $page = tableau_2d('adsl/rewrite.tbl','ligne-fas',\@leslignes);

	my @tasks_liazo = task_par_name_state('change-adslL', 'running');
	my @change_lignes;
        foreach my $task ( @tasks_liazo ) {
		if ($task->{'TASK_NBCHILD'} != 2) {
                        # L'ligibilit n'a pas encore t teste,
			# ou Les FAS ont dj t confirms
			next;
		}
		my $props = get_task_properties($task->{'TASK_ID'});
		my $did = $props->{'deplid'};
		my $ligne = ligne_par_id($props->{'ligneid'});
		$ligne->{'DEPLOIEMENT_ID'} = $did;
		my $suff = "";
		if ($ligne->{'LIGNE_STATE'} ne 'new') {
			$suff = "-".$did;
		}
		my $event = 'FAS-'.$ligne->{'LIGNE_ID'}.$suff;
		if (event_par_event($event)) {
			# Dj valid, juste pas encore trait par le cron
			next;
		}
		push @change_lignes, $ligne;
	}

	$page .= tableau_2d('adsl/rewrite.tbl','ligne-change',\@change_lignes);

	@lignes = ligne_par_state('building');
	@lignes = sort { $a->{'LIGNE_DTIN'} cmp $b->{'LIGNE_DTIN'} } @lignes;
	$page .= tableau_2d('adsl/rewrite.tbl','ligne',\@lignes,'Lignes en construction');

	eject_page($page);
	exit(0);
}

my $lid = $vars->{'ligneid'};
my $ligne = ligne_par_id($lid);
my $dpl;
my $did = $vars->{'deplid'};
if (defined($did)) {
	$dpl = deploiement_par_id($did);
} else {
	$dpl = deploiement_par_ligne_id($lid);
	$did = $dpl->{'DEPLOIEMENT_ID'};
}
my $tid = $dpl->{'TASK_ID'};
my $props = get_task_properties($tid);
my $eid = $props->{'eligibilite-id'};
my $elig = eligibilite_par_id($eid);
if ( $vars->{'do'} eq 'delligne' ) {
	my $waiting = waiting_par_event('FAS-'.$lid);
	if ( defined $waiting ) {
		del_waiting($waiting->{'WAITING_ID'});
	}
	$waiting = waiting_par_task($tid);
	if ( defined $waiting ) {
		del_waiting($waiting->{'WAITING_ID'});
	}
	set_ligne_state($lid,'del');
	set_ligne_tel($lid,$ligne->{'LIGNE_TEL'}.'.r'.$tid);
	print "Location: paye-fas.cgi\n\n";
	exit(0);
}
if ( $vars->{'do'} eq 'purgetask' ) {
	purge_task($tid);
	print "Location: paye-fas.cgi\n\n";
	exit(0);
}
if ( $vars->{'do'} ne 'yes' ) {
	my $cid = $ligne->{'CLIENT_ID'};
	my $client = client_par_id($cid);
	my $adresse = adresse_par_id($ligne->{'ADRESSE_ID'});
	my $prix = prix_par_id($props->{'tarif'});
	my $page;
	my @infos;

	$page  = tableau_1d('adsl/rewrite.tbl','ligne',$ligne,"Description de la ligne");
	$page .= tableau_1d('adsl/rewrite.tbl','deploiement',$dpl,"Suivi de dploiement");
	$page .= "<p>Numro demand: ".$props->{telephone}."</p>";
	$page .= "<p>Type du numro demand: ".$props->{typend}."</p>";
	$page .= "<p>Option demande: ".$props->{'debit'}." ".$props->{'option'}."</p>";
	my $ret = 'pas encore vrifi';
	if ($elig) {
		$ret = $elig->{'ELIGIBILITE_REPONSE'};
		$ret =~ s/\n/, /g;
	}
	$page .= "<p>Retour ligibilit au moment de la commande: ".$ret."</p>";
	my $addr = get_addr_nd_pto($props->{cp}, $props->{refimmeuble}, $props->{telephone}, $props->{typend});
	$addr =~ s/\n/, /g;
	$page .= "<p>Adresse maintenant Adresse ligne selon Liazo: ".$addr;
	$page .= tableau_1d('adsl/rewrite.tbl','prix',$prix,"Tarif");
	$page .= tableau_1d('adsl/rewrite.tbl','adresse',$adresse,"Adresse de la ligne");
	$page .= tableau_1d('adsl/rewrite.tbl','client',$client,"Description du client");
	if ( $client->{'PRELEVEMT_ID'} ) {
		my $prel = prelevemt_par_id($client->{'PRELEVEMT_ID'});
		$page .= tableau_1d('adsl/rewrite.tbl','prelevemt',$prel,"Compte bancaire pour prelevement");
	} else {
		$page .= '<div class="error">Pas de compte bancaire pour les prelevements. Paiement par facture ?</div>'."\n\n";
	}

	@infos = toutes_infos_ligne($lid);
	$page .= tableau_2d('adsl/rewrite.tbl','infos',\@infos,"Informations sur la ligne");

	@infos = toutes_infos_client($cid);
	$page .= tableau_2d('adsl/rewrite.tbl','infos',\@infos,"Informations sur le client");

	if ($ligne->{'LIGNE_STATE'} eq 'new') {
		$page .= "<p>Il faut vrifier la prsence de tous ces lments:<br>\n".
			 "<b>Nouvel adhrent:</b></p>\n".
			 "<ul>\n".
			 "<li>l'image du bulletin d'adhsion;</li>\n".
			 "<li>l'image de l'autorisation de prlvement;</li>\n".
			 "<li>la saisie du RIB pour le prlvement (l'item &nbsp;Paiment&nbsp;\n".
			 "    dans la description du client ne doit pas mentionner de Facture, mais\n".
			 "    l'identifiant du RIB);</li>\n".
			 "<li>l'image de la pice d'identit de l'adhrent;</li>\n".
			 "<li>l'image du justificatif (carte d'tudiant, etc) si l'adhrent\n".
			 "    demande le tarif prfrentiel.\n".
			 "</ul>\n".
			 "<p><b>Nouvelle ligne xDSL/FTTH:</b></p>\n".
			 "<ul>\n".
			 "<li>l'image du bulletin d'abonnement pour la ligne (deux pages, donc\n".
			 "    deux images)</li>\n".
			 "<li>l'image de la facture Orange</li>\n".
			 "<li>ou l'image d'un justificatif de domicile correspondant  l'adresse selon l'ligibilit</li>\n".
			 #"<li>l'image du mandat de dgroupage, si le dbit souscrit est en\n".
			 #"    dgroup</li>\n".
			 "</ul>\n".
			 "<p>Si un des lments manque, il faut retourner le scanner, ou attendre\n".
			 "de le recevoir selon le cas. <b>Ne pas valider de ligne s'il manque des\n".
			 "documents!</b></p>";

		$page .= '<form method="get" action="paye-fas.cgi">';
		$page .= "<p>La construction sera demande  la date entre ci-dessous.</p>";
		$page .= '<input type="text" name="date" value="'.$props->{'date'}.'"/>';
		$page .= '<input type="hidden" name="ligneid" value="'.$lid.'"/>';
		$page .= '<input type="hidden" name="do" value="yes"/>';
		$page .= '<input type="submit" value="Valider"/> ou <a href="paye-fas.cgi?ligneid='.$lid.'&do=delligne">Annuler la demande</a></p>';
		$page .= "</form>";
	} else {

		my $today = time;
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($today);

		# contrainte de date de Nerim: envoi de rsiliation au plus tard le 15 du 2e
		# mois du bimestre.

		# passer au dbut du bimestre
		$mon -= $mon % 2;
		# deadline le 16 du 2e mois du bimestre
		my $bimestre_deadline = mktime(0, 0, 0, 16, $mon+1, $year);

		# avec de la chance, on peut migrer au dbut du bimestre suivant
		$mon += 2;
		if ($today > $bimestre_deadline)
		{
			# Trop tard, on est oblig de passer au bimestre suivant
			$mon += 2;
		}
		if ($mon >= 12) {
			$mon -= 12;
			$year += 1;
		}
		my $debut = mktime(0, 0, 0, 1, $mon, $year);

		# On commande la migration un jour avant
		my $date = strftime "%Y-%m-%d %H:%M:%S", localtime ($debut - 1*24*3600);

		$page .= '<form method="get" action="paye-fas.cgi">';
		$page .= "<p>Il faut lancer la rsiliation de la ligne chez Nrim avant de valider ci-dessous.</p>";
		$page .= "<p>Indiquer ci-dessous la date  laquelle on veut faire lancer la commande Liazo par le SI:</p>\n";
		$page .= '<input type="text" name="date" value="'.$date.'"/>';
		$page .= '<input type="hidden" name="ligneid" value="'.$lid.'"/>';
		$page .= '<input type="hidden" name="deplid" value="'.$did.'"/>';
		$page .= '<input type="hidden" name="do" value="yes"/>';
		$page .= '<input type="submit" value="Valider"/> ou <a href="paye-fas.cgi?ligneid='.$lid.'&deplid='.$did.'&do=purgetask">Annuler la demande</a></p>';
		$page .= "</form>";
	}
	eject_page($page);
	exit(0);
}

# Noter qui a valid la commande et la date confirme
my $newprop = { 'validateur' => $cgi->remote_user(), 'date' => $vars->{'date'} };
set_task_properties($tid, $newprop);

my $ligne = ligne_par_id($lid);
my $cid = $ligne->{'CLIENT_ID'};
my $client = client_par_id($cid);
if ($client->{'CLIENT_NUMADH'} < 0 && $client->{'CLIENT_TYPE'} ne 'blanche' && $client->{'CLIENT_TYPE'} ne 'collecte') {
	eject_page_error("Le.la futur.e abonn.e n'est pas encore adhrent, il faut d'abord vrifier son adhsion et cliquer sur 'Initialiser'");
	exit(0);
}

my $tel = $ligne->{'LIGNE_TEL'};
my $suff = "";
if ($ligne->{'LIGNE_STATE'} ne 'new') {
	$suff = "-".$did;
}
my $event = 'FAS-'.$lid.$suff;

# Mise en facturation sera faite en fin de construction.
#set_ligne_dtfac($lid);
send_event($event);
#TODO Creer une creance negative correspondant au paiement.
my $index = Config_Key('adsl/global.cfg','web','index');
print "Location: paye-fas.cgi\n\n";

