#!/usr/bin/perl

use strict;
use CGI;
use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Web::TemplateParse;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::TeX::TeX;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;
use FDN::Adsl::Factu;
use FDN::Adsl::Wf;

set_tex_config('adsl/tex.cfg');
my $cgi = new CGI;
my $vars = $cgi->Vars;


my $form = form_read('adsl/form/factu.cfg','pprllig');
# Ajoute la liste des prelevements dans l'ordre (dus, envoyes, payes)
ajoute_liste_prelevements($form);
if ( $vars->{'do'} ne 'yes' ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}
if ( !check_transition($form,$vars) ) {
	eject_page(affiche_form($form,$vars));
	exit(0);
}

# Affichage du prelevement

my $pid = $vars->{prlbq};
$pid =~ s/^prlbq//;
my $prlbq = prlbq_par_id($pid);
my $type;
my $table;
if ( $prlbq->{'PRLBQ_TYPE'} eq 'sepa') {
	$type = 'sepa';
	$table = 'PRLLIGSEPA';
} else {
	$type = 'etebac';
	$table = 'PRLLIG';
}

my @pls;
if ( $type eq 'sepa' ) {
	@pls = prlligsepa_par_prlbq_id($pid);
} else {
	@pls = prllig_par_prlbq_id($pid);
}


my $nb_gen = 0;
my $nb_err = 0;
my $page;
for my $p ( @pls ) {
	my $plid = $p->{$table.'_ID'};
	my $cid  = $p->{'CLIENT_ID'};
	my $dir = Config_Key('adsl/global.cfg','dirs','stockpdf');
	my $file = $dir.'/prllig/'.$cid.'/prl-'.$prlbq->{'PRLBQ_DATE'}.'-'.$plid.'.pdf';
	if ( ! -f $file ) {
		my $hash = {};
		$hash->{'cid'} = $cid;

		my @crs;
		if ( $type eq 'sepa' ) {
			@crs = creance_par_prlligsepa_id($plid);
		} else {
			@crs = creance_par_prllig_id($plid);
		}
		if ( ! @crs ) {
			# Il n'y a pas de creances. Probablement suite a rejet et re-prelevement.
			next;
		}
		my $date = '9999-99-99 99:99:99';
		for my $c ( @crs ) {
			if ( $c->{'CREANCE_DATEDUE'} lt $date ) {
				$date = $c->{'CREANCE_DATEDUE'};
			}
		}
		# Attention, le tableau est plein de balises TeX, ne SURTOUT PAS l'encoder !
		$hash->{tableau} = tableau_2d_pognon_tex('adsl/rewrite.tbl','facture-tex','CREANCE',\@crs);
		$hash->{pid} = tex_encode($plid);
		my $tmp = $date;
		$tmp =~ s/^(\d\d\d\d)-(\d\d)-(\d\d) .*$/$1$2$3/;
		$hash->{numero} = tex_encode($tmp.'-'.$cid.'-'.$pid.'-'.$plid);
		my $cl = client_par_id($cid);
		my $adr = adresse_par_id($cl->{'ADRESSE_ID'});
		$tmp = 	$cl->{'CLIENT_NOM'}."\n".
			$adr->{'ADRESSE_NOM'}."\n".
			$adr->{'ADRESSE_ADR'}."\n".
			$adr->{'ADRESSE_CP'}." ".$adr->{'ADRESSE_VILLE'};
		$hash->{adresse} = tex_encode($tmp);
		$hash->{dateprl} = tex_encode($prlbq->{'PRLBQ_DATE'});
		$hash->{date} = tex_encode($date);
		if ( $type eq 'sepa' ) {
			$hash->{'bic'} = tex_encode($p->{'PRLLIGSEPA_BIC'});
			$hash->{'iban'} = tex_encode($p->{'PRLLIGSEPA_IBAN'});
		} else {
			$hash->{banque} = tex_encode($p->{'PRLLIG_BANQUE'});
			$hash->{guichet} = tex_encode($p->{'PRLLIG_GUICHET'});
			$hash->{compte} = tex_encode($p->{'PRLLIG_COMPTE'});
		}
		my $res = make_pdf_file('prllig',$cid,$hash);
		system("cp $res $file");
		$nb_gen++;
	}
	if ( -f $file ) {
		if ( $type eq 'sepa' ) {
			$page .= "<p>Fichier <a href=\"print-prllig2.cgi?sepa_pid=$plid\">$file</a>.</p>";
		} else {
			$page .= "<p>Fichier <a href=\"print-prllig2.cgi?pid=$plid\">$file</a>.</p>";
		}
	} else {
		$page .= "<div class=\"error\">Fichier $file en echec.</div>";
		$nb_err++;
	}
}

if ($nb_gen) {
	envoie_mail('tresorier@fdn.fr', "treso-memos", {
		'nb_gen', $nb_gen,
		'nb_err', $nb_err,
		});
}

eject_page($page);



