#!/usr/bin/perl

use strict;
use CGI;
use FDN::Web::Forms;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;
use JSON::Any;

my $cgi = new CGI;
my $vars = $cgi->Vars;

my $cfg = 'adsl/subs/eligibilite.cfg';
my $tbl = 'adsl/subs/eligibilite.tbl';

if (!defined $vars->{typend}) {
	# compatibilit avec l'ancienne interface: que du Nerim donc actif
	$vars->{typend} = "actif";
}

if ( $vars->{'etape'} eq 'eligibilite' ) {
	eligibilite();
} else {
	erreur();
}

sub eligibilite() {
	conduse('FDN::Adsl::Basic');
	conduse('FDN::Adsl::Souscription');
	my $tarifs = $FDN::Adsl::Souscription::tarifs;
	my @offres = @$FDN::Adsl::Souscription::offres;
	my $form = form_read ( $cfg, 'offre' );
	
	my $retour = {};
	$retour->{erreur} = 0;

	my $tarif = $vars->{tarif};
	my $tel = $vars->{tel};
	$tel =~ s/^MIGRATION//;

	if ($tel =~ /^FTTH/) {
		@offres = ("1GK1LiazoKosc", "300MK1LiazoKosc", "1GK2LiazoKosc", "300MK2LiazoKosc", "1GB1LiazoBouygues", "1GB2LiazoBouygues");
	}

	my $elig_offres = {};
	if ( $vars->{typend} eq "actif" && Config_Key('adsl/subs.cfg', 'GLOBAL', 'use_nerim') ) {
		conduse('FDN::Adsl::SelN');
		my $elig = FDN::Adsl::SelN::eligibilite($tel,$vars->{cp},\@offres);
		if ( $elig->{etat} == 1 ) {
			$retour->{erreur} = 1;
			$retour->{message} = "Une erreur technique sur le serveur d'ligibilit ".
				   "nous met dans ".
				   "l'impossibilit de tester l'ligilit de votre ligne. Merci de ".
				   "ressayer plus tard.\n".
				   "Le texte des messages d'erreur:\n";
			for my $m ( @{$elig->{'message'}} ) {
				$retour->{message} .= $m."\n";
			}
		}
		if ( $elig->{etat} == 2 ) {
			$retour->{erreur} = 1;
			$retour->{message} = "Une erreur technique chez FDN empche de poursuivre votre inscription\n";
		}
		if ( $elig->{etat} == 3 ) {
			$retour->{erreur} = 2;
			my $page = "Il y a un problme avec le numro de tlphone que vous avez saisi.\n";
			for my $m ( @{$elig->{'message'}} ) {
				$retour->{message} .= $m."\n";
			}
		}
		if ( $elig->{etat} ) {
			$retour->{erreur} = 1;
			$retour->{message} = "Une erreur technique GRAVE chez FDN empche de traiter votre demande. Merci de ".
				   "transmettre l'information a FDN.\n";
		}
		$elig_offres = { %$elig_offres, %{$elig->{offres}} };
	}

	my $whitelist = Config_Key('adsl/elig/liazo.cfg', 'ariane', '@whitelist');
	if ( ( Config_Key('adsl/subs.cfg', 'GLOBAL', 'use_liazo') && $vars->{liazo} ) || grep { $_ eq $vars->{tel} } @$whitelist ) {
		conduse('FDN::Adsl::SelL');
		my $elig2 = FDN::Adsl::SelL::eligibilite_liazo($tel,$vars->{typend},$vars->{refimmeuble},$vars->{cp},\@offres);
	
		if ( $elig2->{etat} == 1 ) {
			$retour->{erreur} = 1;
			$retour->{message} = "Une erreur technique sur le serveur d'ligibilit ".
				   "nous met dans ".
				   "l'impossibilit de tester l'ligilit de votre ligne. Merci de ".
				   "ressayer plus tard.\n".
				   "Le texte des messages d'erreur:\n";
			for my $m ( @{$elig2->{'message'}} ) {
				$retour->{message} .= $m."\n";
			}
		}
		if ( $elig2->{etat} == 2 ) {
			$retour->{erreur} = 1;
			$retour->{message} = "Une erreur technique chez FDN empche de poursuivre votre inscription\n";
		}
		if ( $elig2->{etat} == 3 ) {
			$retour->{erreur} = 2;
			my $page = "Il y a un problme avec le numro de tlphone que vous avez saisi.\n";
			for my $m ( @{$elig2->{'message'}} ) {
				$retour->{message} .= $m."\n";
			}
		}
		if ( $elig2->{etat} ) {
			$retour->{erreur} = 1;
			$retour->{message} = "Une erreur technique GRAVE chez FDN empche de traiter votre demande. Merci de ".
				   "transmettre l'information a FDN.\n";
		}
		$elig_offres = { %$elig_offres, %{$elig2->{offres}} };
	}

	$tel =~ s/^VOISIN//;
	$tel =~ s/^FTTH//;

	if ( $retour->{erreur} == 0) {

	my $rejets;
	my $debits = 0;
	my $infoligne = {};
	my @offres_dispo;
	for my $d ( @offres ) {
		my $option = $d;
		my $debit = $d;
		my $info_offre = {};

		if ( ($d eq '512kopt5') or ($d eq '512REopt5') or ($d eq '512kLiazoOrange') or ($d eq '512kLiazoOrangeNu') ) {
			my $info = $elig_offres->{$d};
			if (($info->{'nra'})) {
				$infoligne->{'nra'} = $info->{'nra'};
			}
			if (($info->{'LineLength'})) {
				$infoligne->{'length'} = $info->{'LineLength'};
			}
			if (($info->{'Sections'})) {
				$infoligne->{'sections'} = $info->{'Sections'};
			}
			if (($info->{'Attenuation'})) {
				$infoligne->{'attenuation'} = $info->{'Attenuation'};
			}
			if (($info->{'debit'})) {
				$infoligne->{'debit'} = $info->{'debit'};
			}
			if ($info->{'infos'} =~ /actuellement pas attribu/) {
				$retour->{erreur} = 1;
				$retour->{message} = "Le numro n'est actuellement pas attribu.\n";
			}
		}

		if ( ! defined $elig_offres->{$d} ||
		     ! $elig_offres->{$d}->{'faisable'} ) {
			debug "Debit $d pas faisable";
			conduse('FDN::Web::TemplateParse');
			$rejets .= remplace('/usr/share/fdn/templates/adsl/html/rejet-elig.tpl',$elig_offres->{$d},1);
			next;
		}

		$debits++;
		$debit =~ s/opt\d$//;
		$debit =~ s/Liazo.*//;
		$option =~ s/^\d+(k|M|G|RE)(K1|K2|B1|B2|)//;
		$option = $FDN::Adsl::Souscription::options->{$option};
		my $did = debit_id_par_debit($debit);
		debug "Debit: $debit, donne $did; option: $option; tarif: ".$tarif;
		my $prid = prix_id_par_debit_id_type_option($did,$tarif,$option);
		debug "Prix id: $prid";
		my $pr = prix_par_id($prid);
		use Data::Dumper;
		debug "Prix: ".Dumper($pr);
		my $fas;
		my $abo;
		if ( $vars->{tarif} ne 'entreprise' ) {
			$fas = $pr->{PRIX_FASTTC}.' TTC';
			$abo = $pr->{PRIX_ABOTTC}.' TTC';
		} else {
			$fas = $pr->{PRIX_FASHT}.' HT';
			$abo = $pr->{PRIX_ABOHT}.' HT';
		}
		$fas =~ s/(\d{2}) /.$1 E /;
		$abo =~ s/(\d{2}) /.$1 E /;

		$info_offre = {};
		$info_offre->{debit} = $debit;
		$info_offre->{code_offre} = $d;
		$info_offre->{fas} = $fas;
		$info_offre->{abo} = $abo;
		$info_offre->{reserve} = $elig_offres->{$d}->{'reserve'};

		push (@offres_dispo, $info_offre);
	}
	$retour->{offres} = \@offres_dispo;
	$retour->{rejets} = $rejets;
	$retour->{info_ligne} = $infoligne;
	if ( $retour->{erreur} == 0) {
		if ( ! $debits ) {
			$retour->{message} = "Il semble que la ligne ne soit pas ligible  un raccordement xDSL ou FTTH.\n";
		}
	}

	}

	my $j = JSON::Any->new;
	my $json = $j->objToJson($retour);

	eject_page_text($json);
}

sub erreur() {
	my $page = "Cette adresse est destine  tre accde par un programme.";
	eject_page($page);
}

