#!/usr/bin/perl

use strict;
use CGI;
use FDN::Web::Forms;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Adsl::Basic;
use FDN::Adsl::Utils;
use FDN::Adsl::Souscription;

use Data::Dumper;
use List::MoreUtils qw(any);

my $cgi = new CGI;
my $vars = $cgi->Vars;

my $cfg = 'adsl/subs/souscription.cfg';
my $tbl = 'adsl/subs/souscription.tbl';

if ( !defined($vars->{'etape'}) ) {
	choixtarif();
} elsif ( $vars->{'etape'} eq 'ndinconnu' ) {
	recherche_nd();
} elsif ( $vars->{'etape'} eq 'recherchend' ) {
	choix_nd();
} elsif ( $vars->{'etape'} eq 'choixpto' ) {
	choix_pto();
} elsif ( $vars->{'etape'} eq 'choixnd' ) {
	choixtarif();
} elsif ( $vars->{'etape'} eq 'choixstructure' ) {
	offre();
} elsif ( $vars->{'etape'} eq 'choixtarif' ) {
	offre();
} elsif ( $vars->{'etape'} eq 'choixtypend' ) {
	offre();
} elsif ( $vars->{'etape'} eq 'offre' ) {
	identite();
} elsif ( $vars->{'etape'} eq 'identite' ) {
	commande();
} elsif ( $vars->{'etape'} eq 'recap' ) {
	valide_commande();
} else {
	choixtarif();
}

sub choixtarif() {
	my $form = form_read ( $cfg, 'choixtarif' );
	my $head = "";
	if ((not defined $vars->{numadh}) or ($vars->{numadh} eq '')) {
		$head = 
"<p>Attention, si vous tes dj adhrente, ne passez par cette page que pour tester
l'ligibilit, ne validez pas de commande. Si vous souhaitez commander une
ligne xDSL, veuillez passer par <a href='https://vador.fdn.fr/'>votre espace adhrente</a>.</p>";
	} else {
		$head = "<p>Vous vous apprtez  passer commande pour l'adhrente ".$vars->{numadh}.".</p>";
	}
	$form->{byname}->{infocherchend}->{content} = $head.
		$form->{byname}->{infocherchend}->{content};
	if (defined $vars->{teltype} && $vars->{teltype} =~ /^((|VOISIN|MIGRATION|FTTH)0[1-5]\d{8}|42C[A-Z][A-Z][0-9]{15})-(actif|inactif|precab)$/) {
		$vars->{tel} = $1;
		$vars->{typend} = $3;
	}
	my $page = affiche_form($form,$vars);
	$page =~ s/NUMADH/$vars->{numadh}/;
	eject_page($page);
}

sub recherche_nd() {
	my $form = form_read ( $cfg, 'recherchend' );
	my $page = affiche_form($form,$vars);
	$page .= "<link href=\"typeaheadjs.css\" rel=stylesheet />\n";
	$page .= "<script src=\"jquery.min.js\"></script>\n";
	$page .= "<script src=\"typeahead.bundle.min.js\"></script>\n";
	$page .= "<script src=\"typeahead-link.js\"></script>\n";
	eject_page($page);
}

sub kill_accents($) {
	my ( $str ) = @_;
	$str =~ s/[]/A/g;
	$str =~ s/[]/a/g;
	$str =~ s/[]/AE/g;
	$str =~ s/[]/ae/g;
	$str =~ s/[]/C/g;
	$str =~ s/[]/c/g;
	$str =~ s/[]/E/g;
	$str =~ s/[]/e/g;
	$str =~ s/[]/I/g;
	$str =~ s/[]/i/g;
	$str =~ s/[]/N/g;
	$str =~ s/[]/n/g;
	$str =~ s/[]/O/g;
	$str =~ s/[]/o/g;
	$str =~ s/[]/U/g;
	$str =~ s/[]/u/g;
	$str =~ s/[]/Y/g;
	$str =~ s/[]/y/g;
	$str =~ s/[]/ss/g;
	return $str;
}

sub choix_nd() {
	my $form = form_read ( $cfg, 'recherchend' );
	if ( !check_transition($form,$vars) ) {
		return recherche_nd();
	}
	conduse('FDN::Adsl::SelL');
	$vars->{villedflt} = kill_accents($vars->{villedflt});
	$vars->{rue} = kill_accents($vars->{rue});
	my $liste = FDN::Adsl::SelL::get_nd($vars->{villedflt},$vars->{cpdflt},$vars->{rue},$vars->{numero});
	my $gps;
	my $fromgps;
	if (exists $vars->{gps} and $vars->{gps} =~ /^ *(-?[0-9]{0,2}.[0-9]+), *(-?[0-9]{0,3}.[0-9]+) *$/) {
		$gps = { 'lat' => $1, 'lon' => $2 };
		$fromgps = 1;
	} else {
		$gps = FDN::Adsl::SelL::get_gps($vars->{numero},$vars->{rue},$vars->{cpdflt},$vars->{villedflt});
		$fromgps = 0;
	}
	# Compter le nombre de numros en liste rouge
	my $rouges = grep { my $flags = $_->{flags}; grep( /^LR$/, @$flags) } @$liste;
	# Et les enlever du rsultat
	@$liste = grep { my $flags = $_->{flags}; ! grep( /^LR$/, @$flags) } @$liste;
	if (!defined $liste and !defined $gps) {
		eject_page_error("<p>Rue $vars->{rue} ou ville $vars->{villedflt} non trouve.</p>");
		exit(0);
	}
	if (scalar @$liste == 0 and not defined $gps) {
		if ($rouges) {
			if ($rouges > 1) {
				eject_page_error("<p>Aucun numro public de ligne tlphonique trouv  cette adresse, mais $rouges numros trouvs sur liste rouge, vous pouvez demander plus de dtails <br/>suivi-adsl\@fdn.fr</p>");
			} else {
				eject_page_error("<p>Aucun numro public de ligne tlphonique trouv  cette adresse, mais un numro trouv sur liste rouge, vous pouvez demander plus de dtails <br/>suivi-adsl\@fdn.fr</p>");
			}
		} else {
			eject_page_error("<p>Aucun numro de ligne tlphonique trouv  cette adresse.</p>");
		}
		exit(0);
	}
	my $page = '';
	if (scalar @$liste == 0) {
		$page .= "<p style='text-align:center'>Pas de numro de ligne tlphonique trouv  cette adresse</p>\n";
	} else {
		my $form = form_read ( $cfg, 'choixnd' );
		my $blk = $form->{'blocksbyname'}->{'choixnd'};
		for my $choix ( @$liste ) {
			my $nd = FDN::Adsl::SelL::get_nd_nd($choix);
			my $typend = FDN::Adsl::SelL::get_nd_typend($choix);
			my $label = FDN::Adsl::SelL::affiche_get_nd($choix, 0);

			my $fld = {
				'field_name' => $nd.'-'.$typend,
				'input_type' => 'radio',
				'type_radio' => 1,
				'mandatory' => 1,
				'mandatory_error' => 'Il faut faire un choix',
				'keep' => 0,
				'input_name' => 'teltype',
				'input_id' => 'teltype_'.$nd.'_'.$typend,
				'input_value' => $nd.'-'.$typend,
				'input_label' => $label,
			};

			FDN::Web::Forms::form_add_field($blk,$fld);
		}
		if ($rouges) {
			my $fld = {
				'field_name' => 'rouges',
				'input_type' => 'html',
				'type_html' => 1,
				'input_name' => 'rouges',
				'input_id' => 'rouges',
			};
			if ($rouges > 1) {
				$fld->{'content'} = "Note: $rouges numros en de ligne tlphonique liste rouge ont t filtrs, vous pouvez demander plus de dtails <br/>suivi-adsl\@fdn.fr",
			} else {
				$fld->{'content'} = "Note: 1 numro en liste rouge a t filtr, vous pouvez demander plus de dtails <br/>suivi-adsl\@fdn.fr",
			}
			FDN::Web::Forms::form_add_field($blk,$fld);
		}
		$page .= affiche_form($form,$vars);
	}

	if (defined $gps)
	{
		my ($x, $y) = FDN::Adsl::SelL::gps_to_web($gps->{lat}, $gps->{lon});
		my $immeubles = FDN::Adsl::SelL::get_immeubles_geom($x-200, $y-150, $x+200, $y+150);
		my $liste = $immeubles->{data}->{features};
		# FIXME: quand l'API Liazo sera disponible, demander l'adresse des immeubles affichs seulement
		my $tous_immeubles = FDN::Adsl::SelL::get_immeubles_liste($vars->{cpdflt},$vars->{villedflt});
		if (not defined $liste or scalar @$liste == 0) {
			if ($fromgps) {
				$page .= "<p style='text-align:center'>Pas de btiment rfrenc IPE trouv  ces coordonnes GPS</p>\n";
			} else {
				$page .= "<p style='text-align:center'>Pas de btiment rfrenc IPE trouv  cette adresse</p>\n";
			}
		} else {
			$page .= '<link rel="stylesheet" href="leaflet.css" />';
			$page .= "<center><p style='text-align:center'>Voici les btiments potentiellement ligibles FTTH:</p>\n";
			$page .= '<div id="map" style="width: 800px; height: 600px"></div>';
			$page .= '<script src="leaflet.js"></script>';
			$page .= '<script>';
			$page .= "var map = L.map('map').setView([$gps->{lat}, $gps->{lon}], 18);";
			$page .= "L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 19 }).addTo(map);";
			$page .= "var bullet = L.icon({iconUrl: 'bullet.png', shadowUrl: 'bullet.png', ";
			$page .= "iconSize: [16,16], shadowSize: [16,16], iconAnchor: [8,8], shadowAnchor: [8,8], popupAnchor: [0,-10]});";
			for my $immeuble ( @$liste ) {
				my $ref = $immeuble->{properties}->{pk};
				my $coords = $immeuble->{geometry}->{coordinates};
				my $xx = @$coords[0];
				my $yy = @$coords[1];
				my ($lat, $lon) = FDN::Adsl::SelL::web_to_gps($xx, $yy);
				my $addr = "";
				for my $i (@$tous_immeubles) {
					if ($i->{reference} eq $ref) {
						$addr = "<br/>".$i->{street_nb}." ".$i->{street};
						last;
					}
				}
				$addr =~ s/'/&apos;/g;
				$page .= "L.marker([$lat, $lon]).addTo(map).bindPopup('";
				$page .= "<a href=souscription.cgi?etape=choixpto&refimmeuble=$ref&cpdflt=$vars->{cpdflt}>Choisir cet immeuble$addr<br/>$ref</a>'";
				$page .= ").openPopup();\n";
			}
			my $rue = $vars->{rue} =~ s/'/&apos;/gr;
			$page .= "L.marker([$gps->{lat}, $gps->{lon}],{icon:bullet}).addTo(map).bindPopup('$vars->{numero} $rue').openPopup();";
			$page .= '</script>';
			$page .= '</center>';
		}
	}

	eject_page($page);
}



sub form_add_webbit($$) {
	my ( $blk, $webbit ) = @_;
	my $fld = {
		'input_name' => 'webbit',
		'type_html' => 1,
		'nodiv' => 1,
		'content' => $webbit,
	};
	FDN::Web::Forms::form_add_field($blk, $fld);
}

sub choix_partie_pto($$$$$) {
	my ( $blk, $info, $name, $text, $fun ) = @_;
	my $liste = $info->{pto};

	if ( defined $liste ) {
		for my $pto ( @$liste ) {
			my $ptounderscore = $pto;
			$ptounderscore =~ s/-/_/g;
			my $fld = {
				'field_name' => $pto,
				'input_type' => 'radio',
				'type_radio' => 1,
				'mandatory' => 1,
				'mandatory_error' => 'Il faut faire un choix',
				'keep' => 0,
				'input_name' => 'tel',
				'input_id' => 'tel_'.$ptounderscore,
				'input_value' => $pto,
				'input_label' => "$pto",
			};
			FDN::Web::Forms::form_add_field($blk,$fld);
		}
	}

	my $liste = $info->{$name};
	if ( defined $liste ) {
		form_add_webbit($blk, '<ul>');
		for my $elem ( @$liste ) {
			form_add_webbit($blk, '<li>');

			my $id = $elem->{id};

			if ($id eq '_NA_') {
				$id = '';
			}

			form_add_webbit($blk, " $text $id");

			$fun->($blk, $elem);

			form_add_webbit($blk, '</li>');
		}
		form_add_webbit($blk, '</ul>');
		return;
	}
	$fun->($blk, $info);
}

sub choix_fin_pto($$) {
}

sub choix_door_pto($$) {
	my ( $blk, $info ) = @_;
	choix_partie_pto($blk, $info, 'door', 'porte', \&choix_fin_pto);
}

sub choix_floor_pto($$) {
	my ( $blk, $info ) = @_;
	choix_partie_pto($blk, $info, 'floor', 'tage', \&choix_door_pto);
}

sub choix_stair_pto($$) {
	my ( $blk, $info ) = @_;
	choix_partie_pto($blk, $info, 'stairs', 'escalier', \&choix_floor_pto);
}

sub choix_building_pto($$) {
	my ( $blk, $info ) = @_;
	choix_partie_pto($blk, $info, 'building', 'btiment', \&choix_stair_pto);
}

sub choix_ensemble_pto($$) {
	my ( $blk, $info ) = @_;
	choix_partie_pto($blk, $info, 'ensemble', 'ensemble', \&choix_building_pto);
}

sub choix_pto() {
	my $ref = $vars->{refimmeuble};
	my $form = form_read ( $cfg, 'choixpto' );
	conduse('FDN::Adsl::SelL');
	my $immeuble = FDN::Adsl::SelL::get_info_immeuble($ref);

	if (!defined $immeuble) {
		eject_page_error("Cette rfrence d'immeuble IPE $ref a l'air incorrecte. Vous pouvez vrifier sur <a href=https://cartefibre.arcep.fr/>Cartefibre</a> si c'est bien la bonne rfrence, ou <a href=https://vador.fdn.fr/souscription/souscription.cgi?numadh=&etape=ndinconnu>chercher la rfrence par adresse.</a>");
		return;
	}

	my $info_ftth = FDN::Adsl::SelL::get_ftth(undef,undef,$ref);

	if (!defined $info_ftth || !defined $info_ftth->[0]) {
		eject_page_error("Cette rfrence d'immeuble IPE $ref n'a pas l'air connue chez Ielo. Cela signifie probablement qu'elle est inligible via Ielo et Kosc ou Bouygues :/");
		return;
	}

	my $ok = 0;
	for my $prod ( @$info_ftth ) {
		if ($prod->{product}->{code} =~ /^kosc.ftth./ or
		    $prod->{product}->{code} =~ /^bouygues.ftth./) {
			$ok = 1;
			last;
		}
	}

	if ($ok == 0) {
		eject_page_error("Btiment $ref connu, mais non ligible via Ielo et Kosc ou Bouygues :/");
		return;
	}

	my $blk = $form->{'blocksbyname'}->{'choixpto'};

	choix_ensemble_pto($blk, $immeuble);

	my $fld = {
		'field_name' => "NOUVEAU",
		'input_type' => 'radio',
		'type_radio' => 1,
		'mandatory' => 1,
		'mandatory_error' => 'Il faut faire un choix',
		'keep' => 0,
		'input_name' => 'tel',
		'input_id' => 'tel_nouveau',
		'input_value' => "NOUVEAU",
		'input_label' => "Livrer une nouvelle ligne",
	};
	FDN::Web::Forms::form_add_field($blk, $fld);

	my $form1 = affiche_form($form,$vars);


	eject_page($form1);
}

sub choix_partie_structure($$$$$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $name, $text, $fun, $choose ) = @_;
	# choose vaut 0 pour les niveaux o le choix est dj fait
	# choose vaut 1 pour les niveaux o le choix est en cours
	# choose vaut 2 pour les niveaux o le choix n'est pas encore activ
	my $choose_this = $choose;
	my $ret = 0;

	my $liste = $info->{$name};
	if ( defined $liste ) {
		my $choose_this = 0;

		my $multiple = scalar @$liste > 1;

		if ($multiple and (not defined $vars->{$name} or $vars->{$name} eq '')) {
			# Prvenir en amont qu'on a un choix  faire
			$ret = 1;
		}

		if ($choose <= 1 and (not defined $vars->{$name} or $vars->{$name} eq '')) {
			# On n'a pas encore choisi ce niveau-l
			$choose_this = 1;
			if ($multiple) {
				# On arrte les choix ici
				$choose = 2;
			} else {
				# On commence  faire des choix
				$choose = 1;
			}
		}

		form_add_webbit($blk, '<ul>');
		for my $elem ( @$liste ) {
			form_add_webbit($blk, '<li>');

			my $label2 = $label;
			my $id = $elem->{id};
			my $id_nona = $id;

			if ($id_nona eq '_NA_') {
				$id_nona = '';
			}

			my $idunderscore = $id;
			$idunderscore =~ s/-/_/g;

			if (($choose <= 1 or $choose_this) and $multiple) {
				# Il faut vraiment faire choisir
				my $fld = {
					'field_name' => $name.'-'.$id,
					'input_type' => 'radio',
					'type_radio' => 1,
					'mandatory' => 1,
					'mandatory_error' => 'Il faut faire un choix',
					'keep' => 0,
					'input_name' => $name,
					'input_id' => $name.'_'.$idunderscore,
					'input_value' => $id_nona,
					'input_label' => "$text $id",
				};
				if (!$multiple) {
					$vars->{$name} = $id_nona;
				}
				if ($vars->{$name} eq "$id_nona") {
					$fld->{checked} = 1;
				}
				if (defined $vars->{$name} and $vars->{$name} ne $id_nona) {
					$fld->{disabled} = 1;
				}
				FDN::Web::Forms::form_add_field($blk,$fld);
			} else {
				if ($choose <= 1) {
					# pas besoin de faire choisir, c'est dj choisi ou un seul choix
					my $fld = {
						'field_name' => $name.'-'.$id,
						'input_type' => 'hidden',
						'type_hidden' => 1,
						'keep' => 1,
						'input_name' => $name,
						'input_id' => $name.'_'.$idunderscore,
						'input_value' => $id_nona,
					};
					FDN::Web::Forms::form_add_field($hblk,$fld);
				}

				# On a dj prcdemment choisi ou c'est trivial de toutes faons, on affiche juste le texte
				form_add_webbit($blk, " $text $id_nona");
			}

			my $subchoose = $choose;

			if (defined $vars->{$name} and $vars->{$name} ne $id) {
				# On n'a pas choisi ce chemin-l, on arrte les choix de ce ct-l du coup
				$subchoose = 2;
			}
			my $subret = $fun->($form, $blk, $hblk, $elem, $label2, $subchoose);

			if ($ret and $subret) {
				# On voudra encore une autre page pour faire le sous-choix
				$ret = 2;
			}

			form_add_webbit($blk, '</li>');
		}
		form_add_webbit($blk, '</ul>');
		return $ret;
	}
	return $fun->($form, $blk, $hblk, $info, $label, $choose);
}

sub choix_fin_structure($$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $choose ) = @_;
	if ($choose == 1) {
		form_add_webbit($blk, '<ul><li>Ici</li></ul>');
	}
	return 0;
}

sub choix_door_structure($$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $choose ) = @_;
	return choix_partie_structure($form, $blk, $hblk, $info, $label, 'door', 'porte', \&choix_fin_structure, $choose);
}

sub choix_floor_structure($$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $choose ) = @_;
	return choix_partie_structure($form, $blk, $hblk, $info, $label, 'floor', 'tage', \&choix_door_structure, $choose);
}

sub choix_stair_structure($$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $choose ) = @_;
	return choix_partie_structure($form, $blk, $hblk, $info, $label, 'stairs', 'escalier', \&choix_floor_structure, $choose);
}

sub choix_building_structure($$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $choose ) = @_;
	return choix_partie_structure($form, $blk, $hblk, $info, $label, 'building', 'btiment', \&choix_stair_structure, $choose);
}

sub choix_ensemble_structure($$$$$$) {
	my ( $form, $blk, $hblk, $info, $label, $choose ) = @_;
	return choix_partie_structure($form, $blk, $hblk, $info, $label, 'ensemble', 'ensemble', \&choix_building_structure, $choose);
}

sub choix_structure() {
	my $ref = $vars->{refimmeuble};
	my $form = form_read ( $cfg, 'choixstructure' );
	conduse('FDN::Adsl::SelL');
	my $immeuble = FDN::Adsl::SelL::get_info_immeuble($ref);

	if (not defined $vars->{street} or $vars->{street} eq '' or
	    not defined $vars->{street_nb} or $vars->{street_nb} eq '' or
	    not defined $vars->{villedflt} or $vars->{villedflt} eq '')
	{
		my $addr = FDN::Adsl::SelL::get_addr_immeuble($vars->{cpdflt}, $ref);

		$vars->{street} = $addr->{street};
		$vars->{street_nb} = $addr->{street_nb};
		$vars->{villedflt} = $addr->{city};
	}

	my $blk = $form->{'blocksbyname'}->{'choixstructure'};
	my $hblk = $form->{'blocksbyname'}->{'hchoixstructure'};

	my $ret = choix_ensemble_structure($form, $blk, $hblk, $immeuble, '', 0);
	if ($ret <= 1) {
		# Un seul choix, on peut terminer
		my $fld = $form->{'byname'}->{'hchoixstructure'};
		$fld->{input_value} = 'choixstructure';
	}

	eject_page(affiche_form($form, $vars));
	exit(0);
}

sub fake_form_offre() {
	my $form = form_read ( $cfg, 'offre' );
	ajoute_champ_config($form,$cfg,'hoffre','hdebit');
	return $form;
}

sub make_form_offre($) {
	my ( $rej ) = @_;
	conduse('FDN::Adsl::Basic');
	conduse('FDN::Adsl::Souscription');
	conduse('FDN::Adsl::SelL');
	my $tarifs = $FDN::Adsl::Souscription::tarifs;
	my @offres = @$FDN::Adsl::Souscription::offres;
	my $form = form_read ( $cfg, 'offre' );
	my $rejets;
	my $tel = $vars->{tel};

	if ($tel =~ /^VOISIN/) {
		if ($vars->{typend} ne "actif") {
			eject_page_error("Ce numro $tel est un numro inactif, il ne permet pas de passer une commande par voisin. Il vous faut trouver un voisin qui a un numro actif.");
			return;
		}
	}

	if ($tel =~ /^FTTH/) {
		@offres = ("1GK1LiazoKosc", "300MK1LiazoKosc", "1GK2LiazoKosc", "300MK2LiazoKosc", "1GB1LiazoBouygues", "1GB2LiazoBouygues");
	}

	$tel =~ s/^MIGRATION//;
	$tel =~ s/^FTTH//;

	if ($tel =~ /^09\d{8}$/) {
		eject_page_error("Attention, $tel n'est pas un numro de ligne tlphonique mais seulement un numro de tlphone branch sur un abonnement ADSL/VDSL. <a href=https://vador.fdn.fr/souscription/souscription.cgi?numadh=&etape=ndinconnu>Si vous ne connaissez pas votre numro de ligne tlphonique ou de PTO, cliquez ici pour chercher par adresse.</a>");
		return;
	}

	if ($vars->{refimmeuble}) {
		my $info = FDN::Adsl::SelL::get_info_immeuble($vars->{refimmeuble});
		if (! defined $info) {
			eject_page_error("Cette rfrence d'immeuble IPE $vars->{refimmeuble} a l'air incorrecte. Vous pouvez vrifier sur <a href=https://cartefibre.arcep.fr/>Cartefibre</a> si c'est bien la bonne rfrence, ou <a href=https://vador.fdn.fr/souscription/souscription.cgi?numadh=&etape=ndinconnu>chercher la rfrence par adresse.</a>");
			return;
		}
		my $pto = $tel;
		$pto =~ s/^VOISIN//;
		if ($pto !~ /^0[1-5]\d{8}$/ and $pto ne 'NOUVEAU') {
			# pas un numro de tlphone, on vrifie si on le trouve dans l'immeuble
			my $list = FDN::Adsl::SelL::get_list_immeuble_pto($info);
			if (not any { $_ eq $pto } @$list) {
				my $pto0 = $pto =~ s/O/0/r;
				if (not any { $_ eq $pto0 } @$list) {
					my $l = "";
					for my $p (@$list) {
						$l .= "<li>$p</li>";
					}
					eject_page_error("Ce numro de PTO $pto n'apparat pas dans l'immeuble de rfrence IPE $vars->{refimmeuble}, voici la liste des numros de PTO de cet immeuble: <ul>$l</ul>");
					return;
				} else {
					$tel = $pto0;
					$vars->{tel} =~ s/O/0/;
				};
			}
		}
	}

	my $elig_offres = {};
	if ( $vars->{typend} eq "actif" && Config_Key('adsl/global.cfg', 'fdn', 'use_nerim') ) {
		conduse('FDN::Adsl::SelN');
		my $elig = FDN::Adsl::SelN::eligibilite($tel,$vars->{cpdflt},\@offres);
		if ( $elig->{etat} == 1 ) {
			my $page = "<p>Une erreur technique sur le serveur d'ligibilit de Nerim ".
				   "nous met dans ".
				   "l'impossibilit de tester l'ligilit de votre ligne chez Nerim. Merci de ".
				   "ressayer plus tard.</p>\n".
				   "<p>Le texte des messages d'erreur:</p>\n";
			$page .= "<p>".$elig->{'message'}."</p>\n";
			$rejets .= $page;
		}
		elsif ( $elig->{etat} == 2 ) {
			$rejets .= "<p>Une erreur technique chez FDN empche de poursuivre l'ligibilit chez Nerim</p>";
		}
		elsif ( $elig->{etat} == 3 ) {
			my $page = "<p>Il y a un problme avec le numro de tlphone que vous avez saisi.</p>\n";
			$page .= "<p>".$elig->{'message'}."</p>\n";
			$rejets .= $page;
		}
		elsif ( $elig->{etat} ) {
			my $page = "<p>Une erreur technique GRAVE entre FDN et Nerim empche de traiter votre demande. Merci de ".
				   "transmettre l'information a FDN.</p>";
			$rejets .= $page;
		}
		else {
			$elig_offres = { %$elig_offres, %{$elig->{'offres'}} };
		}
	}

	my $whitelist = Config_Key('adsl/elig/liazo.cfg', 'ariane', '@whitelist');
	if ( Config_Key('adsl/global.cfg', 'fdn', 'use_liazo') || grep { $_ eq $vars->{tel} } @$whitelist)  {
		conduse('FDN::Adsl::SelL');
		my $elig2 = FDN::Adsl::SelL::eligibilite_liazo($tel,$vars->{typend},$vars->{refimmeuble},$vars->{cpdflt},\@offres);
		if ( $elig2->{etat} == 1 ) {
			my $page = "<p>Une erreur technique sur le serveur d'ligibilit de Liazo ".
				   "nous met dans ".
				   "l'impossibilit de tester l'ligilit de votre ligne chez Liazo. Merci de ".
				   "ressayer plus tard.</p>\n".
				   "<p>Le texte des messages d'erreur:</p>\n";
			$page .= "<p>".$elig2->{'message'}."</p>\n";
			$rejets .= $page;
		}
		if ( $elig2->{etat} == 2 ) {
			$rejets .= "<p>Une erreur technique chez FDN empche de poursuivre l'ligibilit chez Liazo</p>";
		}
		if ( $elig2->{etat} == 3 ) {
			my $page = "<p>Il y a un problme avec le numro de tlphone que vous avez saisi.</p>\n";
			$page .= "<p>".$elig2->{'message'}."</p>\n";
			$rejets .= $page;
		}
		if ( $elig2->{etat} ) {
			my $page = "<p>Une erreur technique GRAVE chez FDN empche de traiter votre demande. Merci de ".
				   "transmettre l'information a FDN.</p>";
			$rejets .= $page;
		}
		$elig_offres = { %$elig_offres, %{$elig2->{'offres'}} };
	}

	$tel =~ s/^VOISIN//;

	my $debits = 0;
	my $infoligne;
	my $meilleure_offre = {}; # Donne la meilleure offre pour une option et tarif donns
	my @meilleures_offres = (); # liste des option+tarif, dans l'ordre
	my @toutes_offres = ();
	my $index_affaiblissement = 0;

	if ( ! Config_Key('adsl/global.cfg', 'fdn', 'use_liazo') && $vars->{typend} ne "actif" ) {
		$rejets .= "<p>Ce numro n'est pas actif (vous n'avez pas d'abonnement tlphonique Orange dessus), et la collecte Liazo n'est pas encore active chez FDN.</p>";
	}

	for my $d ( @offres ) {
		debug "je regarde l'offre $d";
		my $option = $d;
		my $debit = $d;
		if ( ! defined $elig_offres->{$d} ||
		     ! $elig_offres->{$d}->{'faisable'} ) {
			debug "Debit $d pas faisable";
			conduse('FDN::Web::TemplateParse');
			$rejets .= remplace('/usr/share/fdn/templates/adsl/html/rejet-elig.tpl',$elig_offres->{$d},1);
			next;
		}

		if ( ($d eq '512kopt5') or ($d eq '512REopt5' or ($d eq '512kLiazoOrange') or ($d eq '512kLiazoOrangeNu')) ) {
			my $info = $elig_offres->{$d};
			if (($info->{'nra'})) {
				$infoligne->{'nra'} = $info->{'nra'};
			}
			if (($info->{'LineLength'})) {
				$infoligne->{'length'} = $info->{'LineLength'};
			}
			if (($info->{'Sections'})) {
				$infoligne->{'sections'} = $info->{'Sections'};
			}
			if (($info->{'Attenuation'})) {
				$infoligne->{'attenuation'} = $info->{'Attenuation'};
			}
			if (($info->{'debit'})) {
				$infoligne->{'debit'} = $info->{'debit'};
			}
		}
		push @toutes_offres, $d;
		$debit =~ s/opt\d$//;
		$debit =~ s/Liazo.*//;
		$option =~ s/^\d+(k|M|G|RE)(K1|K2|B1|B2|)//;
		$option = $FDN::Adsl::Souscription::options->{$option};
		my $did = debit_id_par_debit($debit);
		my $prid = prix_id_par_debit_id_type_option($did,$vars->{tarif},$option);
		my $pr = prix_par_id($prid);
		my $fas;
		my $abo;
		if ( $vars->{tarif} ne 'entreprise' ) {
			$fas = $pr->{PRIX_FASTTC};
			$abo = $pr->{PRIX_ABOTTC};
		} else {
			$fas = $pr->{PRIX_FASHT};
			$abo = $pr->{PRIX_ABOHT};
		}

		# Ne garder que la dernire offre (la plus rapide) pour une option et un tarif donn
		my $index = $option." ".$fas." ".$abo;
		if ($debit eq "80M") {
			if (!defined $elig_offres->{$d}->{'reserve'}) {
				$elig_offres->{$d}->{'reserve'} = "";
			}
			$elig_offres->{$d}->{'reserve'} .= ' Ce choix ncessite un modem compatible <font color="red">VDSL</font>.';
			$index .= $debit;
		}
		if (($debit eq "1GK1") or ($debit eq "1GK2")
		 or ($debit eq "300MK1") or ($debit eq "300MK2")
		 or ($debit eq "1GB1") or ($debit eq "1GB2")) {
			if (!defined $elig_offres->{$d}->{'reserve'}) {
				$elig_offres->{$d}->{'reserve'} = "";
			}
			$elig_offres->{$d}->{'reserve'} .= ' Priode d\'engagement d\'<font color="red">un an</font>.';
			$index .= $debit;
		}

		# Montrer les diffrentes offres avec diffrentes rserves aussi
		my $reserve = $elig_offres->{$d}->{'reserve'};
		if (defined($reserve)) {
			if ( $reserve =~ "affaiblissement" ) {
				# C'est une rserve d'affaiblissement, montrer tous les
				# dbits intermdiaires, au cas o
				$reserve .= $index_affaiblissement;
				$index_affaiblissement++;
			}
			$index .= $reserve;
		}

		if ( ! exists $meilleure_offre->{$index} ) {
			push @meilleures_offres, $index;
		}
		if (!defined $meilleure_offre->{$index}) {
			debug "offre pour $index est $d";
			$meilleure_offre->{$index} = $d;
		}

		$elig_offres->{$d}->{'prix_fas'} = $fas;
		$elig_offres->{$d}->{'prix_abo'} = $abo;
		$elig_offres->{$d}->{'option'} = $option;
	}

	my @liste = ();

	if ($vars->{toutesoffres} == 1) {
		# Montrer toutes les offres
		@liste = @toutes_offres;
	} else {
		# Ne montrer que les meilleures offres
		for my $index ( @meilleures_offres ) {
			push @liste, $meilleure_offre->{$index};
		}
	}

	my $last_option;
	for my $d ( @liste ) {
		debug "je montre l'offre $d";
		my $option = $elig_offres->{$d}->{'option'};
		if ((!defined($last_option)) || $last_option ne $option) {
			ajoute_champ_config($form,$cfg,'toutes_offres',$option);
			$last_option = $option;
		}
		ajoute_champ_config($form,$cfg,'toutes_offres',$d);
		$debits++;
		my $fas = $elig_offres->{$d}->{'prix_fas'};
		my $abo = $elig_offres->{$d}->{'prix_abo'};
		if ( $vars->{tarif} ne 'entreprise' ) {
			$fas = $fas.' TTC';
			$abo = $abo.' TTC';
		} else {
			$fas = $fas.' HT';
			$abo = $abo.' HT';
		}
		$fas =~ s/(\d{2}) /.$1&euro; /;
		$abo =~ s/(\d{2}) /.$1&euro; /;
		$form->{byname}->{$d}->{'input_label'}.=" $fas d'installation puis $abo par mois";
		$form->{byname}->{$d}->{'input_type'} = "radio-large";

		my $reserve = $elig_offres->{$d}->{'reserve'};
		if (defined($reserve)) {
			$form->{byname}->{$d}->{'input_label'}.= " Attention: $reserve";
		}
	}

	my $info_nd = get_info_addr_nd_pto($vars->{cpdflt}, $vars->{refimmeuble}, $tel, $vars->{typend});

	if (defined $vars->{street} and $vars->{street} ne '') {
		$info_nd->{info}{street} = $vars->{street};
	}
	if (defined $vars->{street_nb} and $vars->{street_nb} ne '') {
		$info_nd->{info}{street_nb} = $vars->{street_nb};
	}
	if (defined $vars->{ensemble} and $vars->{ensemble} ne '') {
		$info_nd->{info}{ensemble} = $vars->{ensemble};
	}
	if (defined $vars->{building} and $vars->{building} ne '') {
		$info_nd->{info}{building} = $vars->{building};
	}
	if (defined $vars->{stairs} and $vars->{stairs} ne '') {
		$info_nd->{info}{stairs} = $vars->{stairs};
	}
	if (defined $vars->{floor} and $vars->{floor} ne '') {
		$info_nd->{info}{floor} = $vars->{floor};
	}
	if (defined $vars->{door} and $vars->{door} ne '') {
		$info_nd->{info}{door} = $vars->{door};
	}
	if (defined $vars->{logo} and $vars->{logo} ne '') {
		$info_nd->{info}{logo} = $vars->{logo};
	}
	if (defined $vars->{villedflt} and $vars->{villedflt} ne '') {
		$info_nd->{info}{city} = $vars->{villedflt};
	}

	$vars->{adressedflt} = $info_nd->{info}{street_nb}.' '.$info_nd->{info}{street};
	if (defined $info_nd->{info}{ensemble} and $info_nd->{info}{ensemble} ne '') {
		$vars->{adressedflt} .= " ensemble $info_nd->{info}{ensemble}";
		$vars->{ensemble} = $info_nd->{info}{ensemble};
	}
	if (defined $info_nd->{info}{building} and $info_nd->{info}{building} ne '') {
		$vars->{adressedflt} .= " btiment $info_nd->{info}{building}";
		$vars->{building} = $info_nd->{info}{building};
	}
	if (defined $info_nd->{info}{stairs} and $info_nd->{info}{stairs} ne '') {
		$vars->{adressedflt} .= " escalier $info_nd->{info}{stairs}";
		$vars->{stairs} = $info_nd->{info}{stairs};
	}
	if (defined $info_nd->{info}{floor} and $info_nd->{info}{floor} ne '') {
		$vars->{adressedflt} .= " tage $info_nd->{info}{floor}";
		$vars->{floor} = $info_nd->{info}{floor};
	}
	if (defined $info_nd->{info}{door} and $info_nd->{info}{door} ne '') {
		$vars->{adressedflt} .= " porte $info_nd->{info}{door}";
		$vars->{door} = $info_nd->{info}{door};
	}
	if (defined $info_nd->{info}{logo} and $info_nd->{info}{logo} ne '') {
		$vars->{adressedflt} .= " logo $info_nd->{info}{logo}";
		$vars->{logo} = $info_nd->{info}{logo};
	}
	$info_nd->{info}{post_code} = $vars->{cpdflt};
	$vars->{villedflt} = $info_nd->{info}{city};

	$info_nd->{info}{name} = "";

	my $addr = get_addr_from_info_addr_nd($info_nd);

	$form->{byname}->{'techligne'}->{'content'} .= "<br> tlphone ou PTO : ". $vars->{'tel'};
	if ($tel =~ /^0[1-5]\d{8}/) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> type : ". $vars->{'typend'};
	}
	if ($vars->{'refimmeuble'}) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> rfrence immeuble : ". $vars->{'refimmeuble'};
	}
	$form->{byname}->{'techligne'}->{'content'} .= "<br> adresse : <pre>". $addr."</pre>";
	if (($infoligne->{'nra'})) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> NRA xDSL : ". $infoligne->{'nra'};
	}
	if (($infoligne->{'length'})) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> longueur xDSL : ". $infoligne->{'length'}. "m";
	}
	if (($infoligne->{'sections'})) {
		my $sections = $infoligne->{'sections'};
		my $txt = "";
		for my $s ( @$sections ) {
			if ($txt ne "") {
				$txt .= ", ";
			}
			$txt .= @$s[0]."m  0.".@$s[1]."mm";

		}
		$form->{byname}->{'techligne'}->{'content'} .= "<br> sections : $txt";
	}
	if (($infoligne->{'attenuation'})) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> attnuation thorique xDSL : ". $infoligne->{'attenuation'}. "dB";
	}
	if (($infoligne->{'debit'})) {
		$form->{byname}->{'techligne'}->{'content'} .= "<br> dbit IP thorique xDSL : ". $infoligne->{'debit'}. "Mbps";
	}
	if ( ! $debits ) {
		eject_page_error("<p>Il semble que vous ne soyiez pas ligible  un raccordement xDSL ou FTTH. Peut-tre votre ligne tlphonique est dgroupe totalement ? Il faut alors utiliser le numro de la ligne tlphonique support (donc pas le numro de tlphone sur xDSL, mais le numro de la ligne en cuivre), votre fournisseur d'accs xDSL actuel devrait tre en mesure de vous le fournir. Si vous ne parvenez pas  obtenir de numro, vous pouvez essayez avec le numro de ligne tlphonique d'un(e) voisin(e) qui n'a pas de dgroupage total en mettant VOISIN devant le numro (par exemple VOISIN0123456789). Voil l'ensemble des informations que nous avons obtenu  propos de votre ligne. Contactez-nous ventuellement pour que l'on regarde plus prcisment.</p>".
				 $rejets);
		exit(0);
	}
	if ( defined $rej ) {
		$$rej = $rejets;
	}
	return $form;
}

sub essaie_typend($$$$$) {
	my ( $tel, $inactif, $precab, $typend, $form ) = @_;

	if ( $tel !~ /0[1-5]\d{8}|42C[A-Z][A-Z][0-9]{15}/ && $tel !~ /^VOISIN/ && $tel !~ /^MIGRATION/ ) {
		if ( not defined $vars->{refimmeuble} or $vars->{refimmeuble} eq '') {
			eject_page_error("<p>Pour la FTTH il faut renseigner le champ d'identifiant immeuble IPE. Vous pouvez le trouver sur <a href=https://cartefibre.arcep.fr/>Cartefibre</a>, ou <a href=https://vador.fdn.fr/souscription/souscription.cgi?numadh=&etape=ndinconnu>chercher la rfrence par adresse.</a></p>");
			exit(0);
		}
	}

	conduse('FDN::Adsl::SelL');
	$tel =~ s/^VOISIN//;
	$tel =~ s/^FTTH//;
	$tel =~ s/^MIGRATION//;
	if ($tel !~ /0[1-5]\d{8}|42C[A-Z][A-Z][0-9]{15}/ ) {
		return $inactif eq "true";
	}
	my $info = FDN::Adsl::SelL::get_info_nd($tel, $inactif, $precab);
	if (! defined $info || defined $info->{'error'}{'msg'}) {
		return 0;
	}
	debug "Le numro $tel est $typend";
	ajoute_champ_config($form,$cfg,'tous_typend',$typend);
	$form->{byname}->{$typend}->{adresse} = 
		 $info->{'info'}{'name'}."<br/>"
		.$info->{'info'}{'street_nb'}." "
		.$info->{'info'}{'street'}."<br/>"
		.$info->{'info'}{'city'};
	$form->{byname}->{$typend}->{input_label} .= " et votre adresse est:<br/>"
		.$form->{byname}->{$typend}->{adresse};
	if ($inactif eq "true") {
		$form->{byname}->{$typend}->{input_label} .= "<br/>(si l'adresse est correcte, prfrez ce choix pour obtenir l'ligibilit FTTH)";
	}
	return 1;
}

sub offre() {
	$vars->{tel} =~ s/ //g;
	$vars->{refimmeuble} =~ s/ //g;

	my $tel = $vars->{'tel'};
	if ( $tel !~ /0[1-5]\d{8}|42C[A-Z][A-Z][0-9]{15}/ && $tel !~ /^VOISIN/ && $tel !~ /^MIGRATION/ ) {
		if ( not defined $vars->{refimmeuble} or $vars->{refimmeuble} eq '') {
			eject_page_error("<p>Pour la FTTH il faut renseigner le champ d'identifiant immeuble IPE. Vous pouvez le trouver sur <a href=https://cartefibre.arcep.fr/>Cartefibre</a>, ou <a href=https://vador.fdn.fr/souscription/souscription.cgi?numadh=&etape=ndinconnu>chercher la rfrence par adresse.</a></p>");
			exit(0);
		}
	}
	if ( $vars->{'tel'} eq 'NOUVEAU' and $vars->{'etape'} ne 'choixstructure') {
		choix_structure();
	}
	if ( $vars->{'etape'} eq 'choixtarif' ) {
		my $form = form_read ( $cfg, 'choixtarif' );
		if ( !check_transition($form,$vars) ) {
			return choixtarif();
		}
	}
	if (! $vars->{typend}) {
		my $whitelist = Config_Key('adsl/elig/liazo.cfg', 'ariane', '@whitelist');
		if ( Config_Key('adsl/global.cfg', 'fdn', 'use_liazo') || grep { $_ eq $vars->{tel} } @$whitelist) {
			my $form = form_read ( $cfg, 'choixtypend' );
			my $n = 0;
			my $typend;
			my $actif_ok;
			my $inactif_ok;
			if (essaie_typend($vars->{tel}, "false", "false", "actif", $form)) {
				$typend = "actif";
				$actif_ok = 1;
				$n++;
			}
			if (essaie_typend($vars->{tel}, "true", "false", "inactif", $form)) {
				$typend = "inactif";
				$inactif_ok = 1;
				$n++;
			}
			if (essaie_typend($vars->{tel}, "false", "true", "precab", $form)) {
				$typend = "precab";
				$n++;
			}
			if ($n == 0) {
				eject_page_error("Ce numro $vars->{tel} n'apparat pas dans les bases de lignes tlphoniques. C'est peut-tre un numro de tlphone via ADSL ? Cela ne nous permettra pas de passer commande, il nous faut le numro de la ligne tlphonique sous-jacente (appel ND, Numro de Dsignation). Si vous ne le connaissez pas, vous pouvez <a href='https://vador.fdn.fr/souscription/souscription.cgi?etape=ndinconnu'>passer par votre adresse</a>.");
				return;
			}
			if ($vars->{tel} =~ /^VOISIN/) {
				# On ne peut commander par voisin que sur numro actif
				$typend = "actif";
				if ($inactif_ok and $actif_ok) {
					if ($form->{byname}->{actif}->{adresse} eq 
					    $form->{byname}->{inactif}->{adresse}) {
						# C'est en fait la mme adresse. On peut prendre l'actif sans se poser de question
						$n = 1;
					} else {
						# Cacher le choix inactif
						$form->{byname}->{inactif}->{disabled} = 'true';
						$form->{byname}->{inactif}->{input_label} .= "<br/>Mais ce numro est un numro inactif, il ne permet pas de passer une commande par voisin. Il vous faut trouver un voisin qui a un numro actif.</p>";
						# Et faire vrifier le choix actif
						$form->{byname}->{infotypend}->{content} = "<p>Le numro de tlphone que vous avez fourni correspond  plusieurs logements, probablement suite  un dmnagement avec portabilit de numro. Il faut donc vrifier que l'adresse ci-dessous est bien la bonne, sinon la livraison sera faite au mauvais endroit, et il faudra recommencer avec  la fois des frais d'crasement et de reconstruction.</p>";
					}
				}
			}
			if ($n > 1) {
				debug "oops, c'est ambigu donc... Il faut demander  l'abonn ce qu'il en est vraiment";
				# On suppose a priori le moins actif, c'est en gnral le plus courant dsormais
				$vars->{typend} = $typend;
				eject_page(affiche_form($form,$vars));
				return;
			}
			$vars->{typend} = $typend;
		} else {
			# Nerim ne sait livre que sur des numros actif
			$vars->{typend} = 'actif';
		}
	}
	my $rejets;
	my $form = make_form_offre(\$rejets);
	my $page = affiche_form($form,$vars);
	$page .= "<div class=\"rejets\"><p>Il y a d'autres dbits, non affich ci-dessus, pour lesquels vous n'tes pas ligibles. Pour votre information, voil les raisons qui nous ont t donnes. Si certaines vous semblent fausses, n'hsitez pas  nous contacter.</p>$rejets</div>";
	eject_page($page);
}

sub identite() {
	my $form = fake_form_offre();
	if (( $vars->{debit} =~ /LiazoKosc$/ or $vars->{debit} =~ /LiazoBouygues$/ ) and $vars->{tel} =~ /^0[1-5]\d{8}$/) {
                # Bricole pour l'instant pour avoir un numro relativement
		# unique, tant qu'on n'obtient pas le numro de PTO livr
		$vars->{tel} = "FTTH".$vars->{tel};
	}
	if ( !check_transition($form,$vars) ) {
		return offre();
	}
	$form = form_read ( $cfg, 'identite' );

	if ($vars->{tel} !~ /^VOISIN/ && $vars->{villedflt}) {
		$form->{byname}->{'adresse'}->{'readonly'} = "true";
		$form->{byname}->{'ensemble'}->{'readonly'} = "true";
		$form->{byname}->{'building'}->{'readonly'} = "true";
		$form->{byname}->{'stairs'}->{'readonly'} = "true";
		$form->{byname}->{'floor'}->{'readonly'} = "true";
		$form->{byname}->{'door'}->{'readonly'} = "true";
		$form->{byname}->{'logo'}->{'readonly'} = "true";
		$form->{byname}->{'cp'}->{'readonly'} = "true";
		$form->{byname}->{'ville'}->{'readonly'} = "true";
	}

	eject_page(affiche_form($form,$vars));
}

sub commande() {
	my $form = form_read ( $cfg, 'identite' );
	if ( !check_transition($form,$vars) ) {
		eject_page(affiche_form($form,$vars));
		return;
	}

	if ( $vars->{debit} =~ /^(1G|300M)/ and ! $vars->{telfac} and ! $vars->{teldflt} ) {
		eject_page_error("<p>Pour une livraisons FTTH, il est ncessaire d'avoir un numro de contact pour pouvoir tablir le rendez-vous de livraison.</p>");
		exit(0);
	}

	#if ( $vars->{banque} && $vars->{guichet} && $vars->{compte} && $vars->{rib} ) {
	#	my $ok = verif_clef_rib(
	#		$vars->{banque},
	#		$vars->{guichet},
	#		$vars->{compte},
	#		$vars->{rib});
	#	if ( ! $ok ) {
	#		add_form_error('syntax',
	#			"Il y a une erreur dans les coordonnees bancaires.");
	#		eject_page(affiche_form($form,$vars));
	#		return;
	#	}
	#}
	#if ( $vars->{'bic'} ) {
	$vars->{'iban'} =~ s/ //g;
	my $ok = verif_clef_iban($vars->{'iban'});
	if ( ! $ok ) {
		add_form_error('syntax',
			"Il y a une erreur dans les coordonnees bancaires.");
		eject_page(affiche_form($form,$vars));
		return;
	}
	#}
	if ($vars->{'numadh'}) {
		my $cl = client_par_numadh($vars->{'numadh'});
		if (defined $cl) {
			$vars->{'nomdflt'} = $cl->{'CLIENT_NOM'};
		}
	}

	conduse('FDN::Web::Array');
	$form = form_read ( $cfg, 'recap' );
	my $page = tableau_1d($tbl,'recap',$vars);
	$page .= affiche_form($form,$vars);
	eject_page($page);
}

sub valide_commande() {
	conduse('FDN::Adsl::Souscription');
	if ( ! $vars->{adressefac} ) {
		$vars->{adressefac} = $vars->{adressedflt};
	}
	if ( ! $vars->{cpfac} ) {
		$vars->{cpfac} = $vars->{cpdflt};
	}
	if ( ! $vars->{villefac} ) {
		$vars->{villefac} = $vars->{villedflt};
	}
	if ( ! $vars->{telfac} ) {
		$vars->{telfac} = $vars->{teldflt};
	}
	if ( ! $vars->{nomlig} ) {
		$vars->{nomlig} = $vars->{nomdflt};
	}
	if ( ! $vars->{tellig} ) {
		$vars->{tellig} = $vars->{teldflt};
	}

	my $dplnum = souscription_fdn($vars);
	if ( $dplnum ) {
		print $cgi->redirect('suivi-commande.cgi?dplid='.$dplnum.'&tel='.$vars->{tel}.'&do=yes');
	} else {
		my $page;
		$page = "<div class=\"error\"><p>Une erreur empeche de prendre vote commande\n".
			"en compte. Si cette erreur persiste, elle est probablement serieuse.\n".
			"Quoi qu'il en soit, les informations utiles ont ete transmises au\n".
			"bureau de l'association qui fera tout son possible pour que tout\n".
			"rentre dans l'ordre et vous re-contactera en cas de besoin.</p></div>";
		eject_page($page);
	}
}


