#!/usr/bin/perl

use strict;
use CGI;
use FDN::Web::Forms;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Adsl::Utils;

my $cgi = new CGI;
my $vars = $cgi->Vars;

my $cfg = 'adsl/subs/vpn.cfg';
my $tbl = 'adsl/subs/vpn.tbl';
my $form = form_read($cfg, 'vpn');
my $recap = form_read($cfg, 'recap');

if ( ! defined($vars->{'do'}) ) {
	info 'Affichage du formulaire vierge';
	show_form();
} elsif ( ! defined ($vars->{'validation'}) ) {
	info 'Affichage du recap de validation de la commande';
	show_recap();
} else {
	info 'Validation de la commande';
	valide_commande();
}


sub control_form() {
	if ( !check_transition($form,$vars) ) {
		info 'Formulaire invalide';
		show_form();
		exit(0);
	}
	$vars->{'iban'} =~ s/ //g;
	my $ok = verif_clef_iban($vars->{'iban'});
	return if $ok;

	info 'Clef RIB invalide';
	add_form_error('syntax', "Il y a une erreur dans les coordonnees bancaires.");
	show_form();
	exit(0);
}

sub show_form() {
	eject_page(affiche_form($form,$vars));
}

sub show_recap() {
	control_form();
	info 'Formulaire valide, affichage du recap de commande';
	conduse('FDN::Web::Array');
	my $page = tableau_1d($tbl,'recap',$vars);
	$page .= affiche_form($recap,$vars);
	eject_page($page);
}

sub valide_commande() {
	control_form();
	conduse('FDN::Adsl::Souscription');

	my ( $dplnum, $vpnid ) = souscription_vpn($vars);
	if ( $dplnum ) {
		print $cgi->redirect('suivi-commande.cgi?dplid='.$dplnum.'&tel=vpn'.$vpnid.'&do=yes');
	} else {
		my $page;
		$page = "<div class=\"error\"><p>Une erreur empeche de prendre vote commande\n".
			"en compte. Si cette erreur persiste, elle est probablement serieuse.\n".
			"Quoi qu'il en soit, les informations utiles ont ete transmises au\n".
			"bureau de l'association qui fera tout son possible pour que tout\n".
			"rentre dans l'ordre et vous re-contactera en cas de besoin.</p></div>";
		eject_page($page);
	}
}


